@@ -120,19 +120,19 @@ conn_init_await(BroConn *bc, int conn_state)
 
       if (bc->state->conn_state_peer >= conn_state)
 	D_RETURN_(TRUE);
-      
+
       if (select(0, NULL, NULL, NULL, &timeout) < 0)
 	{
 	  D(("select() caused '%s'\n", strerror(errno)));
 	  D_RETURN_(FALSE);
 	}
-      
-      __bro_io_process_input(bc);      
+
+      __bro_io_process_input(bc);
     }
-  
+
   D_RETURN_(FALSE);
 }
-  
+
 static int
 conn_init_setup(BroConn *bc)
 {
@@ -141,7 +141,7 @@ conn_init_setup(BroConn *bc)
   int result;
 
   D_ENTER;
-   
+
   descriptor[0] = htonl((uint32) BRO_PROTOCOL_VERSION);
   descriptor[1] = htonl((uint32) BRO_MAX_CACHE_SIZE);
   descriptor[2] = htonl((uint32) BRO_DATA_FORMAT_VERSION);
@@ -165,9 +165,9 @@ conn_init_setup(BroConn *bc)
   __bro_buf_cleanup(&buf);
 
   /* This phase does NOT send PHASE_END ... */
-  
+
   D(("Phase done to peer on %p, self now in HANDSHAKE stage.\n", bc));
-  result = conn_init_await(bc, BRO_CONNSTATE_HANDSHAKE);  
+  result = conn_init_await(bc, BRO_CONNSTATE_HANDSHAKE);
   D_RETURN_(result);
 }
 
@@ -190,8 +190,10 @@ conn_init_handshake(BroConn *bc)
   /* Unless user requested caching, tell peer we do not cache. Note
    * that at the moment we never cache data we send, so this only
    * affects received data.
+   *
+   * We always identify ourselves as a Broccoli instance.
    */
-  caps[0] = 0;
+  caps[0] = BRO_CAP_BROCCOLI_PEER;
   caps[1] = 0;
   caps[2] = 0;
 
@@ -209,22 +211,22 @@ conn_init_handshake(BroConn *bc)
     }
 
   /* --- End of phase ------------------------------------------------ */
-  
+
   if (! __bro_io_raw_queue(bc, BRO_MSG_PHASE_DONE, NULL, 0))
     {
       D(("End-of-Handshake not sent to %p\n", bc));
       D_RETURN_(FALSE);
     }
-  
+
   if (bc->state->sync_state_requested)
     {
       D(("Phase done to peer on %p, sync requested, self now in SYNC stage.\n", bc));
-      result = conn_init_await(bc, BRO_CONNSTATE_SYNC);  
+      result = conn_init_await(bc, BRO_CONNSTATE_SYNC);
     }
   else
     {
       D(("Phase done to peer on %p, no sync requested, self now in RUNNING stage.\n", bc));
-      result = conn_init_await(bc, BRO_CONNSTATE_RUNNING);  
+      result = conn_init_await(bc, BRO_CONNSTATE_RUNNING);
     }
 
   D_RETURN_(result);
@@ -251,10 +253,10 @@ conn_init_sync(BroConn *bc)
 
       D(("Phase done to peer on %p\n", bc));
     }
-  
+
   D(("Self now in RUNNING stage.\n"));
   result = conn_init_await(bc, BRO_CONNSTATE_RUNNING);
-  
+
   D_RETURN_(result);
 }
 
@@ -270,7 +272,7 @@ conn_init_configure(BroConn *bc)
     return FALSE;
   if (! conn_init_sync(bc))
     return FALSE;
-  
+
   return TRUE;
 }
 
@@ -284,9 +286,11 @@ conn_init(BroConn *bc)
   if (! (bc->tx_buf = __bro_buf_new()))
     goto error_exit;
 
-  if (! (bc->state = calloc(1, sizeof(BroConnState))))
+  if (! (bc->state = calloc(1, sizeof(BroConnState)))) {
+    D(("Out of memory in conn_init.\n"));
     goto error_exit;
-  
+  }
+
   bc->state->conn_state_self = BRO_CONNSTATE_SETUP;
   bc->state->conn_state_peer = BRO_CONNSTATE_SETUP;
 
@@ -294,7 +298,7 @@ conn_init(BroConn *bc)
     goto error_exit;
 
   D_RETURN_(TRUE);
-  
+
  error_exit:
   __bro_buf_free(bc->rx_buf);
   __bro_buf_free(bc->tx_buf);
@@ -343,6 +347,11 @@ bro_init(const BroCtx* ctx)
 
   if (ctx == NULL) {
     ctx = calloc(1, sizeof(BroCtx));
+    if (ctx == NULL) {
+      D(("Unable to allocate memory in bro_init.\n"));
+      return FALSE;
+    }
+
     bro_ctx_init((BroCtx*) ctx);
   }
 
@@ -365,17 +374,19 @@ bro_conn_new_str(const char *hostname, int flags)
 {
   BroConn *bc;
   static int counter = 0;
-  
+
   BRO_SAFETY_CHECK;
 
   D_ENTER;
-  
+
   if (! hostname || !*hostname)
     D_RETURN_(NULL);
-  
-  if (! (bc = (BroConn *) calloc(1, sizeof(BroConn))))
+
+  if (! (bc = (BroConn *) calloc(1, sizeof(BroConn)))) {
+    D(("Unable to allocate memory in bro_conn_new_str.\n"));
     D_RETURN_(NULL);
-  
+  }
+
   D(("Connecting to host %s\n", hostname));
 
   bc->conn_flags = flags;
@@ -397,19 +408,19 @@ bro_conn_new_str(const char *hostname, int flags)
 				     (BroHTFreeFunc) __bro_sobject_release,
 				     TRUE)))
     goto error_exit;
-  
+
   if (! (bc->data = __bro_ht_new(__bro_ht_str_hash,
 				 __bro_ht_str_cmp,
 				 __bro_ht_mem_free,
 				 NULL, FALSE)))
     goto error_exit;
-  
+
   if (! (bc->ev_mask = __bro_ht_new(__bro_ht_str_hash,
 				    __bro_ht_str_cmp,
 				    __bro_ht_mem_free,
 				    NULL, FALSE)))
-    goto error_exit;  
-  
+    goto error_exit;
+
   D_RETURN_(bc);
 
  error_exit:
@@ -417,10 +428,10 @@ bro_conn_new_str(const char *hostname, int flags)
   __bro_ht_free(bc->ev_mask);
   __bro_ht_free(bc->io_cache);
   __bro_ht_free(bc->data);
-  
+
   if (bc->peer)
     free(bc->peer);
-  
+
   free(bc);
   D_RETURN_(NULL);
 }
@@ -431,11 +442,29 @@ bro_conn_new(struct in_addr *ip_addr, uint16 port, int flags)
 {
   BroConn *bc;
   char hostname[1024];
+  char addr[INET_ADDRSTRLEN];
 
   BRO_SAFETY_CHECK;
 
   D_ENTER;
-  __bro_util_snprintf(hostname, 1024, "%s:%hu", inet_ntoa(*ip_addr), ntohs(port));
+  inet_ntop(AF_INET, ip_addr, addr, INET_ADDRSTRLEN);
+  __bro_util_snprintf(hostname, 1024, "%s:%hu", addr, ntohs(port));
+  bc = bro_conn_new_str(hostname, flags);
+  D_RETURN_(bc);
+}
+
+BroConn *
+bro_conn_new6(struct in6_addr *ip_addr, uint16 port, int flags)
+{
+  BroConn *bc;
+  char hostname[1024];
+  char addr[INET6_ADDRSTRLEN];
+
+  BRO_SAFETY_CHECK;
+
+  D_ENTER;
+  inet_ntop(AF_INET6, ip_addr, addr, INET6_ADDRSTRLEN);
+  __bro_util_snprintf(hostname, 1024, "%s:%hu", addr, ntohs(port));
   bc = bro_conn_new_str(hostname, flags);
   D_RETURN_(bc);
 }
@@ -505,7 +534,7 @@ bro_conn_connect(BroConn *bc)
 
   if (! conn_init(bc))
     D_RETURN_(FALSE);
-  
+
   /* Handshake procedure. */
   if (! conn_init_configure(bc))
     D_RETURN_(FALSE);
@@ -528,7 +557,7 @@ bro_conn_reconnect(BroConn *bc)
 
   if (bc->state->in_reconnect)
     D_RETURN_(FALSE);
-  
+
   if ( (current_time = time(NULL)) > 0)
     {
       if (current_time - bc->state->last_reconnect < BRO_RECONNECT_MAX_RATE)
@@ -540,20 +569,20 @@ bro_conn_reconnect(BroConn *bc)
 
       bc->state->last_reconnect = current_time;
     }
-  
+
   D(("Attempting reconnection...\n"));
   bc->state->in_reconnect = TRUE;
-  
+
   /* NOTE: the sequencing in this function is quite tricky and very picky.
    * Don't move things around unneccesarily ...
    */
   bc->state->tx_dead = bc->state->rx_dead = FALSE;
-  
+
   /* Clean up the old connection state: */
   bc->state->conn_state_self = BRO_CONNSTATE_SETUP;
   bc->state->conn_state_peer = BRO_CONNSTATE_SETUP;
   bc->state->sync_state_requested = FALSE;
-  
+
   if (bc->bio)
     {
       BIO_free_all(bc->bio);
@@ -587,11 +616,11 @@ bro_conn_reconnect(BroConn *bc)
   if (! (bc->ev_mask = __bro_ht_new(__bro_ht_str_hash,
 				    __bro_ht_str_cmp,
 				    __bro_ht_mem_free,
-				    NULL, FALSE)))				    
+				    NULL, FALSE)))
     goto reset_error_return;
 
   __bro_ht_free(bc->io_cache);
-  
+
   if (! (bc->io_cache = __bro_ht_new(__bro_ht_int_hash,
 				     __bro_ht_int_cmp,
 				     NULL,
@@ -616,7 +645,7 @@ bro_conn_reconnect(BroConn *bc)
 
   D(("Reconnect completed successfully.\n"));
   bc->state->in_reconnect = FALSE;
-  
+
   D_RETURN_(TRUE);
 
  reset_error_return:
@@ -629,11 +658,11 @@ bro_conn_reconnect(BroConn *bc)
       TAILQ_REMOVE(&bc->msg_queue, msg, msg_queue);
       __bro_io_msg_free(msg);
     }
-  
+
   bc->msg_queue.tqh_first = msg_first;
   bc->msg_queue.tqh_last = msg_last;
   bc->msg_queue_len = msg_queue_len;
-  
+
  error_return:
   bc->state->tx_dead = bc->state->rx_dead = TRUE;
   bc->state->in_reconnect = FALSE;
@@ -642,7 +671,7 @@ bro_conn_reconnect(BroConn *bc)
 }
 
 
-int 
+int
 bro_conn_delete(BroConn *bc)
 {
   BroMsg *msg;
@@ -702,11 +731,11 @@ conn_adopt_events_cb(char *ev_name, void *data, BroHT *dst)
   /* If the event is already reported, ignore. */
   if (__bro_ht_get(dst, ev_name))
     return TRUE;
-  
+
   D(("Adopting event %s\n", ev_name));
   key = strdup(ev_name);
   __bro_ht_add(dst, key, key);
-  
+
   return TRUE;
   data = NULL;
 }
@@ -732,7 +761,7 @@ bro_conn_get_fd(BroConn *bc)
 
   if (! bc || !bc->state || bc->state->tx_dead || bc->state->rx_dead || !bc->bio)
     return -1;
-  
+
 #ifdef __MINGW32__
   return 0;
 #else
@@ -770,7 +799,7 @@ bro_event_queue_length_max(BroConn *bc)
 }
 
 
-int            
+int
 bro_event_queue_flush(BroConn *bc)
 {
   return __bro_io_msg_queue_flush(bc);
@@ -820,9 +849,18 @@ bro_event_send_raw(BroConn *bc, const uchar *data, int data_len)
       D(("Out of memory\n"));
       D_RETURN_(FALSE);
     }
-      
-  __bro_buf_write_char(buf, 'e');
-  __bro_buf_write_data(buf, data, data_len);
+
+  if (! __bro_buf_write_char(buf, 'e'))
+    {
+      __bro_buf_free(buf);
+      D_RETURN_(FALSE);
+    }
+
+  if (! __bro_buf_write_data(buf, data, data_len))
+    {
+      __bro_buf_free(buf);
+      D_RETURN_(FALSE);
+    }
 
   __bro_io_rawbuf_queue(bc, BRO_MSG_SERIAL, buf);
   __bro_io_msg_queue_flush(bc);
@@ -856,7 +894,7 @@ bro_conn_data_del(BroConn *bc, const char *key)
 {
   if (!bc || !key || !*key)
     return NULL;
-  
+
   return __bro_ht_del(bc->data, (void *) key);
 }
 
@@ -896,7 +934,7 @@ bro_event_add_val(BroEvent *be, int type, const char *type_name, const void *val
       D(("Invalid input: (%p, %i, %p)\n", be, type, val));
       D_RETURN_(FALSE);
     }
-  
+
   if (! (v = __bro_val_new_of_type(type, type_name)))
     D_RETURN_(FALSE);
 
@@ -936,7 +974,7 @@ bro_event_set_val(BroEvent *be, int val_num,
       __bro_sobject_release((BroSObject *) v);
       D_RETURN_(FALSE);
     }
-  
+
   result = __bro_event_set_val(be, val_num, v);
   D_RETURN_(result);
 }
@@ -946,7 +984,7 @@ void
 bro_event_registry_add(BroConn *bc,
 		       const char *event_name,
 		       BroEventFunc func,
-		       void *user_data)		       
+		       void *user_data)
 {
   __bro_event_reg_add(bc, event_name, func, user_data);
 }
@@ -956,7 +994,7 @@ void
 bro_event_registry_add_compact(BroConn *bc,
 			       const char *event_name,
 			       BroCompactEventFunc func,
-			       void *user_data)		       
+			       void *user_data)
 {
   __bro_event_reg_add_compact(bc, event_name, func, user_data);
 }
@@ -973,7 +1011,7 @@ void
 bro_event_registry_request(BroConn *bc)
 {
   D_ENTER;
-  
+
   if (!bc || !bc->state)
     D_RETURN;
 
@@ -983,9 +1021,9 @@ bro_event_registry_request(BroConn *bc)
    */
   if (bc->state->conn_state_self != BRO_CONNSTATE_RUNNING)
     D_RETURN;
-  
+
   __bro_event_reg_request(bc);
-  
+
   D_RETURN;
 }
 
@@ -998,8 +1036,10 @@ bro_buf_new(void)
 {
   BroBuf *buf;
 
-  if (! (buf = calloc(1, sizeof(BroBuf))))
+  if (! (buf = calloc(1, sizeof(BroBuf)))) {
+    D(("Unable to allocate memory for buffer in bro_buf_new.\n"));
     return NULL;
+  }
 
   __bro_buf_init(buf);
   return buf;
@@ -1150,7 +1190,7 @@ bro_conf_get_str(const char *val_name)
 
   if (! val_name)
     return FALSE;
-  
+
   return __bro_conf_get_str(val_name);
 }
 
@@ -1189,13 +1229,13 @@ bro_record_add_val(BroRecord *rec, const char *name,
       D(("Input error: (%p, %s, %i, %p)\n", rec, name, type, val));
       D_RETURN_(FALSE);
     }
-  
+
   if (! (v = __bro_val_new_of_type(type, type_name)))
     {
       D(("Could not get val of type %i\n", type));
       D_RETURN_(FALSE);
     }
-  
+
   if (! name)
     name = "";
 
@@ -1213,17 +1253,17 @@ bro_record_add_val(BroRecord *rec, const char *name,
 }
 
 
-const char* 
+const char*
 bro_record_get_nth_name(BroRecord *rec, int num)
 {
   const char *name;
- 
+
   if ( (name = __bro_record_get_nth_name(rec, num)))
     return name;
 
   return NULL;
 }
- 
+
 
 void *
 bro_record_get_nth_val(BroRecord *rec, int num, int *type)
@@ -1246,7 +1286,7 @@ bro_record_get_nth_val(BroRecord *rec, int num, int *type)
    */
   if (! __bro_val_get_data(val, &type_found, &result))
     return NULL;
-  
+
   if (type)
     {
       if (*type != BRO_TYPE_UNKNOWN && type_found != *type)
@@ -1254,10 +1294,10 @@ bro_record_get_nth_val(BroRecord *rec, int num, int *type)
 	  D(("Type mismatch: expected type tag %i, found type tag %i\n", *type, type_found));
 	  result = NULL;
 	}
-      
+
       *type = type_found;
     }
-  
+
   return result;
 }
 
@@ -1268,13 +1308,13 @@ bro_record_get_named_val(BroRecord *rec, const char *name, int *type)
   BroVal *val;
   int type_found;
   void *result = NULL;
- 
+
   if (type && (*type < BRO_TYPE_UNKNOWN || *type >= BRO_TYPE_MAX))
     {
       D(("Invalid value for type pointer (%i)\n", *type));
       return NULL;
     }
- 
+
   if (! (val = __bro_record_get_named_val(rec, name)))
     return NULL;
 
@@ -1283,7 +1323,7 @@ bro_record_get_named_val(BroRecord *rec, const char *name, int *type)
    */
   if (! __bro_val_get_data(val, &type_found, &result))
     return NULL;
-  
+
   if (type)
     {
       if (*type != BRO_TYPE_UNKNOWN && type_found != *type)
@@ -1295,7 +1335,7 @@ bro_record_get_named_val(BroRecord *rec, const char *name, int *type)
 
       *type = type_found;
     }
-  
+
   return result;
 }
 
@@ -1315,19 +1355,19 @@ bro_record_set_nth_val(BroRecord *rec, int num,
       D(("Input error: (%p, %i, %i, %p)\n", rec, num, type, val));
       D_RETURN_(FALSE);
     }
-  
+
   if (! (v = __bro_record_get_nth_val(rec, num)))
     D_RETURN_(FALSE);
 
   if (! (name = __bro_sobject_data_get((BroSObject *) v, "field")))
     D_RETURN_(FALSE);
-  
+
   if (! (v = __bro_val_new_of_type(type, type_name)))
     {
       D(("Could not get val of type %i\n", type));
       D_RETURN_(FALSE);
     }
-  
+
   __bro_sobject_data_set((BroSObject *) v, "field", strdup(name));
 
   if (! __bro_val_assign(v, val))
@@ -1356,20 +1396,20 @@ bro_record_set_named_val(BroRecord *rec, const char *name,
       D(("Input error: (%p, %s, %i, %p)\n", rec, name, type, val));
       D_RETURN_(FALSE);
     }
-  
+
   if (! (v = __bro_val_new_of_type(type, type_name)))
     {
       D(("Could not get val of type %i\n", type));
       D_RETURN_(FALSE);
     }
-    
+
   if (! __bro_val_assign(v, val))
     {
       D(("Could not assign value to the new val.\n"));
       __bro_sobject_release((BroSObject *) v);
       D_RETURN_(FALSE);
     }
-  
+
   __bro_record_set_named_val(rec, name, v);
   D_RETURN_(TRUE);
 }
@@ -1438,13 +1478,13 @@ bro_table_insert(BroTable *tbl,
    * used in the obvious way.
    */
   lv = __bro_list_val_new();
-  
+
   if (key_type == BRO_TYPE_LIST)
     {
       /* We need to unroll the record elements and put them
        * all in the list val.
        */
-      
+
       BroRecord *rec = (BroRecord*) key;
       int i;
 
@@ -1455,7 +1495,7 @@ bro_table_insert(BroTable *tbl,
 	   */
 	  BroVal *v = __bro_record_get_nth_val(rec, i);
 	  BroVal *v_copy = (BroVal*) __bro_sobject_copy((BroSObject*) v);
-	  __bro_list_val_append(lv, v_copy); 
+	  __bro_list_val_append(lv, v_copy);
 	}
     }
   else
@@ -1468,26 +1508,28 @@ bro_table_insert(BroTable *tbl,
       if (! (kv = __bro_val_new_of_type(key_type, NULL)))
 	{
 	  D(("Could not create val of type %d\n", key_type));
+	  __bro_sobject_release((BroSObject*) lv);
 	  D_RETURN_(FALSE);
 	}
-      
+
       __bro_val_assign(kv, key);
       __bro_list_val_append(lv, kv);
-    }  
-  
+    }
+
   if (val)
     {
       if (! (vv = __bro_val_new_of_type(val_type, NULL)))
 	{
 	  D(("Could not crate val of type %d\n", val_type));
+	  __bro_sobject_release((BroSObject*) lv);
 	  D_RETURN_(FALSE);
 	}
-      
+
       __bro_val_assign(vv, val);
     }
 
-  __bro_table_insert(tbl, (BroVal*) lv, vv);  
-  
+  __bro_table_insert(tbl, (BroVal*) lv, vv);
+
   D_RETURN_(TRUE);
 }
 
@@ -1519,22 +1561,23 @@ bro_table_find(BroTable *tbl, const void *key)
       if (! (val = __bro_val_new_of_type(tbl->tbl_key_type, NULL)))
 	{
 	  D(("Could not create val of type %d.\n", tbl->tbl_key_type));
+	  __bro_sobject_release((BroSObject*) lv);
 	  D_RETURN_(NULL);
 	}
-      
+
       __bro_val_assign(val, key);
       __bro_list_val_append(lv, val);
     }
-  
+
   if ( (result_val = __bro_table_find(tbl, (BroVal*) lv)))
     {
       if (! __bro_val_get_data(result_val, NULL, &result))
 	{
 	  __bro_sobject_release((BroSObject*) lv);
-	  D_RETURN_(NULL);	  
+	  D_RETURN_(NULL);
 	}
     }
-  
+
   if (rec)
     {
       lv->list = NULL;
@@ -1542,7 +1585,7 @@ bro_table_find(BroTable *tbl, const void *key)
     }
 
   __bro_sobject_release((BroSObject*) lv);
-  
+
   D_RETURN_(result);
 }
 
@@ -1558,7 +1601,7 @@ bro_table_get_size(BroTable *tbl)
 }
 
 typedef struct bro_table_cb_data
-{ 
+{
   void *user_data;
   BroTableCallback cb;
   int is_set;
@@ -1570,17 +1613,17 @@ bro_table_foreach_cb(BroListVal *key, BroVal *val, BroTableCBData *data)
   int result;
   void *key_data = NULL, *val_data = NULL;
   BroRecord *rec = NULL;
-  
+
   if (__bro_list_val_get_length(key) > 1)
     {
       /* Need to shrink-wrap it into a record. */
       BroList *l;
       rec = __bro_record_new();
-      
+
       for (l = key->list; l; l = __bro_list_next(l))
 	{
 	  BroVal *tmp = (BroVal*) __bro_list_data(l);
-	  
+
 	  /* __bro_record_add_val() does not internally copy the added
 	   * val. Without bumping up the val's refcount, __bro_record_free()
 	   * below would possibly nuke the val when it decrements the count
@@ -1598,27 +1641,27 @@ bro_table_foreach_cb(BroListVal *key, BroVal *val, BroTableCBData *data)
       BroVal *v = __bro_list_val_get_front(key);
       D(("Index type is atomic, type %d/%d\n",
 	 v->val_type->tag, v->val_type->internal_tag));
-    
+
       if (! __bro_val_get_data(v, NULL, &key_data))
 	{
 	  D(("Failed to obtain user-suitable data representation.\n"));
 	  return TRUE;
 	}
     }
-  
+
   if (! data->is_set && ! __bro_val_get_data(val, NULL, &val_data))
     {
       D(("Failed to obtain user-suitable data representation.\n"));
       result = FALSE;
       goto return_result;
     }
-  
+
   result = data->cb(key_data, val_data, data->user_data);
-  
+
  return_result:
   if (rec)
     __bro_record_free(rec);
-  
+
   return result;
 }
 
@@ -1679,11 +1722,11 @@ bro_set_insert(BroSet *set, int type, const void *val)
   int result;
 
   D_ENTER;
-  
+
   result = bro_table_insert((BroTable*) set,
 			    type, val,
 			    BRO_TYPE_UNKNOWN, NULL);
-  
+
   D_RETURN_(result);
 }
 
@@ -1708,7 +1751,7 @@ bro_set_get_size(BroSet *set)
 }
 
 typedef struct bro_set_cb_data
-{ 
+{
   void *user_data;
   BroSetCallback cb;
 } BroSetCBData;
@@ -1727,14 +1770,14 @@ bro_set_foreach(BroSet *set,
   BroSetCBData data;
 
   D_ENTER;
-  
+
   data.user_data = user_data;
   data.cb = cb;
 
   bro_table_foreach((BroTable*) set,
 		    (BroTableCallback) bro_set_foreach_cb,
 		    &data);
-  
+
   D_RETURN;
 }
 
@@ -1761,7 +1804,7 @@ bro_string_set(BroString *bs, const char *s)
 {
   if (! bs || !s)
     return FALSE;
-  
+
   return bro_string_set_data(bs, (const uchar *) s, strlen(s));
 }
 
@@ -1770,16 +1813,18 @@ int
 bro_string_set_data(BroString *bs, const uchar *data, int data_len)
 {
   uchar *data_copy;
-  
+
   if (! bs || !data || data_len < 0)
     return FALSE;
-  
-  if (! (data_copy = malloc(data_len + 1)))
+
+  if (! (data_copy = malloc(data_len + 1))) {
+    D(("Out of memory in bro_string_set_data.\n"));
     return FALSE;
-  
+  }
+
   memcpy(data_copy, data, data_len);
   data_copy[data_len] = '\0';
-  
+
   bs->str_len = data_len;
   bs->str_val = data_copy;
 
@@ -1809,8 +1854,10 @@ bro_string_copy(BroString *bs)
   if (! bs)
     return NULL;
 
-  if (! (result = calloc(1, sizeof(BroString))))
+  if (! (result = calloc(1, sizeof(BroString)))) {
+    D(("Unable to allocate memory for BroString in bro_string_copy.\n"));
     return NULL;
+  }
 
   bro_string_assign(bs, result);
   return result;
@@ -1893,14 +1940,17 @@ bro_packet_new(const struct pcap_pkthdr *hdr, const u_char *data, const char *ta
   if (! hdr || ! data)
     return NULL;
 
-  if (! (packet = calloc(1, sizeof(BroPacket))))
+  if (! (packet = calloc(1, sizeof(BroPacket)))) {
+    D(("Unable to allocate memory for packet in bro_packet_new.\n"));
     return NULL;
- 
+  }
+
   packet->pkt_pcap_hdr = *hdr;
   packet->pkt_tag = strdup(tag ? tag : "");
 
   if (! (packet->pkt_data = malloc(hdr->caplen)))
     {
+      D(("Unable to allocate memory for pkt->data in bro_packet_new.\n"));
       free(packet);
       return NULL;
     }
@@ -1914,10 +1964,12 @@ BroPacket *
 bro_packet_clone(const BroPacket *src)
 {
   BroPacket *dst;
-  
-  if (! (dst = calloc(1, sizeof(BroPacket))))
+
+  if (! (dst = calloc(1, sizeof(BroPacket)))) {
+    D(("Unable to allocate memory in bro_packet_clone.\n"));
     return NULL;
-  
+  }
+
   if (! __bro_packet_clone(dst, src))
     {
       bro_packet_free(dst);
@@ -1966,11 +2018,17 @@ bro_util_current_time(void)
 }
 
 
-double 
+double
 bro_util_timeval_to_double(const struct timeval *tv)
 {
   return __bro_util_timeval_to_double(tv);
 }
 
+int
+bro_util_is_v4_addr(const BroAddr *a)
+{
+  return __bro_util_is_v4_addr(a);
+}
 
+const uint8 BRO_IPV4_MAPPED_PREFIX[12] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0xff, 0xff };
 
