@@ -37,7 +37,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #ifdef __EMX__
 #include <strings.h>
-#endif 
+#endif
 
 #include <bro_types.h>
 #include <bro_type.h>
@@ -46,6 +46,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <bro_val.h>
 #include <bro_record.h>
 #include <bro_table.h>
+#include <bro_util.h>
 
 /* The virtual implementations of BroSObject's functions are
  * kept static in this module, and so are the ..._init() methods
@@ -131,13 +132,12 @@ __bro_val_new_of_type(int type, const char *type_name)
     case BRO_TYPE_TIMER:
     case BRO_TYPE_PORT:
     case BRO_TYPE_IPADDR:
-    case BRO_TYPE_NET:
     case BRO_TYPE_SUBNET:
     case BRO_TYPE_ENUM:
       if (! (val = __bro_val_new()))
 	D_RETURN_(NULL);
       break;
-      
+
     case BRO_TYPE_SET:
       /* A hack -- sets are table vals, but have set type,
        * at least while Bro still has SetType.
@@ -151,7 +151,7 @@ __bro_val_new_of_type(int type, const char *type_name)
       if (! (val = (BroVal *) __bro_record_val_new()))
 	D_RETURN_(NULL);
       break;
-      
+
     case BRO_TYPE_PATTERN:
     case BRO_TYPE_ANY:
     case BRO_TYPE_UNION:
@@ -164,13 +164,13 @@ __bro_val_new_of_type(int type, const char *type_name)
       D(("Unsupported value type %i\n", type));
       D_RETURN_(NULL);
     }
-  
+
   if (! (val->val_type = __bro_type_new_of_type(type, type_name)))
     {
       __bro_val_free(val);
       D_RETURN_(NULL);
     }
-  
+
   D_RETURN_(val);
 }
 
@@ -185,7 +185,7 @@ __bro_val_assign(BroVal *val, const void *data)
       D(("Input error: (%p, %p)\n", val, data));
       D_RETURN_(FALSE);
     }
-  
+
   if (! data)
     {
       if (val->val_type)
@@ -219,7 +219,7 @@ __bro_val_assign(BroVal *val, const void *data)
     case BRO_TYPE_COUNT:
     case BRO_TYPE_COUNTER:
     case BRO_TYPE_ENUM:
-      val->val_int = *((int *) data);
+      val->val_int64 = *((uint64 *) data);
       break;
 
     case BRO_TYPE_DOUBLE:
@@ -234,11 +234,11 @@ __bro_val_assign(BroVal *val, const void *data)
 	bro_string_set_data(&val->val_str, str->str_val, str->str_len);
       }
       break;
-      
+
     case BRO_TYPE_PORT:
       {
 	BroPort *tmp = (BroPort *) data;
-	
+
 	if (tmp->port_proto != IPPROTO_TCP &&
 	    tmp->port_proto != IPPROTO_UDP &&
 	    tmp->port_proto != IPPROTO_ICMP)
@@ -246,64 +246,63 @@ __bro_val_assign(BroVal *val, const void *data)
 	    __bro_sobject_release((BroSObject *) data);
 	    D_RETURN_(FALSE);
 	  }
-	
+
 	val->val_port = *tmp;
       }
       break;
-      
+
     case BRO_TYPE_IPADDR:
-    case BRO_TYPE_NET:
-      val->val_int = *((uint32 *) data);
+      val->val_addr = *((BroAddr *) data);
       break;
-      
+
     case BRO_TYPE_SUBNET:
       val->val_subnet = *((BroSubnet *) data);
       break;
-      
+
     case BRO_TYPE_RECORD:
       {
 	BroList *l;
 	BroVal *tmp_val;
 	BroRecordVal *rv = (BroRecordVal *) val;
 	BroRecord *rec = (BroRecord *) data;
-	
+
 	if (rv->rec)
 	  __bro_record_free(rv->rec);
-	
+
 	rv->rec = __bro_record_copy(rec);
 
 	/* Record vals also have a record type, copy that: */
 	for (l = rec->val_list; l; l = __bro_list_next(l))
 	  {
 	    char *field;
-	    
+
 	    tmp_val = __bro_list_data(l);
-	    
+
 	    if (! tmp_val->val_type)
 	      {
 		D(("Cannot create record type component from val without type.\n"));
 		D_RETURN_(FALSE);
 	      }
-	    
+
 	    if (! (field = __bro_sobject_data_get((BroSObject *) tmp_val, "field")))
 	      {
 		D(("Val in record doesn't have field name associated with it.\n"));
 		D_RETURN_(FALSE);
 	      }
-	    
+
 	    __bro_record_type_add_type((BroRecordType *) val->val_type, field, tmp_val->val_type);;
 	  }
       }
       break;
-      
+
     case BRO_TYPE_TABLE:
       {
 	BroTableVal *tv = (BroTableVal *) val;
 	BroTable *table = (BroTable *) data;
-	
+
 	if (tv->table)
 	  __bro_table_free(tv->table);
-	
+
 	tv->table = __bro_table_copy(table);
 
 	/* XXX need to create the appropriate content in (BroTableType*) val->val_type! */
@@ -321,7 +320,7 @@ __bro_val_assign(BroVal *val, const void *data)
     case BRO_TYPE_ERROR:
       D(("Type %i currently unsupported.\n", val->val_type->tag));
       D_RETURN_(FALSE);
-      
+
     default:
       D(("Unknown type identifier %i\n", val->val_type->tag));
       D_RETURN_(FALSE);
@@ -337,9 +336,9 @@ __bro_val_init(BroVal *val)
   BroSObject *sobj = (BroSObject *) val;
 
   D_ENTER;
-  
+
   __bro_object_init((BroObject *) val);
-  
+
   sobj->read  = (BroSObjectRead) __bro_val_read;
   sobj->write = (BroSObjectWrite) __bro_val_write;
   sobj->free  = (BroSObjectFree) __bro_val_free;
@@ -375,13 +374,13 @@ __bro_val_free(BroVal *val)
 	case BRO_TYPE_STRING:
 	  bro_string_cleanup(&val->val_str);
 	  break;
-	  
+
 	default:
 	  /* Nothing to do */
 	  break;
 	}
     }
-  
+
   __bro_sobject_release((BroSObject *) val->val_type);
   __bro_object_free((BroObject *) val);
 
@@ -410,7 +409,7 @@ __bro_val_get_data(BroVal *val, int *type, void **data)
 
   if (type && val->val_type)
     *type = val->val_type->tag;
-  
+
   *data = val->get_data(val);
   return TRUE;
 }
@@ -421,6 +420,7 @@ __bro_val_read(BroVal *val, BroConn *bc)
 {
   char opt;
   uint32 tmp;
+  int i;
 
   D_ENTER;
 
@@ -443,9 +443,9 @@ __bro_val_read(BroVal *val, BroConn *bc)
 
   D(("Type in val has type tags %i/%i\n",
      val->val_type->tag, val->val_type->internal_tag));
-     
+
   /* Read optional Attributes */
-  
+
   if (val->val_attrs)
     {
       __bro_sobject_release((BroSObject *) val->val_attrs);
@@ -459,7 +459,7 @@ __bro_val_read(BroVal *val, BroConn *bc)
       if (! (val->val_attrs = (BroRecordVal *) __bro_sobject_unserialize(SER_RECORD_VAL, bc)))
 	D_RETURN_(FALSE);
     }
-  
+
   switch (val->val_type->internal_tag)
     {
     case BRO_INTTYPE_INT:
@@ -467,68 +467,100 @@ __bro_val_read(BroVal *val, BroConn *bc)
       /* Hack for ports */
       if (val->val_type->tag == BRO_TYPE_PORT)
 	{
-	  if (! __bro_buf_read_int(bc->rx_buf, &tmp))
+      uint64 tmp;
+	  if (! __bro_buf_read_int64(bc->rx_buf, &tmp))
 	    D_RETURN_(FALSE);
-	  
+
 	  if ( (tmp & 0xf0000) == 0x10000 )
 	    val->val_port.port_proto = IPPROTO_TCP;
 	  else if ( (tmp & 0xf0000) == 0x20000 )
 	    val->val_port.port_proto = IPPROTO_UDP;
 	  else if ( (tmp & 0xf0000) == 0x30000 )
 	    val->val_port.port_proto = IPPROTO_ICMP;
-	    
+
 	  val->val_port.port_num = (tmp & 0xFFFF);
 	}
       else
 	{
-	  if (! __bro_buf_read_int(bc->rx_buf, &val->val_int))
+	  if (! __bro_buf_read_int64(bc->rx_buf, &val->val_int64))
 	    D_RETURN_(FALSE);
 	}
       break;
-      
+
     case BRO_INTTYPE_DOUBLE:
       if (! __bro_buf_read_double(bc->rx_buf, &val->val_double))
 	D_RETURN_(FALSE);
       break;
-      
+
     case BRO_INTTYPE_STRING:
       if (! __bro_buf_read_string(bc->rx_buf, &val->val_str))
 	D_RETURN_(FALSE);
       break;
 
     case BRO_INTTYPE_IPADDR:
-      if (! __bro_buf_read_int(bc->rx_buf, &tmp))
-	D_RETURN_(FALSE);
-      
-      if (tmp != 1)
-	{
-	  D(("We don't handle IPv6 addresses yet.\n"));
-	  D_RETURN_(FALSE);
-	}
-      
-      if (! __bro_buf_read_int(bc->rx_buf, &val->val_int))
-	D_RETURN_(FALSE);
+		if (! __bro_buf_read_int(bc->rx_buf, &tmp))
+			D_RETURN_(FALSE);
 
-      val->val_int = ntohl(val->val_int);
-      break;
+		if (tmp != 1 && tmp != 4)
+			{
+			D(("Bad IP addresses word length: %d.\n", tmp));
+			D_RETURN_(FALSE);
+			}
+
+		if ( tmp == 1 )
+			{
+			if (! __bro_buf_read_int(bc->rx_buf, &tmp))
+				D_RETURN_(FALSE);
+			__bro_util_fill_v4_addr(&val->val_addr, ntohl(tmp));
+			}
+
+		else
+			{
+			for ( i = 0; i < tmp; ++i )
+				{
+				if (! __bro_buf_read_int(bc->rx_buf, &val->val_addr.addr[i]))
+					D_RETURN_(FALSE);
+
+				val->val_addr.addr[i] = ntohl(val->val_addr.addr[i]);
+				}
+			}
+
+		break;
 
     case BRO_INTTYPE_SUBNET:
-      if (! __bro_buf_read_int(bc->rx_buf, &tmp))
-	D_RETURN_(FALSE);
-      
-      if (tmp != 1)
-	{
-	  D(("We don't handle IPv6 addresses yet.\n"));
-	  D_RETURN_(FALSE);
-	}
+		if (! __bro_buf_read_int(bc->rx_buf, &tmp))
+			D_RETURN_(FALSE);
 
-      if (! __bro_buf_read_int(bc->rx_buf, &val->val_subnet.sn_net))
-	D_RETURN_(FALSE);
-      val->val_subnet.sn_net = ntohl(val->val_subnet.sn_net);
+		if (tmp != 1 && tmp != 4)
+			{
+			D(("Bad IP addresses word length: %d.\n", tmp));
+			D_RETURN_(FALSE);
+			}
 
-      if (! __bro_buf_read_int(bc->rx_buf, &val->val_subnet.sn_width))
-	D_RETURN_(FALSE);
-      break;
+		if ( tmp == 1 )
+			{
+			if (! __bro_buf_read_int(bc->rx_buf, &tmp))
+				D_RETURN_(FALSE);
+			__bro_util_fill_v4_addr(&val->val_subnet.sn_net, ntohl(tmp));
+			}
+
+		else
+			{
+			for ( i = 0; i < tmp; ++i )
+				{
+				if (! __bro_buf_read_int(bc->rx_buf,
+										 &val->val_subnet.sn_net.addr[i]))
+					D_RETURN_(FALSE);
+
+				val->val_subnet.sn_net.addr[i] =
+					ntohl(val->val_subnet.sn_net.addr[i]);
+				}
+			}
+
+		if (! __bro_buf_read_int(bc->rx_buf, &val->val_subnet.sn_width))
+			D_RETURN_(FALSE);
+
+		break;
 
     case BRO_INTTYPE_OTHER:
       /* See Val.cc around 165 -- these are handled by derived classes.
@@ -537,14 +569,14 @@ __bro_val_read(BroVal *val, BroConn *bc)
       if (val->val_type->tag != BRO_TYPE_FUNC &&
 	  val->val_type->tag != BRO_TYPE_FILE)
 	break;
-      
+
       /* Otherwise fall through to warning. */
 
     default:
       D(("Unsupported internal type tag: %i\n", val->val_type->internal_tag));
       D_RETURN_(FALSE);
     }
-  
+
   D_RETURN_(TRUE);
 }
 
@@ -554,9 +586,10 @@ __bro_val_write(BroVal *val, BroConn *bc)
 {
   BroType *type;
   BroSObject *obj;
+  int i;
 
   D_ENTER;
-  
+
   if (! val || !bc)
     D_RETURN_(FALSE);
 
@@ -572,7 +605,7 @@ __bro_val_write(BroVal *val, BroConn *bc)
 
   type = (BroType *) val->val_type;
   obj  = (BroSObject *) val;
-  
+
   switch (type->tag)
     {
     case BRO_TYPE_BOOL:
@@ -592,19 +625,15 @@ __bro_val_write(BroVal *val, BroConn *bc)
     case BRO_TYPE_PORT:
       obj->type_id = SER_PORT_VAL;
       break;
-      
+
     case BRO_TYPE_INTERVAL:
       obj->type_id = SER_INTERVAL_VAL;
       break;
-      
+
     case BRO_TYPE_IPADDR:
       obj->type_id = SER_ADDR_VAL;
       break;
-      
-    case BRO_TYPE_NET:
-      obj->type_id = SER_NET_VAL;
-      break;
-      
+
     case BRO_TYPE_SUBNET:
       obj->type_id = SER_SUBNET_VAL;
       break;
@@ -612,7 +641,7 @@ __bro_val_write(BroVal *val, BroConn *bc)
     case BRO_TYPE_RECORD:
       obj->type_id = SER_RECORD_VAL;
       break;
-      
+
     default:
       D(("Val %p's type unhandled: type tag is %i.\n", val, type->tag));
       D_RETURN_(FALSE);
@@ -626,10 +655,10 @@ __bro_val_write(BroVal *val, BroConn *bc)
 
   if (! __bro_buf_write_char(bc->tx_buf, val->val_attrs ? 1 : 0))
     D_RETURN_(FALSE);
-  
+
   if (val->val_attrs && ! __bro_sobject_serialize((BroSObject *) val->val_attrs, bc))
     D_RETURN_(FALSE);
-  
+
   switch (val->val_type->internal_tag)
     {
     case BRO_INTTYPE_INT:
@@ -637,8 +666,8 @@ __bro_val_write(BroVal *val, BroConn *bc)
       /* Hack for ports */
       if (val->val_type->tag == BRO_TYPE_PORT)
 	{
-	  int tmp = val->val_port.port_num;
-	  
+	  uint64 tmp = val->val_port.port_num;
+
 	  if (val->val_port.port_proto == IPPROTO_TCP)
 	    tmp |= 0x10000;
 	  else if (val->val_port.port_proto == IPPROTO_UDP)
@@ -646,40 +675,58 @@ __bro_val_write(BroVal *val, BroConn *bc)
       else if (val->val_port.port_proto == IPPROTO_ICMP)
 	    tmp |= 0x30000;
 
-	  if (! __bro_buf_write_int(bc->tx_buf, tmp))
+	  if (! __bro_buf_write_int64(bc->tx_buf, tmp))
 	    D_RETURN_(FALSE);
 	}
       else
 	{
-	  if (! __bro_buf_write_int(bc->tx_buf, val->val_int))
+	  if (! __bro_buf_write_int64(bc->tx_buf, val->val_int64))
 	    D_RETURN_(FALSE);
 	}
       break;
-      
+
     case BRO_INTTYPE_DOUBLE:
       if (! __bro_buf_write_double(bc->tx_buf, val->val_double))
 	D_RETURN_(FALSE);
       break;
-      
+
     case BRO_INTTYPE_STRING:
       if (! __bro_buf_write_string(bc->tx_buf, &val->val_str))
 	D_RETURN_(FALSE);
       break;
 
     case BRO_INTTYPE_IPADDR:
-      if (! __bro_buf_write_int(bc->tx_buf, 1))
-	D_RETURN_(FALSE);
-      if (! __bro_buf_write_int(bc->tx_buf, htonl(val->val_int)))
-	D_RETURN_(FALSE);
-      break;
+		if ( __bro_util_is_v4_addr(&val->val_addr) )
+			i = 1;
+		else
+			i = 4;
+
+		if (! __bro_buf_write_int(bc->tx_buf, i))
+			D_RETURN_(FALSE);
+
+		for ( i = 4 - i; i < 4; ++i )
+			if (! __bro_buf_write_int(bc->tx_buf, htonl(val->val_addr.addr[i])))
+				D_RETURN_(FALSE);
+
+		break;
 
     case BRO_INTTYPE_SUBNET:
-      if (! __bro_buf_write_int(bc->tx_buf, 1))
-	D_RETURN_(FALSE);
-      if (! __bro_buf_write_int(bc->tx_buf, htonl(val->val_subnet.sn_net)))
-	D_RETURN_(FALSE);
-      if (! __bro_buf_write_int(bc->tx_buf, val->val_subnet.sn_width))
-	D_RETURN_(FALSE);
+		if ( __bro_util_is_v4_addr(&val->val_subnet.sn_net) )
+			i = 1;
+		else
+			i = 4;
+
+		if (! __bro_buf_write_int(bc->tx_buf, i))
+			D_RETURN_(FALSE);
+
+		for ( i = 4 - i; i < 4; ++i )
+			if (! __bro_buf_write_int(bc->tx_buf,
+			                          htonl(val->val_subnet.sn_net.addr[i])))
+				D_RETURN_(FALSE);
+
+		if (! __bro_buf_write_int(bc->tx_buf, val->val_subnet.sn_width))
+			D_RETURN_(FALSE);
+
       break;
 
     case BRO_INTTYPE_OTHER:
@@ -687,12 +734,12 @@ __bro_val_write(BroVal *val, BroConn *bc)
        * like __bro_record_val_write().
        */
       break;
-      
+
     default:
       D(("Unknown internal type tag: %i\n", val->val_type->internal_tag));
       D_RETURN_(FALSE);
-    }  
-  
+    }
+
   D_RETURN_(TRUE);
 }
 
@@ -700,7 +747,7 @@ static int
 __bro_val_clone(BroVal *dst, BroVal *src)
 {
   D_ENTER;
-  
+
   if (! __bro_object_clone((BroObject *) dst, (BroObject *) src))
     {
       D(("Cloning parent failed.\n"));
@@ -711,9 +758,9 @@ __bro_val_clone(BroVal *dst, BroVal *src)
       ! (dst->val_type = (BroType *) __bro_sobject_copy((BroSObject *) src->val_type)))
     {
       D(("Cloning type failed.\n"));
-      D_RETURN_(FALSE);      
+      D_RETURN_(FALSE);
     }
-  
+
   if (src->val_attrs &&
       ! (dst->val_attrs = (BroRecordVal *) __bro_sobject_copy((BroSObject *) src->val_attrs)))
     {
@@ -725,18 +772,20 @@ __bro_val_clone(BroVal *dst, BroVal *src)
     {
     case BRO_INTTYPE_INT:
     case BRO_INTTYPE_UNSIGNED:
-    case BRO_INTTYPE_IPADDR:
       /* Hack for ports */
       if (src->val_type->tag == BRO_TYPE_PORT)
-	dst->val_port = src->val_port;
+        dst->val_port = src->val_port;
       else
-	dst->val_int = src->val_int;
+        dst->val_int64 = src->val_int64;
       break;
-      
+    case BRO_INTTYPE_IPADDR:
+	  dst->val_addr = src->val_addr;
+      break;
+
     case BRO_INTTYPE_DOUBLE:
       dst->val_double = src->val_double;
       break;
-      
+
     case BRO_INTTYPE_STRING:
       bro_string_assign(&src->val_str, &dst->val_str);
       break;
@@ -748,12 +797,12 @@ __bro_val_clone(BroVal *dst, BroVal *src)
     case BRO_INTTYPE_OTHER:
       /* That's okay, handled in subtype */
       break;
-      
+
     default:
       D(("Unknown internal type tag: %i\n", dst->val_type->internal_tag));
       D_RETURN_(FALSE);
-    }  
-  
+    }
+
   D_RETURN_(TRUE);
 }
 
@@ -761,6 +810,7 @@ static uint32
 __bro_val_hash(BroVal *val)
 {
   uint32 result;
+  int i;
 
   D_ENTER;
 
@@ -768,36 +818,41 @@ __bro_val_hash(BroVal *val)
     D_RETURN_(0);
 
   result = __bro_sobject_hash((BroSObject*) val->val_type);
-  
+
   switch (val->val_type->internal_tag)
     {
     case BRO_INTTYPE_INT:
     case BRO_INTTYPE_UNSIGNED:
-    case BRO_INTTYPE_IPADDR:
-      result ^= val->val_int;
+      result ^= val->val_int64;
       break;
-      
+    case BRO_INTTYPE_IPADDR:
+      for ( i = 0; i < 4; ++i )
+        result ^= val->val_addr.addr[i];
+      break;
+
     case BRO_INTTYPE_DOUBLE:
       result ^= (uint32) val->val_double;
       break;
-      
+
     case BRO_INTTYPE_STRING:
       result ^= __bro_ht_str_hash(val->val_str.str_val);
       break;
-      
+
     case BRO_INTTYPE_SUBNET:
-      result ^= val->val_subnet.sn_net;
+      for ( i = 0; i < 4; ++i )
+        result ^= val->val_subnet.sn_net.addr[i];
+
       result ^= val->val_subnet.sn_width;
       break;
 
     case BRO_INTTYPE_OTHER:
       D(("WARNING -- __bro_val_hash() invoked on derived type.\n"));
       break;
-      
+
     default:
       D(("Unknown internal type tag: %i\n", val->val_type->internal_tag));
       break;
-    }  
+    }
 
   D_RETURN_(result);
 }
@@ -805,49 +860,57 @@ __bro_val_hash(BroVal *val)
 static int
 __bro_val_cmp(BroVal *val1, BroVal *val2)
 {
+  int i;
   D_ENTER;
-  
+
   if (! val1 || ! val2)
     D_RETURN_(FALSE);
 
   if (! __bro_sobject_cmp((BroSObject*) val1->val_type,
 			  (BroSObject*) val2->val_type))
     D_RETURN_(FALSE);
-  
+
   switch (val1->val_type->internal_tag)
     {
     case BRO_INTTYPE_INT:
     case BRO_INTTYPE_UNSIGNED:
-    case BRO_INTTYPE_IPADDR:
-      if (val1->val_int != val2->val_int)
+      if (val1->val_int64 != val2->val_int64)
 	D_RETURN_(FALSE);
       break;
-      
+    case BRO_INTTYPE_IPADDR:
+      for (i = 3; i >= 0; --i)
+        if (val1->val_addr.addr[i] != val2->val_addr.addr[i])
+          D_RETURN_(FALSE);
+      break;
+
     case BRO_INTTYPE_DOUBLE:
       if (val1->val_double != val2->val_double)
 	D_RETURN_(FALSE);
       break;
-      
+
     case BRO_INTTYPE_STRING:
       if (! __bro_ht_str_cmp(val1->val_str.str_val, val2->val_str.str_val))
 	D_RETURN_(FALSE);
       break;
-      
+
     case BRO_INTTYPE_SUBNET:
-      if (val1->val_subnet.sn_net != val2->val_subnet.sn_net ||
-	  val1->val_subnet.sn_width != val2->val_subnet.sn_width)
-	D_RETURN_(FALSE);
+      for (i = 3; i >= 0; --i)
+        if (val1->val_subnet.sn_net.addr[i] != val2->val_subnet.sn_net.addr[i])
+          D_RETURN_(FALSE);
+
+      if (val1->val_subnet.sn_width != val2->val_subnet.sn_width)
+        D_RETURN_(FALSE);
       break;
 
     case BRO_INTTYPE_OTHER:
       D(("WARNING -- __bro_val_cmp() invoked on derived type.\n"));
       break;
-      
+
     default:
       D(("Unknown internal type tag: %i\n", val1->val_type->internal_tag));
       break;
-    }  
-  
+    }
+
   D_RETURN_(TRUE);
 }
 
@@ -857,7 +920,7 @@ __bro_val_get(BroVal *val)
   /* Following the comments in broccoli.h, we return atomic values
    * as copies into *result, and complex types (i.e., structs) have
    * all members assigned to point to internal values so the user
-   * does not have to clean up the returned value. The user can 
+   * does not have to clean up the returned value. The user can
    * still keep those values around if necessary by copying them.
    */
   if (! val->val_type)
@@ -865,32 +928,32 @@ __bro_val_get(BroVal *val)
       D(("No type in val %p\n", val));
       return NULL;
     }
-  
-   switch (val->val_type->tag)    
+
+   switch (val->val_type->tag)
     {
     case BRO_TYPE_BOOL:
     case BRO_TYPE_INT:
     case BRO_TYPE_ENUM:
     case BRO_TYPE_COUNT:
     case BRO_TYPE_COUNTER:
+      return &val->val_int64;
     case BRO_TYPE_IPADDR:
-    case BRO_TYPE_NET:
-      return &val->val_int;
-      
+      return &val->val_addr;
+
     case BRO_TYPE_PORT:
       return &val->val_port;
-      
+
     case BRO_TYPE_DOUBLE:
     case BRO_TYPE_TIME:
     case BRO_TYPE_INTERVAL:
       return &val->val_double;
-      
+
     case BRO_TYPE_STRING:
       return &val->val_str;
-      
+
     case BRO_TYPE_SUBNET:
       return &val->val_subnet;
-      
+
     case BRO_TYPE_RECORD:
       D(("WARNING: Inheritance broken -- record types should not be handled here.\n"));
       return NULL;
@@ -898,11 +961,11 @@ __bro_val_get(BroVal *val)
     case BRO_TYPE_TABLE:
       D(("WARNING: Inheritance broken -- table types should not be handled here.\n"));
       return NULL;
-      
+
     default:
       D(("Type %i currently not extractable.\n", val->val_type->tag));
     }
-   
+
    return NULL;
 }
 
@@ -918,7 +981,7 @@ __bro_list_val_new(void)
     D_RETURN_(NULL);
 
   __bro_list_val_init(val);
-  
+
   D_RETURN_(val);
 }
 
@@ -927,18 +990,18 @@ __bro_list_val_init(BroListVal *lv)
 {
   BroSObject *sobj = (BroSObject *) lv;
   BroVal *val = (BroVal *) lv;
-  
+
   D_ENTER;
-  
+
   __bro_val_init((BroVal *) lv);
-  
+
   sobj->read  = (BroSObjectRead) __bro_list_val_read;
   sobj->write = (BroSObjectWrite) __bro_list_val_write;
   sobj->free  = (BroSObjectFree) __bro_list_val_free;
   sobj->clone = (BroSObjectClone) __bro_list_val_clone;
   sobj->hash  = (BroSObjectHash) __bro_list_val_hash;
   sobj->cmp   = (BroSObjectCmp) __bro_list_val_cmp;
-  
+
   sobj->type_id = SER_LIST_VAL;
 
   val->get_data = (BroValAccessor) __bro_list_val_get;
@@ -950,13 +1013,13 @@ static void
 __bro_list_val_free(BroListVal *lv)
 {
   D_ENTER;
-  
+
   if (! lv)
     D_RETURN;
-  
+
   __bro_list_free(lv->list, (BroFunc) __bro_sobject_release);
   __bro_val_free((BroVal *) lv);
-  
+
   D_RETURN;
 }
 
@@ -967,32 +1030,32 @@ __bro_list_val_read(BroListVal *lv, BroConn *bc)
   uint32 ui;
 
   D_ENTER;
-  
+
   if (! __bro_val_read((BroVal *) lv, bc))
     D_RETURN_(FALSE);
-  
+
   __bro_list_free(lv->list, (BroFunc) __bro_sobject_release);
   lv->list = NULL;
-  
+
   if (! __bro_buf_read_char(bc->rx_buf, &lv->type_tag))
     goto error_return;
   if (! __bro_buf_read_int(bc->rx_buf, &ui))
     goto error_return;
-  
+
   lv->len = (int) ui;
 
   for (i = 0; i < lv->len; i++)
     {
       BroVal *val;
-      
+
       if (! (val = (BroVal *) __bro_sobject_unserialize(SER_IS_VAL, bc)))
 	goto error_return;
-      
+
       lv->list = __bro_list_append(lv->list, val);
     }
-  
+
   D_RETURN_(TRUE);
-  
+
  error_return:
   __bro_list_free(lv->list, (BroFunc) __bro_sobject_release);
   lv->list = NULL;
@@ -1003,26 +1066,26 @@ static int
 __bro_list_val_write(BroListVal *lv, BroConn *bc)
 {
   BroList *l;
-  
+
   D_ENTER;
-  
+
   if (! __bro_val_write((BroVal *) lv, bc))
     D_RETURN_(FALSE);
-  
+
   if (! __bro_buf_write_char(bc->tx_buf, lv->type_tag))
     D_RETURN_(FALSE);
-  
+
   if (! __bro_buf_write_int(bc->tx_buf, lv->len))
     D_RETURN_(FALSE);
-  
+
   for (l = lv->list; l; l = __bro_list_next(l))
     {
       BroVal *val = __bro_list_data(l);
-      
+
       if (! __bro_sobject_serialize((BroSObject *) val, bc))
 	D_RETURN_(FALSE);
     }
-  
+
   D_RETURN_(TRUE);
 }
 
@@ -1038,7 +1101,7 @@ __bro_list_val_clone(BroListVal *dst, BroListVal *src)
 
   dst->type_tag = src->type_tag;
   dst->len = src->len;
-  
+
   if (dst->list)
     {
       __bro_list_free(dst->list, (BroFunc) __bro_sobject_release);
@@ -1047,7 +1110,7 @@ __bro_list_val_clone(BroListVal *dst, BroListVal *src)
 
   for (l = src->list; l; l = __bro_list_next(l))
     dst->list = __bro_list_append(dst->list, __bro_sobject_copy(__bro_list_data(l)));
-  
+
   D_RETURN_(TRUE);
 }
 
@@ -1058,15 +1121,15 @@ __bro_list_val_hash(BroListVal *lv)
   BroList *l;
 
   D_ENTER;
-  
+
   if (! lv)
     D_RETURN_(0);
-  
+
   result = lv->len ^ lv->type_tag;
-  
+
   for (l = lv->list; l; l = __bro_list_next(l))
     result ^= __bro_sobject_hash((BroSObject *) __bro_list_data(l));
-  
+
   D_RETURN_(result);
 }
 
@@ -1076,14 +1139,14 @@ __bro_list_val_cmp(BroListVal *lv1, BroListVal *lv2)
   BroList *l1, *l2;
 
   D_ENTER;
-  
+
   if (! lv1 || ! lv2)
     D_RETURN_(FALSE);
 
   if (lv1->len != lv2->len ||
-      lv1->type_tag != lv2->type_tag)    
+      lv1->type_tag != lv2->type_tag)
     D_RETURN_(FALSE);
-      
+
   for (l1 = lv1->list, l2 = lv2->list; l1 && l2;
        l1 = __bro_list_next(l1), l2 = __bro_list_next(l2))
     {
@@ -1097,7 +1160,7 @@ __bro_list_val_cmp(BroListVal *lv1, BroListVal *lv2)
       D(("WARNING -- list length inconsistency.\n"));
       D_RETURN_(FALSE);
     }
-  
+
   D_RETURN_(TRUE);
 }
 
@@ -1132,13 +1195,10 @@ __bro_list_val_pop_front(BroListVal *lv)
 
   if (! lv)
     D_RETURN_(NULL);
-  
-  l = lv->list;
+
+  result = (BroVal*) __bro_list_data(lv->list);
   lv->list = __bro_list_remove(lv->list, lv->list);
-  
-  result = (BroVal*) __bro_list_data(l);
-  __bro_list_free(l, NULL);
-  
+
   D_RETURN_(result);
 }
 
@@ -1152,7 +1212,7 @@ __bro_list_val_get_front(BroListVal *lv)
 
   if (! lv)
     D_RETURN_(NULL);
-  
+
   D_RETURN_((BroVal*) __bro_list_data(lv->list));
 }
 
@@ -1160,10 +1220,10 @@ int
 __bro_list_val_get_length(BroListVal *lv)
 {
   D_ENTER;
-  
+
   if (! lv)
     D_RETURN_(0);
-  
+
   D_RETURN_(lv->len);
 }
 
@@ -1189,18 +1249,18 @@ static void
 __bro_mutable_val_init(BroMutableVal *mv)
 {
   BroSObject *sobj = (BroSObject *) mv;
-  
+
   D_ENTER;
-  
+
   __bro_val_init((BroVal *) mv);
-  
+
   sobj->read  = (BroSObjectRead) __bro_mutable_val_read;
   sobj->write = (BroSObjectWrite) __bro_mutable_val_write;
   sobj->free  = (BroSObjectFree) __bro_mutable_val_free;
   sobj->clone = (BroSObjectClone) __bro_mutable_val_clone;
   sobj->hash  = (BroSObjectHash) __bro_mutable_val_hash;
   sobj->cmp   = (BroSObjectCmp) __bro_mutable_val_cmp;
-  
+
   sobj->type_id = SER_MUTABLE_VAL;
 
   /* BroMutableVal inherits __bro_val_get and doesn't override it. */
@@ -1213,10 +1273,10 @@ static void
 __bro_mutable_val_free(BroMutableVal *mv)
 {
   D_ENTER;
-  
+
   __bro_sobject_release((BroSObject *) mv->id);
   __bro_val_free((BroVal *) mv);
-  
+
   D_RETURN;
 }
 
@@ -1250,16 +1310,16 @@ static int
 __bro_mutable_val_write(BroMutableVal *mv, BroConn *bc)
 {
   D_ENTER;
-  
+
   if (! __bro_val_write((BroVal *) mv, bc))
     D_RETURN_(FALSE);
-  
+
   if (! __bro_buf_write_char(bc->tx_buf, mv->props))
     D_RETURN_(FALSE);
-  
+
   if (! __bro_buf_write_string(bc->tx_buf, (mv->id ? &mv->id->name : NULL)))
     D_RETURN_(FALSE);
-  
+
   D_RETURN_(TRUE);
 }
 
@@ -1268,15 +1328,15 @@ static int
 __bro_mutable_val_clone(BroMutableVal *dst, BroMutableVal *src)
 {
   D_ENTER;
-  
+
   if (! __bro_val_clone((BroVal *) dst, (BroVal *) src))
     D_RETURN_(FALSE);
-  
+
   if (src->id && ! (dst->id = (BroID *) __bro_sobject_copy((BroSObject *) src->id)))
     D_RETURN_(FALSE);
-  
+
   src->props = dst->props;
-  
+
   D_RETURN_(TRUE);
 }
 
@@ -1338,9 +1398,9 @@ __bro_record_val_init(BroRecordVal *rv)
   BroVal *val = (BroVal *) rv;
 
   D_ENTER;
-  
+
   __bro_mutable_val_init((BroMutableVal *) rv);
-  
+
   sobj->read  = (BroSObjectRead) __bro_record_val_read;
   sobj->write = (BroSObjectWrite) __bro_record_val_write;
   sobj->free  = (BroSObjectFree) __bro_record_val_free;
@@ -1351,7 +1411,7 @@ __bro_record_val_init(BroRecordVal *rv)
   sobj->type_id = SER_RECORD_VAL;
 
   val->get_data = (BroValAccessor) __bro_record_val_get;
-  
+
   D_RETURN;
 }
 
@@ -1363,7 +1423,7 @@ __bro_record_val_free(BroRecordVal *rv)
 
   if (! rv)
     D_RETURN;
-  
+
   __bro_record_free(rv->rec);
   __bro_mutable_val_free((BroMutableVal *) rv);
 
@@ -1377,23 +1437,23 @@ __bro_record_val_read(BroRecordVal *rv, BroConn *bc)
   char opt;
   uint32 i, len;
   BroVal *val;
-  
+
   D_ENTER;
-  
+
   if (! __bro_mutable_val_read((BroMutableVal *) rv, bc))
     D_RETURN_(FALSE);
 
   /* Clean out old vals, if any */
   __bro_record_free(rv->rec);
-  
+
   if (! (rv->rec = __bro_record_new()))
     D_RETURN_(FALSE);
-  
+
   /* Read in new vals */
 
   if (! __bro_buf_read_int(bc->rx_buf, &len))
     goto error_return;
-  
+
   for (i = 0; i < len; i++)
     {
       const char *field_name;
@@ -1405,7 +1465,7 @@ __bro_record_val_read(BroRecordVal *rv, BroConn *bc)
 
       if (! __bro_buf_read_char(bc->rx_buf, &opt))
 	goto error_return;
-      
+
       if (opt)
 	{
 	  if (! (val = (BroVal *) __bro_sobject_unserialize(SER_IS_VAL, bc)))
@@ -1424,9 +1484,9 @@ __bro_record_val_read(BroRecordVal *rv, BroConn *bc)
 	  if (! (val = __bro_val_new()))
 	    goto error_return;
 	}
-      
+
       __bro_record_add_val(rv->rec, val);
-      
+
       if (! (field_name = __bro_record_type_get_nth_field((BroRecordType *) rv_type, i)))
 	{
 	  D(("WARNING -- record type field %i has no name.\n", i));
@@ -1435,7 +1495,7 @@ __bro_record_val_read(BroRecordVal *rv, BroConn *bc)
 
       __bro_record_set_nth_name(rv->rec, i, field_name);
     }
-  
+
   D_RETURN_(TRUE);
 
  error_return:
@@ -1454,33 +1514,33 @@ __bro_record_val_write(BroRecordVal *rv, BroConn *bc)
 
   D_ENTER;
 
+  if (! rv->rec)
+    D_RETURN_(FALSE);
+
   if (! __bro_mutable_val_write((BroMutableVal *) rv, bc))
     D_RETURN_(FALSE);
-  
+
   if (! __bro_buf_write_int(bc->tx_buf, rv->rec->val_len))
     D_RETURN_(FALSE);
-  
-  if (! rv->rec && rv->rec->val_len > 0)
-    D_RETURN_(FALSE);
-  
+
   D(("Writing out %i vals in record %p.\n", rv->rec->val_len, rv->rec));
 
   for (i = 0, l = rv->rec->val_list; l; i++, l = __bro_list_next(l))
     {
       val = __bro_list_data(l);
-      
+
       D(("Val %i/%p's type: %p\n", i, val, val->val_type));
-      
+
       if (! __bro_buf_write_char(bc->tx_buf, (val->val_type ? 1 :0)))
 	D_RETURN_(FALSE);
-      
+
       if (val->val_type)
-	{	  
+	{
 	  if (! __bro_sobject_serialize((BroSObject *) val, bc))
 	    D_RETURN_(FALSE);
 	}
     }
-  
+
   D_RETURN_(TRUE);
 }
 
@@ -1489,13 +1549,13 @@ static int
 __bro_record_val_clone(BroRecordVal *dst, BroRecordVal *src)
 {
   D_ENTER;
-  
+
   if (! __bro_mutable_val_clone((BroMutableVal *) dst, (BroMutableVal *) src))
     D_RETURN_(FALSE);
-  
+
   if (src->rec && ! (dst->rec = __bro_record_copy(src->rec)))
     D_RETURN_(FALSE);
-  
+
   D_RETURN_(TRUE);
 }
 
@@ -1504,9 +1564,9 @@ static uint32
 __bro_record_val_hash(BroRecordVal *rv)
 {
   uint32 result;
-  
+
   D_ENTER;
-  
+
   if (! rv)
     D_RETURN_(0);
 
@@ -1524,7 +1584,7 @@ __bro_record_val_cmp(BroRecordVal *rv1, BroRecordVal *rv2)
 
   if (! rv1 || ! rv2)
     D_RETURN_(FALSE);
-  
+
   if (! __bro_record_cmp(rv1->rec, rv2->rec))
     D_RETURN_(FALSE);
 
@@ -1546,12 +1606,12 @@ __bro_table_val_new(void)
   BroTableVal *val;
 
   D_ENTER;
-  
+
   if (! (val = calloc(1, sizeof(BroTableVal))))
     D_RETURN_(NULL);
 
   __bro_table_val_init(val);
-  
+
   D_RETURN_(val);
 }
 
@@ -1562,19 +1622,19 @@ __bro_table_val_init(BroTableVal *tbl)
   BroVal *val = (BroVal *) tbl;
 
   D_ENTER;
-  
+
   __bro_mutable_val_init((BroMutableVal *) tbl);
-  
+
   sobj->read  = (BroSObjectRead) __bro_table_val_read;
   sobj->write = (BroSObjectWrite) __bro_table_val_write;
   sobj->free  = (BroSObjectFree) __bro_table_val_free;
   sobj->clone = (BroSObjectClone) __bro_table_val_clone;
   sobj->hash  = (BroSObjectHash) __bro_table_val_hash;
   sobj->cmp   = (BroSObjectCmp) __bro_table_val_cmp;
-  
+
   sobj->type_id = SER_TABLE_VAL;
-  
-  val->get_data = (BroValAccessor) __bro_table_val_get;  
+
+  val->get_data = (BroValAccessor) __bro_table_val_get;
 
   D_RETURN;
 }
@@ -1583,13 +1643,13 @@ static void
 __bro_table_val_free(BroTableVal *tbl)
 {
   D_ENTER;
-  
+
   if (! tbl)
     D_RETURN;
-  
+
   __bro_table_free(tbl->table);
   __bro_mutable_val_free((BroMutableVal *) tbl);
-  
+
   D_RETURN;
 }
 
@@ -1601,16 +1661,16 @@ __bro_table_val_read(BroTableVal *tbl, BroConn *bc)
   int num_keys = 0, num_vals = 0;
 
   D_ENTER;
-  
+
   if (! __bro_mutable_val_read((BroMutableVal *) tbl, bc))
     D_RETURN_(FALSE);
-  
+
   /* Clean out old vals, if any */
   __bro_table_free(tbl->table);
-  
+
   if (! (tbl->table = __bro_table_new()))
     D_RETURN_(FALSE);
-  
+
   /* expire_time, currently unused */
   if (! __bro_buf_read_double(bc->rx_buf, &d))
     goto error_return;
@@ -1625,7 +1685,7 @@ __bro_table_val_read(BroTableVal *tbl, BroConn *bc)
 	  goto error_return;
 	}
     }
-  
+
   if (! __bro_buf_read_char(bc->rx_buf, &opt))
     goto error_return;
   if (opt)
@@ -1679,7 +1739,7 @@ __bro_table_val_read(BroTableVal *tbl, BroConn *bc)
 	key_type = __bro_list_val_get_front(keys)->val_type->tag;
       else
 	goto error_return;
-      
+
       if (tbl->table->tbl_key_type != BRO_TYPE_UNKNOWN &&
 	  tbl->table->tbl_key_type != key_type)
 	{
@@ -1689,7 +1749,7 @@ __bro_table_val_read(BroTableVal *tbl, BroConn *bc)
 	}
 
       tbl->table->tbl_key_type = key_type;
-      
+
       if (tbl->table->tbl_val_type != BRO_TYPE_UNKNOWN &&
 	  tbl->table->tbl_val_type != val_type)
 	{
@@ -1698,8 +1758,8 @@ __bro_table_val_read(BroTableVal *tbl, BroConn *bc)
 	  goto error_return;
 	}
 
-      tbl->table->tbl_val_type = val_type;	
-      
+      tbl->table->tbl_val_type = val_type;
+
       /* Eat two doubles -- one for the last access time and
        * one for when the item is supposed to expire.
        * XXX: currently unimplemented.
@@ -1730,32 +1790,38 @@ __bro_table_val_write_cb_direct(BroVal *key, BroVal *val, BroConn *bc)
 {
   if (! __bro_sobject_serialize((BroSObject *) key, bc))
     return FALSE;
-  
+
   if (val && ! __bro_sobject_serialize((BroSObject *) val, bc))
     return FALSE;
-  
+
   return TRUE;
 }
 
 static int
 __bro_table_val_write_cb_unpack(BroVal *key, BroRecordVal *val, BroConn *bc)
 {
-  BroRecord *rec = val->rec;
-  BroListVal *lv = __bro_list_val_new();
-  
+  BroRecord *rec = 0;
+  BroListVal *lv = 0;
+
+  if ( ! val )
+    return FALSE;
+
+  rec = val->rec;
+  lv = __bro_list_val_new();
+
   /* Just hook the list into the list val, we unhook below. */
   lv->list = rec->val_list;
   lv->len = rec->val_len;
-  
+
   if (! __bro_sobject_serialize((BroSObject *) lv, bc))
     goto error_return;
-  
-  if (val && ! __bro_sobject_serialize((BroSObject *) val, bc))
+
+  if (! __bro_sobject_serialize((BroSObject *) val, bc))
     goto error_return;
-  
+
   lv->list = NULL;
   __bro_list_val_free(lv);
-  
+
   return TRUE;
 
  error_return:
@@ -1807,7 +1873,7 @@ __bro_table_val_clone(BroTableVal *dst, BroTableVal *src)
 
   if (src->table && ! (dst->table = __bro_table_copy(src->table)))
     D_RETURN_(FALSE);
-  
+
   D_RETURN_(TRUE);
 }
 
@@ -1817,14 +1883,14 @@ __bro_table_val_hash(BroTableVal *tv)
   uint32 result;
 
   D_ENTER;
-  
+
   if (! tv)
     D_RETURN_(0);
-  
+
   result = __bro_sobject_hash((BroSObject*) tv->table_type);
   result ^= __bro_sobject_hash((BroSObject*) tv->attrs);
   result ^= __bro_table_hash(tv->table);
-  
+
   D_RETURN_(result);
 
 }
@@ -1840,10 +1906,10 @@ __bro_table_val_cmp(BroTableVal *tv1, BroTableVal *tv2)
   if (! __bro_sobject_cmp((BroSObject*) tv1->table_type,
 			  (BroSObject*) tv2->table_type))
     D_RETURN_(FALSE);
-  
+
   if (! __bro_table_cmp(tv1->table, tv2->table))
     D_RETURN_(FALSE);
-  
+
   D_RETURN_(TRUE);
 }
 
@@ -1856,7 +1922,7 @@ __bro_table_val_get(BroTableVal *tbl)
 int
 __bro_table_val_has_atomic_key(BroTableVal *tbl)
 {
-  if (! tbl || ! tbl->table_type) 
+  if (! tbl || ! tbl->table_type)
     return FALSE;
 
   return ((BroIndexType *) tbl->table_type)->indices->num_types == 1;
