@@ -1,5 +1,3 @@
-// $Id: ChunkedIO.cc 6888 2009-08-20 18:23:11Z vern $
-
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -7,15 +5,17 @@
 #include <sys/time.h>
 #include <netinet/in.h>
 #include <assert.h>
+#include <openssl/ssl.h>
+
+#include <algorithm>
 
 #include "config.h"
 #include "ChunkedIO.h"
 #include "NetVar.h"
 #include "RemoteSerializer.h"
 
-ChunkedIO::ChunkedIO()
+ChunkedIO::ChunkedIO() : stats(), tag(), pure()
 	{
-	pure = false;
 	}
 
 void ChunkedIO::Stats(char* buffer, int length)
@@ -75,7 +75,7 @@ void ChunkedIO::DumpDebugData(const char* basefnname, bool want_reads)
 		ChunkedIOFd io(fd, "dump-file");
 		io.Write(*i);
 		io.Flush();
-		close(fd);
+		safe_close(fd);
 		}
 
 	l->clear();
@@ -126,7 +126,7 @@ ChunkedIOFd::~ChunkedIOFd()
 
 	delete [] read_buffer;
 	delete [] write_buffer;
-	close(fd);
+	safe_close(fd);
 
 	if ( partial )
 		{
@@ -139,7 +139,7 @@ bool ChunkedIOFd::Write(Chunk* chunk)
 	{
 #ifdef DEBUG
 	DBG_LOG(DBG_CHUNKEDIO, "write of size %d [%s]",
-		chunk->len, fmt_bytes(chunk->data, min(20, chunk->len)));
+		chunk->len, fmt_bytes(chunk->data, min((uint32)20, chunk->len)));
 #endif
 
 	// Reject if our queue of pending chunks is way too large. Otherwise,
@@ -165,13 +165,13 @@ bool ChunkedIOFd::Write(Chunk* chunk)
 
 	// We have to split it up.
 	char* p = chunk->data;
-	unsigned long left = chunk->len;
+	uint32 left = chunk->len;
 
 	while ( left )
 		{
 		Chunk* part = new Chunk;
 
-		part->len = min(BUFFER_SIZE - sizeof(uint32), left);
+		part->len = min<uint32>(BUFFER_SIZE - sizeof(uint32), left);
 		part->data = new char[part->len];
 		memcpy(part->data, p, part->len);
 		left -= part->len;
@@ -192,7 +192,7 @@ bool ChunkedIOFd::WriteChunk(Chunk* chunk, bool partial)
 	assert(chunk->len <= BUFFER_SIZE - sizeof(uint32) );
 
 	if ( chunk->len == 0 )
-		internal_error( "attempt to write 0 bytes chunk");
+		InternalError("attempt to write 0 bytes chunk");
 
 	if ( partial )
 		chunk->len |= FLAG_PARTIAL;
@@ -282,7 +282,7 @@ bool ChunkedIOFd::FlushWriteBuffer()
 			}
 
 		if ( written == 0 )
-			internal_error("written==0");
+			InternalError("written==0");
 
 		// Short write.
 		write_pos += written;
@@ -426,7 +426,7 @@ bool ChunkedIOFd::Read(Chunk** chunk, bool may_block)
 				(*chunk)->len & ~FLAG_PARTIAL,
 				(*chunk)->len & FLAG_PARTIAL ? "(P) " : "",
 				fmt_bytes((*chunk)->data,
-						min(20, (*chunk)->len)));
+						min((uint32)20, (*chunk)->len)));
 #endif
 
 	if ( ! ((*chunk)->len & FLAG_PARTIAL) )
@@ -650,16 +650,12 @@ void ChunkedIOFd::Stats(char* buffer, int length)
 	ChunkedIO::Stats(buffer + i, length - i);
 	}
 
-
-#ifdef USE_OPENSSL
-
-#include <openssl/ssl.h>
-
 SSL_CTX* ChunkedIOSSL::ctx;
 
 ChunkedIOSSL::ChunkedIOSSL(int arg_socket, bool arg_server)
 	{
 	socket = arg_socket;
+	last_ret = 0;
 	eof = false;
 	setup = false;
 	server = arg_server;
@@ -690,7 +686,7 @@ ChunkedIOSSL::~ChunkedIOSSL()
 		ssl = 0;
 		}
 
-	close(socket);
+	safe_close(socket);
 	}
 
 
@@ -908,7 +904,7 @@ bool ChunkedIOSSL::WriteData(char* p, uint32 len, bool* error)
 			return false;
 	}
 
-	internal_error("can't be reached");
+	InternalError("can't be reached");
 	return false;
 	}
 
@@ -1028,7 +1024,7 @@ bool ChunkedIOSSL::ReadData(char* p, uint32 len, bool* error)
 		}
 
 	// Can't be reached.
-	internal_error("can't be reached");
+	InternalError("can't be reached");
 	return false;
 	}
 
@@ -1078,9 +1074,8 @@ bool ChunkedIOSSL::Read(Chunk** chunk, bool mayblock)
 	read_state = LEN;
 
 #ifdef DEBUG
-	if ( *chunk )
-		DBG_LOG(DBG_CHUNKEDIO, "ssl read of size %d [%s]",
-			(*chunk)->len, fmt_bytes((*chunk)->data, 20));
+	DBG_LOG(DBG_CHUNKEDIO, "ssl read of size %d [%s]",
+		(*chunk)->len, fmt_bytes((*chunk)->data, 20));
 #endif
 
 	return true;
@@ -1174,10 +1169,6 @@ void ChunkedIOSSL::Stats(char* buffer, int length)
 	ChunkedIO::Stats(buffer + i, length - i);
 	}
 
-#endif	/* USE_OPENSSL */
-
-#ifdef HAVE_LIBZ
-
 bool CompressedChunkedIO::Init()
 	{
 	zin.zalloc = 0;
@@ -1354,5 +1345,3 @@ void CompressedChunkedIO::Stats(char* buffer, int length)
 	io->Stats(buffer + i, length - i);
 	buffer[length-1] = '\0';
 	}
-
-#endif	/* HAVE_LIBZ */
