@@ -1,5 +1,3 @@
-// $Id: ChunkedIO.h 6888 2009-08-20 18:23:11Z vern $
-//
 // Implements non-blocking chunk-wise I/O.
 
 #ifndef CHUNKEDIO_H
@@ -11,6 +9,13 @@
 
 #include <list>
 
+#ifdef NEED_KRB5_H
+# include <krb5.h>
+#endif 
+
+#include <openssl/ssl.h>
+#include <openssl/err.h>
+
 class CompressedChunkedIO;
 
 // #define DEBUG_COMMUNICATION 10
@@ -113,6 +118,11 @@ public:
 #endif
 
 protected:
+	void InternalError(const char* msg)
+		// We can't use the reporter here as we might be running in a
+		// sub-process.
+		{ fprintf(stderr, "%s", msg); abort(); }
+
 	Statistics stats;
 	const char* tag;
 
@@ -214,17 +224,7 @@ private:
 	pid_t pid;
 };
 
-#ifdef USE_OPENSSL
-
-#ifdef NEED_KRB5_H
-# include <krb5.h>
-#endif 
-
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-
 // Chunked I/O using an SSL connection.
-
 class ChunkedIOSSL : public ChunkedIO {
 public:
 	// Argument is an open socket and a flag indicating whether we are the
@@ -287,17 +287,14 @@ private:
 	static SSL_CTX* ctx;
 };
 
-#endif	/* USE_OPENSSL */
-
-#ifdef HAVE_LIBZ
-
 #include <zlib.h>
 
 // Wrapper class around a another ChunkedIO which the (un-)compresses data.
 class CompressedChunkedIO : public ChunkedIO {
 public:
-	CompressedChunkedIO(ChunkedIO* arg_io)
-		: io(arg_io) {} // takes ownership
+	CompressedChunkedIO(ChunkedIO* arg_io) // takes ownership
+	    : io(arg_io), zin(), zout(), error(), compress(), uncompress(),
+	      uncompressed_bytes_read(), uncompressed_bytes_written() {}
 	virtual ~CompressedChunkedIO()	{ delete io; }
 
 	virtual bool Init(); // does *not* call arg_io->Init()
@@ -337,6 +334,4 @@ protected:
 	unsigned long uncompressed_bytes_written;
 };
 
-#endif	/* HAVE_LIBZ */
-
 #endif
