@@ -1,16 +1,14 @@
-// $Id: DNS_Mgr.cc 7073 2010-09-13 00:45:02Z vern $
-//
 // See the file "COPYING" in the main distribution directory for copyright.
 
 #include "config.h"
 
 #include <sys/types.h>
 #include <sys/socket.h>
-#if TIME_WITH_SYS_TIME
+#ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
 #else
-# if HAVE_SYS_TIME_H
+# ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>
 # else
 #  include <time.h>
@@ -29,10 +27,13 @@
 #endif
 #include <stdlib.h>
 
+#include <algorithm>
+
 #include "DNS_Mgr.h"
 #include "Event.h"
 #include "Net.h"
 #include "Var.h"
+#include "Reporter.h"
 
 extern "C" {
 extern int select(int, fd_set *, fd_set *, fd_set *, struct timeval *);
@@ -45,28 +46,35 @@ extern int select(int, fd_set *, fd_set *, fd_set *, struct timeval *);
 
 class DNS_Mgr_Request {
 public:
-	DNS_Mgr_Request(const char* h)	{ host = copy_string(h); addr = 0; }
-	DNS_Mgr_Request(uint32 a)		{ addr = a; host = 0; }
+	DNS_Mgr_Request(const char* h, int af, bool is_txt)
+	    : host(copy_string(h)), fam(af), qtype(is_txt ? 16 : 0), addr(),
+	      request_pending()
+		{ }
+
+	DNS_Mgr_Request(const IPAddr& a)
+	    : host(), fam(), qtype(), addr(a), request_pending()
+		{ }
+
 	~DNS_Mgr_Request()			{ delete [] host; }
 
 	// Returns nil if this was an address request.
 	const char* ReqHost() const	{ return host; }
-	uint32 ReqAddr() const		{ return addr; }
+	const IPAddr& ReqAddr() const		{ return addr; }
+	const bool ReqIsTxt() const	{ return qtype == 16; }
 
-#ifdef HAVE_NB_DNS
 	int MakeRequest(nb_dns_info* nb_dns);
-#endif
 	int RequestPending() const	{ return request_pending; }
 	void RequestDone()	{ request_pending = 0; }
 
 
 protected:
 	char* host;	// if non-nil, this is a host request
-	uint32 addr;
+	int fam;	// address family query type for host requests
+	int qtype;	// Query type
+	IPAddr addr;
 	int request_pending;
 };
 
-#ifdef HAVE_NB_DNS
 int DNS_Mgr_Request::MakeRequest(nb_dns_info* nb_dns)
 	{
 	if ( ! nb_dns )
@@ -76,16 +84,20 @@ int DNS_Mgr_Request::MakeRequest(nb_dns_info* nb_dns)
 
 	char err[NB_DNS_ERRSIZE];
 	if ( host )
-		return nb_dns_host_request(nb_dns, host, (void*) this, err) >= 0;
+		return nb_dns_host_request2(nb_dns, host, fam, qtype, (void*) this, err) >= 0;
 	else
-		return nb_dns_addr_request(nb_dns, addr, (void*) this, err) >= 0;
+		{
+		const uint32* bytes;
+		int len = addr.GetBytes(&bytes);
+		return nb_dns_addr_request2(nb_dns, (char*) bytes,
+				len == 1 ? AF_INET : AF_INET6, (void*) this, err) >= 0;
+		}
 	}
-#endif
 
 class DNS_Mapping {
 public:
-	DNS_Mapping(const char* host, struct hostent* h);
-	DNS_Mapping(uint32 addr, struct hostent* h);
+	DNS_Mapping(const char* host, struct hostent* h, uint32 ttl);
+	DNS_Mapping(const IPAddr& addr, struct hostent* h, uint32 ttl);
 	DNS_Mapping(FILE* f);
 
 	int NoMapping() const		{ return no_mapping; }
@@ -95,9 +107,11 @@ public:
 
 	// Returns nil if this was an address request.
 	const char* ReqHost() const	{ return req_host; }
-	uint32 ReqAddr() const		{ return req_addr; }
-	const char* ReqStr() const
-		{ return req_host ? req_host : dotted_addr(ReqAddr());  }
+	IPAddr ReqAddr() const		{ return req_addr; }
+	string ReqStr() const
+		{
+		return req_host ? req_host : req_addr;
+		}
 
 	ListVal* Addrs();
 	TableVal* AddrsSet();	// addresses returned as a set
@@ -110,6 +124,16 @@ public:
 	int Failed() const		{ return failed; }
 	int Valid() const		{ return ! failed; }
 
+	bool Expired() const
+		{
+		if ( req_host && num_addrs == 0)
+			return false; // nothing to expire
+
+		return current_time() > (creation_time + req_ttl);
+		}
+
+	int Type() const { return map_type; }
+
 protected:
 	friend class DNS_Mgr;
 
@@ -120,18 +144,20 @@ protected:
 	int init_failed;
 
 	char* req_host;
-	uint32 req_addr;
+	IPAddr req_addr;
+	uint32 req_ttl;
 
 	int num_names;
 	char** names;
 	StringVal* host_val;
 
 	int num_addrs;
-	uint32* addrs;
+	IPAddr* addrs;
 	ListVal* addrs_val;
 
 	int failed;
 	double creation_time;
+	int map_type;
 };
 
 void DNS_Mgr_mapping_delete_func(void* v)
@@ -148,21 +174,22 @@ static TableVal* empty_addr_set()
 	return new TableVal(s);
 	}
 
-DNS_Mapping::DNS_Mapping(const char* host, struct hostent* h)
+DNS_Mapping::DNS_Mapping(const char* host, struct hostent* h, uint32 ttl)
 	{
 	Init(h);
 	req_host = copy_string(host);
-	req_addr = 0;
+	req_ttl = ttl;
 
 	if ( names && ! names[0] )
 		names[0] = copy_string(host);
 	}
 
-DNS_Mapping::DNS_Mapping(uint32 addr, struct hostent* h)
+DNS_Mapping::DNS_Mapping(const IPAddr& addr, struct hostent* h, uint32 ttl)
 	{
 	Init(h);
 	req_addr = addr;
 	req_host = 0;
+	req_ttl = ttl;
 	}
 
 DNS_Mapping::DNS_Mapping(FILE* f)
@@ -171,7 +198,8 @@ DNS_Mapping::DNS_Mapping(FILE* f)
 	init_failed = 1;
 
 	req_host = 0;
-	req_addr = 0;
+	req_ttl = 0;
+	creation_time = 0;
 
 	char buf[512];
 
@@ -184,14 +212,15 @@ DNS_Mapping::DNS_Mapping(FILE* f)
 	char req_buf[512+1], name_buf[512+1];
 	int is_req_host;
 
-	if ( sscanf(buf, "%lf %d %512s %d %512s %d", &creation_time, &is_req_host,
-		    req_buf, &failed, name_buf, &num_addrs) != 6 )
+	if ( sscanf(buf, "%lf %d %512s %d %512s %d %d %"PRIu32, &creation_time,
+	     &is_req_host, req_buf, &failed, name_buf, &map_type, &num_addrs,
+	     &req_ttl) != 8 )
 		return;
 
 	if ( is_req_host )
 		req_host = copy_string(req_buf);
 	else
-		req_addr = dotted_to_addr(req_buf);
+		req_addr = IPAddr(req_buf);
 
 	num_names = 1;
 	names = new char*[num_names];
@@ -199,7 +228,7 @@ DNS_Mapping::DNS_Mapping(FILE* f)
 
 	if ( num_addrs > 0 )
 		{
-		addrs = new uint32[num_addrs];
+		addrs = new IPAddr[num_addrs];
 
 		for ( int i = 0; i < num_addrs; ++i )
 			{
@@ -213,7 +242,7 @@ DNS_Mapping::DNS_Mapping(FILE* f)
 			if ( newline )
 				*newline = '\0';
 
-			addrs[i] = dotted_to_addr(buf);
+			addrs[i] = IPAddr(buf);
 			}
 		}
 	else
@@ -276,14 +305,6 @@ StringVal* DNS_Mapping::Host()
 	return host_val;
 	}
 
-// Converts an array of 4 bytes in network order to the corresponding
-// 32-bit network long.
-static uint32 raw_bytes_to_addr(const unsigned char b[4])
-	{
-	uint32 l = (b[0] << 24) | (b[1] << 16) | (b[2] << 8) | b[3];
-	return uint32(htonl(l));
-	}
-
 void DNS_Mapping::Init(struct hostent* h)
 	{
 	no_mapping = 0;
@@ -292,12 +313,13 @@ void DNS_Mapping::Init(struct hostent* h)
 	host_val = 0;
 	addrs_val = 0;
 
-	if ( ! h || h->h_addrtype != AF_INET || h->h_length != 4 )
+	if ( ! h )
 		{
 		Clear();
 		return;
 		}
 
+	map_type = h->h_addrtype;
 	num_names = 1;	// for now, just use official name
 	names = new char*[num_names];
 	names[0] = h->h_name ? copy_string(h->h_name) : 0;
@@ -307,10 +329,14 @@ void DNS_Mapping::Init(struct hostent* h)
 
 	if ( num_addrs > 0 )
 		{
-		addrs = new uint32[num_addrs];
+		addrs = new IPAddr[num_addrs];
 		for ( int i = 0; i < num_addrs; ++i )
-			addrs[i] = raw_bytes_to_addr(
-					(unsigned char*)h->h_addr_list[i]);
+			if ( h->h_addrtype == AF_INET )
+				addrs[i] = IPAddr(IPv4, (uint32*)h->h_addr_list[i],
+				                  IPAddr::Network);
+			else if ( h->h_addrtype == AF_INET6 )
+				addrs[i] = IPAddr(IPv6, (uint32*)h->h_addr_list[i],
+				                  IPAddr::Network);
 		}
 	else
 		addrs = 0;
@@ -326,18 +352,19 @@ void DNS_Mapping::Clear()
 	host_val = 0;
 	addrs_val = 0;
 	no_mapping = 0;
+	map_type = 0;
 	failed = 1;
 	}
 
 void DNS_Mapping::Save(FILE* f) const
 	{
-	fprintf(f, "%.0f %d %s %d %s %d\n", creation_time, req_host != 0,
-		req_host ? req_host : dotted_addr(req_addr),
+	fprintf(f, "%.0f %d %s %d %s %d %d %"PRIu32"\n", creation_time, req_host != 0,
+		req_host ? req_host : req_addr.AsString().c_str(),
 		failed, (names && names[0]) ? names[0] : "*",
-		num_addrs);
+		map_type, num_addrs, req_ttl);
 
 	for ( int i = 0; i < num_addrs; ++i )
-		fprintf(f, "%s\n", dotted_addr(addrs[i]));
+		fprintf(f, "%s\n", addrs[i].AsString().c_str());
 	}
 
 
@@ -347,16 +374,11 @@ DNS_Mgr::DNS_Mgr(DNS_MgrMode arg_mode)
 
 	mode = arg_mode;
 
-	host_mappings.SetDeleteFunc(DNS_Mgr_mapping_delete_func);
-	addr_mappings.SetDeleteFunc(DNS_Mgr_mapping_delete_func);
-
-#ifdef HAVE_NB_DNS
 	char err[NB_DNS_ERRSIZE];
 	nb_dns = nb_dns_init(err);
 
 	if ( ! nb_dns )
-		warn(fmt("problem initializing NB-DNS: %s", err));
-#endif
+		reporter->Warning("problem initializing NB-DNS: %s", err);
 
 	dns_mapping_valid = dns_mapping_unverified = dns_mapping_new_name =
 		dns_mapping_lost_name = dns_mapping_name_changed =
@@ -368,14 +390,15 @@ DNS_Mgr::DNS_Mgr(DNS_MgrMode arg_mode)
 	cache_name = dir = 0;
 
 	asyncs_pending = 0;
+	num_requests = 0;
+	successful = 0;
+	failed = 0;
 	}
 
 DNS_Mgr::~DNS_Mgr()
 	{
-#ifdef HAVE_NB_DNS
 	if ( nb_dns )
 		nb_dns_finish(nb_dns);
-#endif
 
 	delete [] cache_name;
 	delete [] dir;
@@ -410,14 +433,12 @@ bool DNS_Mgr::Init()
 
 	did_init = 1;
 
-#ifdef HAVE_NB_DNS
 	io_sources.Register(this, true);
 
 	// We never set idle to false, having the main loop only calling us from
 	// time to time. If we're issuing more DNS requests than we can handle
 	// in this way, we are having problems anyway ...
 	idle = true;
-#endif
 
 	return true;
 	}
@@ -439,59 +460,71 @@ TableVal* DNS_Mgr::LookupHost(const char* name)
 
 	if ( mode != DNS_PRIME )
 		{
-		DNS_Mapping* d = host_mappings.Lookup(name);
+		HostMap::iterator it = host_mappings.find(name);
 
-		if ( d )
+		if ( it != host_mappings.end() )
 			{
-			if ( d->Valid() )
-				return d->Addrs()->ConvertToSet();
-			else
+			DNS_Mapping* d4 = it->second.first;
+			DNS_Mapping* d6 = it->second.second;
+
+			if ( (d4 && d4->Failed()) || (d6 && d6->Failed()) )
 				{
-				warn("no such host:", name);
+				reporter->Warning("no such host: %s", name);
 				return empty_addr_set();
 				}
+			else if ( d4 && d6 )
+				{
+				TableVal* tv4 = d4->AddrsSet();
+				TableVal* tv6 = d6->AddrsSet();
+				tv4->AddTo(tv6, false);
+				Unref(tv4);
+				return tv6;
+				}
 			}
 		}
 
 	// Not found, or priming.
 	switch ( mode ) {
 	case DNS_PRIME:
-		requests.append(new DNS_Mgr_Request(name));
+		requests.append(new DNS_Mgr_Request(name, AF_INET, false));
+		requests.append(new DNS_Mgr_Request(name, AF_INET6, false));
 		return empty_addr_set();
 
 	case DNS_FORCE:
-		internal_error("can't find DNS entry for %s in cache", name);
+		reporter->FatalError("can't find DNS entry for %s in cache", name);
 		return 0;
 
 	case DNS_DEFAULT:
-		requests.append(new DNS_Mgr_Request(name));
+		requests.append(new DNS_Mgr_Request(name, AF_INET, false));
+		requests.append(new DNS_Mgr_Request(name, AF_INET6, false));
 		Resolve();
 		return LookupHost(name);
 
 	default:
-		internal_error("bad mode in DNS_Mgr::LookupHost");
+		reporter->InternalError("bad mode in DNS_Mgr::LookupHost");
 		return 0;
 	}
 	}
 
-Val* DNS_Mgr::LookupAddr(uint32 addr)
+Val* DNS_Mgr::LookupAddr(const IPAddr& addr)
 	{
 	if ( ! did_init )
 		Init();
 
 	if ( mode != DNS_PRIME )
 		{
-		HashKey h(&addr, 1);
-		DNS_Mapping* d = addr_mappings.Lookup(&h);
+		AddrMap::iterator it = addr_mappings.find(addr);
 
-		if ( d )
+		if ( it != addr_mappings.end() )
 			{
+			DNS_Mapping* d = it->second;
 			if ( d->Valid() )
 				return d->Host();
 			else
 				{
-				warn("can't resolve IP address:", dotted_addr(addr));
-				return new StringVal(dotted_addr(addr));
+				string s(addr);
+				reporter->Warning("can't resolve IP address: %s", s.c_str());
+				return new StringVal(s.c_str());
 				}
 			}
 		}
@@ -503,8 +536,8 @@ Val* DNS_Mgr::LookupAddr(uint32 addr)
 		return new StringVal("<none>");
 
 	case DNS_FORCE:
-		internal_error("can't find DNS entry for %s in cache",
-				dotted_addr(addr));
+		reporter->FatalError("can't find DNS entry for %s in cache",
+		    addr.AsString().c_str());
 		return 0;
 
 	case DNS_DEFAULT:
@@ -513,7 +546,7 @@ Val* DNS_Mgr::LookupAddr(uint32 addr)
 		return LookupAddr(addr);
 
 	default:
-		internal_error("bad mode in DNS_Mgr::LookupHost");
+		reporter->InternalError("bad mode in DNS_Mgr::LookupAddr");
 		return 0;
 	}
 	}
@@ -531,7 +564,6 @@ void DNS_Mgr::Resolve()
 
 	int i;
 
-#ifdef HAVE_NB_DNS
 	int first_req = 0;
 	int num_pending = min(requests.length(), MAX_PENDING_REQUESTS);
 	int last_req = num_pending - 1;
@@ -575,7 +607,7 @@ void DNS_Mgr::Resolve()
 		struct nb_dns_result r;
 		status = nb_dns_activity(nb_dns, &r, err);
 		if ( status < 0 )
-			internal_error(
+			reporter->Warning(
 			    "NB-DNS error in DNS_Mgr::WaitForReplies (%s)",
 			    err);
 		else if ( status > 0 )
@@ -597,7 +629,6 @@ void DNS_Mgr::Resolve()
 				--num_pending;
 			}
 		}
-#endif
 
 	// All done with the list of requests.
 	for ( i = requests.length() - 1; i >= 0; --i )
@@ -616,6 +647,7 @@ int DNS_Mgr::Save()
 
 	Save(f, host_mappings);
 	Save(f, addr_mappings);
+	// Save(f, text_mappings); // We don't save the TXT mappings (yet?).
 
 	fclose(f);
 
@@ -673,6 +705,7 @@ Val* DNS_Mgr::BuildMappingVal(DNS_Mapping* dm)
 void DNS_Mgr::AddResult(DNS_Mgr_Request* dr, struct nb_dns_result* r)
 	{
 	struct hostent* h = (r && r->host_errno == 0) ? r->hostent : 0;
+	u_int32_t ttl = (r && r->host_errno == 0) ? r->ttl : 0;
 
 	DNS_Mapping* new_dm;
 	DNS_Mapping* prev_dm;
@@ -680,34 +713,80 @@ void DNS_Mgr::AddResult(DNS_Mgr_Request* dr, struct nb_dns_result* r)
 
 	if ( dr->ReqHost() )
 		{
-		new_dm = new DNS_Mapping(dr->ReqHost(), h);
-		prev_dm = host_mappings.Insert(dr->ReqHost(), new_dm);
+		new_dm = new DNS_Mapping(dr->ReqHost(), h, ttl);
+		prev_dm = 0;
 
-		if ( new_dm->Failed() && prev_dm && prev_dm->Valid() )
+		if ( dr->ReqIsTxt() )
 			{
-			// Put previous, valid entry back - CompareMappings
-			// will generate a corresponding warning.
-			(void) host_mappings.Insert(dr->ReqHost(), prev_dm);
-			++keep_prev;
+			TextMap::iterator it = text_mappings.find(dr->ReqHost());
+
+			if ( it == text_mappings.end() )
+				text_mappings[dr->ReqHost()] = new_dm;
+			else
+				{
+				prev_dm = it->second;
+				it->second = new_dm;
+				}
+
+			if ( new_dm->Failed() && prev_dm && prev_dm->Valid() )
+				{
+				text_mappings[dr->ReqHost()] = prev_dm;
+				++keep_prev;
+				}
+			}
+		else
+			{
+			HostMap::iterator it = host_mappings.find(dr->ReqHost());
+			if ( it == host_mappings.end() )
+				{
+				host_mappings[dr->ReqHost()].first =
+					new_dm->Type() == AF_INET ? new_dm : 0;
+
+				host_mappings[dr->ReqHost()].second =
+					new_dm->Type() == AF_INET ? 0 : new_dm;
+				}
+			else
+				{
+				if ( new_dm->Type() == AF_INET )
+					{
+					prev_dm = it->second.first;
+					it->second.first = new_dm;
+					}
+				else
+					{
+					prev_dm = it->second.second;
+					it->second.second = new_dm;
+					}
+				}
+
+			if ( new_dm->Failed() && prev_dm && prev_dm->Valid() )
+				{
+				// Put previous, valid entry back - CompareMappings
+				// will generate a corresponding warning.
+				if ( prev_dm->Type() == AF_INET )
+					host_mappings[dr->ReqHost()].first = prev_dm;
+				else
+					host_mappings[dr->ReqHost()].second = prev_dm;
+
+				++keep_prev;
+				}
 			}
 		}
 	else
 		{
-		new_dm = new DNS_Mapping(dr->ReqAddr(), h);
-		uint32 tmp_addr = dr->ReqAddr();
-		HashKey k(&tmp_addr, 1);
-		prev_dm = addr_mappings.Insert(&k, new_dm);
+		new_dm = new DNS_Mapping(dr->ReqAddr(), h, ttl);
+		AddrMap::iterator it = addr_mappings.find(dr->ReqAddr());
+		prev_dm = (it == addr_mappings.end()) ? 0 : it->second;
+		addr_mappings[dr->ReqAddr()] = new_dm;
 
 		if ( new_dm->Failed() && prev_dm && prev_dm->Valid() )
 			{
-			uint32 tmp_addr = dr->ReqAddr();
-			HashKey k2(&tmp_addr, 1);
-			(void) addr_mappings.Insert(&k2, prev_dm);
+			addr_mappings[dr->ReqAddr()] = prev_dm;
 			++keep_prev;
 			}
 		}
 
-	if ( prev_dm )
+	if ( prev_dm && ! dr->ReqIsTxt() )
 		CompareMappings(prev_dm, new_dm);
 
 	if ( keep_prev )
@@ -754,7 +833,10 @@ void DNS_Mgr::CompareMappings(DNS_Mapping* prev_dm, DNS_Mapping* new_dm)
 	ListVal* new_a = new_dm->Addrs();
 
 	if ( ! prev_a || ! new_a )
-		internal_error("confused in DNS_Mgr::CompareMappings");
+		{
+		reporter->InternalWarning("confused in DNS_Mgr::CompareMappings");
+		return;
+		}
 
 	ListVal* prev_delta = AddrListDelta(prev_a, new_a);
 	ListVal* new_delta = AddrListDelta(new_a, prev_a);
@@ -774,17 +856,13 @@ ListVal* DNS_Mgr::AddrListDelta(ListVal* al1, ListVal* al2)
 
 	for ( int i = 0; i < al1->Length(); ++i )
 		{
-		addr_type al1_i = al1->Index(i)->AsAddr();
+		const IPAddr& al1_i = al1->Index(i)->AsAddr();
 
 		int j;
 		for ( j = 0; j < al2->Length(); ++j )
 			{
-			addr_type al2_j = al2->Index(j)->AsAddr();
-#ifdef BROv6
-			if ( addr_eq(al1_i, al2_j) )
-#else
+			const IPAddr& al2_j = al2->Index(j)->AsAddr();
 			if ( al1_i == al2_j )
-#endif
 				break;
 			}
 
@@ -800,8 +878,8 @@ void DNS_Mgr::DumpAddrList(FILE* f, ListVal* al)
 	{
 	for ( int i = 0; i < al->Length(); ++i )
 		{
-		addr_type al_i = al->Index(i)->AsAddr();
-		fprintf(f, "%s%s", i > 0 ? "," : "", dotted_addr(al_i));
+		const IPAddr& al_i = al->Index(i)->AsAddr();
+		fprintf(f, "%s%s", i > 0 ? "," : "", al_i.AsString().c_str());
 		}
 	}
 
@@ -814,38 +892,69 @@ void DNS_Mgr::LoadCache(FILE* f)
 	for ( ; ! m->NoMapping() && ! m->InitFailed(); m = new DNS_Mapping(f) )
 		{
 		if ( m->ReqHost() )
-			host_mappings.Insert(m->ReqHost(), m);
+			{
+			if ( host_mappings.find(m->ReqHost()) == host_mappings.end() )
+				{
+				host_mappings[m->ReqHost()].first = 0;
+				host_mappings[m->ReqHost()].second = 0;
+				}
+			if ( m->Type() == AF_INET )
+				host_mappings[m->ReqHost()].first = m;
+			else
+				host_mappings[m->ReqHost()].second = m;
+			}
 		else
 			{
-			uint32 tmp_addr = m->ReqAddr();
-			HashKey h(&tmp_addr, 1);
-			addr_mappings.Insert(&h, m);
+			addr_mappings[m->ReqAddr()] = m;
 			}
 		}
 
 	if ( ! m->NoMapping() )
-		internal_error("DNS cache corrupted");
+		reporter->FatalError("DNS cache corrupted");
 
 	delete m;
 	fclose(f);
 	}
 
-void DNS_Mgr::Save(FILE* f, PDict(DNS_Mapping)& m)
+void DNS_Mgr::Save(FILE* f, const AddrMap& m)
 	{
-	IterCookie* cookie = m.InitForIteration();
-	DNS_Mapping* dm;
-
-	while ( (dm = m.NextEntry(cookie)) )
-		dm->Save(f);
+	for ( AddrMap::const_iterator it = m.begin(); it != m.end(); ++it )
+		{
+		if ( it->second )
+			it->second->Save(f);
+		}
 	}
 
-const char* DNS_Mgr::LookupAddrInCache(dns_mgr_addr_type addr)
+void DNS_Mgr::Save(FILE* f, const HostMap& m)
 	{
-	HashKey h(&addr, 1);
-	DNS_Mapping* d = dns_mgr->addr_mappings.Lookup(&h);
-	if ( ! d )
+	HostMap::const_iterator it;
+
+	for ( it = m.begin(); it != m.end(); ++it )
+		{
+		if ( it->second.first )
+			it->second.first->Save(f);
+
+		if ( it->second.second )
+			it->second.second->Save(f);
+		}
+	}
+
+const char* DNS_Mgr::LookupAddrInCache(const IPAddr& addr)
+	{
+	AddrMap::iterator it = addr_mappings.find(addr);
+
+	if ( it == addr_mappings.end() )
 		return 0;
 
+	DNS_Mapping* d = it->second;
+
+	if ( d->Expired() )
+		{
+		addr_mappings.erase(it);
+		delete d;
+		return 0;
+		}
+
 	// The escapes in the following strings are to avoid having it
 	// interpreted as a trigraph sequence.
 	return d->names ? d->names[0] : "<\?\?\?>";
@@ -853,15 +962,55 @@ const char* DNS_Mgr::LookupAddrInCache(dns_mgr_addr_type addr)
 
 TableVal* DNS_Mgr::LookupNameInCache(string name)
 	{
-	DNS_Mapping* d = dns_mgr->host_mappings.Lookup(name.c_str());
-	if ( ! d || ! d->names )
+	HostMap::iterator it = host_mappings.find(name);
+	if ( it == host_mappings.end() )
+		{
+		it = host_mappings.begin();
+		return 0;
+		}
+
+	DNS_Mapping* d4 = it->second.first;
+	DNS_Mapping* d6 = it->second.second;
+
+	if ( ! d4 || ! d4->names || ! d6 || ! d6->names )
 		return 0;
 
-	return d->AddrsSet();
+	if ( d4->Expired() || d6->Expired() )
+		{
+		host_mappings.erase(it);
+		delete d4;
+		delete d6;
+		return 0;
+		}
+
+	TableVal* tv4 = d4->AddrsSet();
+	TableVal* tv6 = d6->AddrsSet();
+	tv4->AddTo(tv6, false);
+	Unref(tv4);
+	return tv6;
 	}
 
-#ifdef HAVE_NB_DNS
-void DNS_Mgr::AsyncLookupAddr(dns_mgr_addr_type host, LookupCallback* callback)
+const char* DNS_Mgr::LookupTextInCache(string name)
+	{
+	TextMap::iterator it = text_mappings.find(name);
+	if ( it == text_mappings.end() )
+		return 0;
+
+	DNS_Mapping* d = it->second;
+
+	if ( d->Expired() )
+		{
+		text_mappings.erase(it);
+		delete d;
+		return 0;
+		}
+
+	// The escapes in the following strings are to avoid having it
+	// interpreted as a trigraph sequence.
+	return d->names ? d->names[0] : "<\?\?\?>";
+	}
+
+void DNS_Mgr::AsyncLookupAddr(const IPAddr& host, LookupCallback* callback)
 	{
 	if ( ! did_init )
 		Init();
@@ -930,6 +1079,46 @@ void DNS_Mgr::AsyncLookupName(string name, LookupCallback* callback)
 	IssueAsyncRequests();
 	}
 
+void DNS_Mgr::AsyncLookupNameText(string name, LookupCallback* callback)
+	{
+	if ( ! did_init )
+		Init();
+
+	// Do we already know the answer?
+	TableVal* addrs;
+
+	AsyncRequest* req = 0;
+
+	// Have we already a request waiting for this host?
+	AsyncRequestTextMap::iterator i = asyncs_texts.find(name);
+	if ( i != asyncs_texts.end() )
+		req = i->second;
+	else
+		{
+		// A new one.
+		req = new AsyncRequest;
+		req->name = name;
+		req->is_txt = true;
+		asyncs_queued.push_back(req);
+		asyncs_texts.insert(AsyncRequestTextMap::value_type(name, req));
+		}
+
+	req->callbacks.push_back(callback);
+
+	IssueAsyncRequests();
+	}
+
+static bool DoRequest(nb_dns_info* nb_dns, DNS_Mgr_Request* dr)
+	{
+	if ( dr->MakeRequest(nb_dns) )
+		// dr stored in nb_dns cookie and deleted later when results available.
+		return true;
+
+	reporter->Warning("can't issue DNS request");
+	delete dr;
+	return false;
+	}
+
 void DNS_Mgr::IssueAsyncRequests()
 	{
 	while ( asyncs_queued.size() && asyncs_pending < MAX_PENDING_REQUESTS )
@@ -937,16 +1126,28 @@ void DNS_Mgr::IssueAsyncRequests()
 		AsyncRequest* req = asyncs_queued.front();
 		asyncs_queued.pop_front();
 
-		DNS_Mgr_Request* dr;
-		if ( req->IsAddrReq() )
-			dr = new DNS_Mgr_Request(req->host);
-		else
-			dr = new DNS_Mgr_Request(req->name.c_str());
+		++num_requests;
 
-		if ( ! dr->MakeRequest(nb_dns) )
+		bool success;
+
+		if ( req->IsAddrReq() )
+			success = DoRequest(nb_dns, new DNS_Mgr_Request(req->host));
+		else if ( req->is_txt )
+			success = DoRequest(nb_dns, new DNS_Mgr_Request(req->name.c_str(),
+			                                AF_INET, req->is_txt));
+		else
+			{
+			// If only one request type succeeds, don't consider it a failure.
+			success = DoRequest(nb_dns, new DNS_Mgr_Request(req->name.c_str(),
+			                                AF_INET, req->is_txt));
+			success = DoRequest(nb_dns, new DNS_Mgr_Request(req->name.c_str(),
+			                                AF_INET6, req->is_txt)) || success;
+			}
+
+		if ( ! success )
 			{
-			run_time("can't issue DNS request");
 			req->Timeout();
+			++failed;
 			continue;
 			}
 
@@ -956,13 +1157,10 @@ void DNS_Mgr::IssueAsyncRequests()
 		++asyncs_pending;
 		}
 	}
-#endif
 
 void  DNS_Mgr::GetFds(int* read, int* write, int* except)
 	{
-#ifdef HAVE_NB_DNS
 	*read = nb_dns_fd(nb_dns);
-#endif
 	}
 
 double DNS_Mgr::NextTimestamp(double* network_time)
@@ -971,8 +1169,7 @@ double DNS_Mgr::NextTimestamp(double* network_time)
 	return asyncs_timeouts.size() ? timer_mgr->Time() : -1.0;
 	}
 
-#ifdef HAVE_NB_DNS
-void DNS_Mgr::CheckAsyncAddrRequest(dns_mgr_addr_type addr, bool timeout)
+void DNS_Mgr::CheckAsyncAddrRequest(const IPAddr& addr, bool timeout)
 	{
 	// Note that this code is a mirror of that for CheckAsyncHostRequest.
 
@@ -984,10 +1181,16 @@ void DNS_Mgr::CheckAsyncAddrRequest(dns_mgr_addr_type addr, bool timeout)
 		{
 		const char* name = LookupAddrInCache(addr);
 		if ( name )
+			{
+			++successful;
 			i->second->Resolved(name);
+			}
 
 		else if ( timeout )
+			{
+			++failed;
 			i->second->Timeout();
+			}
 
 		else
 			return;
@@ -1001,6 +1204,38 @@ void DNS_Mgr::CheckAsyncAddrRequest(dns_mgr_addr_type addr, bool timeout)
 
 	}
 
+void DNS_Mgr::CheckAsyncTextRequest(const char* host, bool timeout)
+	{
+	// Note that this code is a mirror of that for CheckAsyncAddrRequest.
+
+	AsyncRequestTextMap::iterator i = asyncs_texts.find(host);
+	if ( i != asyncs_texts.end() )
+		{
+		const char* name = LookupTextInCache(host);
+		if ( name )
+			{
+			++successful;
+			i->second->Resolved(name);
+			}
+
+		else if ( timeout )
+			{
+			AsyncRequestTextMap::iterator it = asyncs_texts.begin();
+			++failed;
+			i->second->Timeout();
+			}
+
+		else
+			return;
+
+		asyncs_texts.erase(i);
+		--asyncs_pending;
+
+		// Don't delete the request.  That will be done once it
+		// eventually times out.
+		}
+	}
+
 void DNS_Mgr::CheckAsyncHostRequest(const char* host, bool timeout)
 	{
 	// Note that this code is a mirror of that for CheckAsyncAddrRequest.
@@ -1013,12 +1248,16 @@ void DNS_Mgr::CheckAsyncHostRequest(const char* host, bool timeout)
 
 		if ( addrs )
 			{
+			++successful;
 			i->second->Resolved(addrs);
 			Unref(addrs);
 			}
 
 		else if ( timeout )
+			{
+			++failed;
 			i->second->Timeout();
+			}
 
 		else
 			return;
@@ -1030,23 +1269,47 @@ void DNS_Mgr::CheckAsyncHostRequest(const char* host, bool timeout)
 		// eventually times out.
 		}
 	}
-#endif
 
-void  DNS_Mgr::Process()
+void DNS_Mgr::Flush()
 	{
-#ifndef HAVE_NB_DNS
-	internal_error("DNS_Mgr::Process(): should never be reached");
-#else
+	DoProcess(false);
 
+	HostMap::iterator it;
+	for ( it = host_mappings.begin(); it != host_mappings.end(); ++it )
+		{
+		delete it->second.first;
+		delete it->second.second;
+		}
+
+	for ( AddrMap::iterator it2 = addr_mappings.begin(); it2 != addr_mappings.end(); ++it2 )
+		delete it2->second;
+
+	for ( TextMap::iterator it3 = text_mappings.begin(); it3 != text_mappings.end(); ++it3 )
+		delete it3->second;
+
+	host_mappings.clear();
+	addr_mappings.clear();
+	text_mappings.clear();
+	}
+
+void DNS_Mgr::Process()
+	{
+	DoProcess(false);
+	}
+
+void DNS_Mgr::DoProcess(bool flush)
+	{
 	while ( asyncs_timeouts.size() > 0 )
 		{
 		AsyncRequest* req = asyncs_timeouts.top();
 
-		if ( req->time + DNS_TIMEOUT > current_time() )
+		if ( req->time + DNS_TIMEOUT > current_time() || flush )
 			break;
 
 		if ( req->IsAddrReq() )
 			CheckAsyncAddrRequest(req->host, true);
+		else if ( req->is_txt )
+			CheckAsyncTextRequest(req->name.c_str(), true);
 		else
 			CheckAsyncHostRequest(req->name.c_str(), true);
 
@@ -1054,7 +1317,7 @@ void  DNS_Mgr::Process()
 		delete req;
 		}
 
-	if ( asyncs_addrs.size() == 0 && asyncs_names.size() == 0 )
+	if ( asyncs_addrs.size() == 0 && asyncs_names.size() == 0 && asyncs_texts.size() == 0 )
 		return;
 
 	if ( AnswerAvailable(0) <= 0 )
@@ -1066,11 +1329,19 @@ void  DNS_Mgr::Process()
 	int status = nb_dns_activity(nb_dns, &r, err);
 
 	if ( status < 0 )
-		internal_error("NB-DNS error in DNS_Mgr::Process (%s)", err);
+		reporter->Warning("NB-DNS error in DNS_Mgr::Process (%s)", err);
 
 	else if ( status > 0 )
 		{
 		DNS_Mgr_Request* dr = (DNS_Mgr_Request*) r.cookie;
+
+		bool do_host_timeout = true;
+		if ( dr->ReqHost() &&
+		     host_mappings.find(dr->ReqHost()) == host_mappings.end() )
+			// Don't timeout when this is the first result in an expected pair
+			// (one result each for A and AAAA queries).
+			do_host_timeout = false;
+
 		if ( dr->RequestPending() )
 			{
 			AddResult(dr, &r);
@@ -1079,19 +1350,25 @@ void  DNS_Mgr::Process()
 
 		if ( ! dr->ReqHost() )
 			CheckAsyncAddrRequest(dr->ReqAddr(), true);
+		else if ( dr->ReqIsTxt() )
+			CheckAsyncTextRequest(dr->ReqHost(), do_host_timeout);
 		else
-			CheckAsyncHostRequest(dr->ReqHost(), true);
+			CheckAsyncHostRequest(dr->ReqHost(), do_host_timeout);
 
 		IssueAsyncRequests();
+
+		delete dr;
 		}
-#endif
 	}
-#ifdef HAVE_NB_DNS
+
 int DNS_Mgr::AnswerAvailable(int timeout)
 	{
 	int fd = nb_dns_fd(nb_dns);
 	if ( fd < 0 )
-		internal_error("nb_dns_fd() failed in DNS_Mgr::WaitForReplies");
+		{
+		reporter->Warning("nb_dns_fd() failed in DNS_Mgr::WaitForReplies");
+		return -1;
+		}
 
 	fd_set read_fds;
 
@@ -1106,14 +1383,29 @@ int DNS_Mgr::AnswerAvailable(int timeout)
 
 	if ( status < 0 )
 		{
-		if ( errno == EINTR )
-			return -1;
-		internal_error("problem with DNS select");
+		if ( errno != EINTR )
+			reporter->Warning("problem with DNS select");
+
+		return -1;
 		}
 
 	if ( status > 1 )
-		internal_error("strange return from DNS select");
+		{
+		reporter->Warning("strange return from DNS select");
+		return -1;
+		}
 
 	return status;
 	}
-#endif
+
+void DNS_Mgr::GetStats(Stats* stats)
+	{
+	stats->requests = num_requests;
+	stats->successful = successful;
+	stats->failed = failed;
+	stats->pending = asyncs_pending;
+	stats->cached_hosts = host_mappings.size();
+	stats->cached_addresses = addr_mappings.size();
+	stats->cached_texts = text_mappings.size();
+	}
+
