@@ -1,6 +1,4 @@
-// $Id: DNS_Mgr.h 6915 2009-09-22 05:04:17Z vern $
-//
-// See the file "COPYING" in the main distribution directory for copyright.
+  // See the file "COPYING" in the main distribution directory for copyright.
 
 #ifndef dnsmgr_h
 #define dnsmgr_h
@@ -8,12 +6,14 @@
 #include <list>
 #include <map>
 #include <queue>
+#include <utility>
 
 #include "util.h"
 #include "BroList.h"
 #include "Dict.h"
 #include "EventHandler.h"
 #include "IOSource.h"
+#include "IPAddr.h"
 
 class Val;
 class ListVal;
@@ -29,7 +29,6 @@ struct nb_dns_result;
 declare(PDict,ListVal);
 
 class DNS_Mapping;
-declare(PDict,DNS_Mapping);
 
 enum DNS_MgrMode {
 	DNS_PRIME,	// used to prime the cache
@@ -41,22 +40,19 @@ enum DNS_MgrMode {
 // Number of seconds we'll wait for a reply.
 #define DNS_TIMEOUT 5
 
-// ### For now, we don't support IPv6 lookups.  When we do, this
-// should become addr_type.
-typedef uint32 dns_mgr_addr_type;
-
 class DNS_Mgr : public IOSource {
 public:
 	DNS_Mgr(DNS_MgrMode mode);
 	virtual ~DNS_Mgr();
 
 	bool Init();
+	void Flush();
 
 	// Looks up the address or addresses of the given host, and returns
 	// a set of addr.
 	TableVal* LookupHost(const char* host);
 
-	Val* LookupAddr(uint32 addr);
+	Val* LookupAddr(const IPAddr& addr);
 
 	// Define the directory where to store the data.
 	void SetDir(const char* arg_dir)	{ dir = copy_string(arg_dir); }
@@ -65,8 +61,9 @@ public:
 	void Resolve();
 	int Save();
 
-	const char* LookupAddrInCache(dns_mgr_addr_type addr);
+	const char* LookupAddrInCache(const IPAddr& addr);
 	TableVal* LookupNameInCache(string name);
+	const char* LookupTextInCache(string name);
 
 	// Support for async lookups.
 	class LookupCallback {
@@ -79,10 +76,21 @@ public:
 		virtual void Timeout() = 0;
 	};
 
-#ifdef HAVE_NB_DNS
-	void AsyncLookupAddr(dns_mgr_addr_type host, LookupCallback* callback);
+	void AsyncLookupAddr(const IPAddr& host, LookupCallback* callback);
 	void AsyncLookupName(string name, LookupCallback* callback);
-#endif
+	void AsyncLookupNameText(string name, LookupCallback* callback);
+
+	struct Stats {
+		unsigned long requests;	// These count only async requests.
+		unsigned long successful;
+		unsigned long failed;
+		unsigned long pending;
+		unsigned long cached_hosts;
+		unsigned long cached_addresses;
+		unsigned long cached_texts;
+	};
+
+	void GetStats(Stats* stats);
 
 protected:
 	friend class LookupCallback;
@@ -99,12 +107,16 @@ protected:
 	ListVal* AddrListDelta(ListVal* al1, ListVal* al2);
 	void DumpAddrList(FILE* f, ListVal* al);
 
+	typedef map<string, pair<DNS_Mapping*, DNS_Mapping*> > HostMap;
+	typedef map<IPAddr, DNS_Mapping*> AddrMap;
+	typedef map<string, DNS_Mapping*> TextMap;
 	void LoadCache(FILE* f);
-	void Save(FILE* f, PDict(DNS_Mapping)& m);
+	void Save(FILE* f, const AddrMap& m);
+	void Save(FILE* f, const HostMap& m);
 
-#ifdef HAVE_NB_DNS
-	// Selects on the fd to see if there is an answer available (timeout is
-	// secs). Returns 0 on timeout, -1 on EINTR, and 1 if answer is ready.
+	// Selects on the fd to see if there is an answer available (timeout
+	// is secs). Returns 0 on timeout, -1 on EINTR or other error, and 1
+	// if answer is ready.
 	int AnswerAvailable(int timeout);
 
 	// Issue as many queued async requests as slots are available.
@@ -112,10 +124,12 @@ protected:
 
 	// Finish the request if we have a result.  If not, time it out if
 	// requested.
-	void CheckAsyncAddrRequest(dns_mgr_addr_type addr, bool timeout);
+	void CheckAsyncAddrRequest(const IPAddr& addr, bool timeout);
 	void CheckAsyncHostRequest(const char* host, bool timeout);
+	void CheckAsyncTextRequest(const char* host, bool timeout);
 
-#endif
+	// Process outstanding requests.
+	void DoProcess(bool flush);
 
 	// IOSource interface.
 	virtual void GetFds(int* read, int* write, int* except);
@@ -127,8 +141,9 @@ protected:
 
 	PDict(ListVal) services;
 
-	PDict(DNS_Mapping) host_mappings;
-	PDict(DNS_Mapping) addr_mappings;
+	HostMap host_mappings;
+	AddrMap addr_mappings;
+	TextMap text_mappings;
 
 	DNS_mgr_request_list requests;
 
@@ -154,10 +169,13 @@ protected:
 
 	struct AsyncRequest {
 		double time;
-		dns_mgr_addr_type host;
+		IPAddr host;
 		string name;
+		bool is_txt;
 		CallbackList callbacks;
 
+		AsyncRequest() : time(0.0), is_txt(false) { }
+
 		bool IsAddrReq() const	{ return name.length() == 0; }
 
 		void Resolved(const char* name)
@@ -195,12 +213,15 @@ protected:
 
 	};
 
-	typedef map<dns_mgr_addr_type, AsyncRequest*> AsyncRequestAddrMap;
+	typedef map<IPAddr, AsyncRequest*> AsyncRequestAddrMap;
 	AsyncRequestAddrMap asyncs_addrs;
 
 	typedef map<string, AsyncRequest*> AsyncRequestNameMap;
 	AsyncRequestNameMap asyncs_names;
 
+	typedef map<string, AsyncRequest*> AsyncRequestTextMap;
+	AsyncRequestTextMap asyncs_texts;
+
 	typedef list<AsyncRequest*> QueuedList;
 	QueuedList asyncs_queued;
 
@@ -208,6 +229,10 @@ protected:
 	TimeoutQueue asyncs_timeouts;
 
 	int asyncs_pending;
+
+	unsigned long num_requests;
+	unsigned long successful;
+	unsigned long failed;
 };
 
 extern DNS_Mgr* dns_mgr;
