@@ -40,6 +40,7 @@ DebuggerState::DebuggerState()
 	next_bp_id = next_watch_id = next_display_id = 1;
 	BreakBeforeNextStmt(false);
 	curr_frame_idx = 0;
+	already_did_list = false;
 	BreakFromSignal(false);
 
 	// ### Don't choose this arbitrary size! Extend Frame.
@@ -54,7 +55,7 @@ DebuggerState::~DebuggerState()
 bool StmtLocMapping::StartsAfter(const StmtLocMapping* m2)
 	{
 	if ( ! m2  )
-		internal_error("Assertion failed: %s", "m2 != 0");
+		reporter->InternalError("Assertion failed: m2 != 0");
 
 	return loc.first_line > m2->loc.first_line ||
 		(loc.first_line == m2->loc.first_line &&
@@ -142,7 +143,7 @@ int TraceState::LogTrace(const char* fmt, ...)
 
 	if ( ! loc.filename )
 		{
-		loc.filename = "<no filename>";
+		loc.filename = copy_string("<no filename>");
 		loc.last_line = 0;
 		}
 
@@ -194,8 +195,7 @@ static void parse_function_name(vector<ParseLocationRec>& result,
 		return;
 		}
 
-	FuncType* ftype;
-	if ( ! (ftype = id->Type()->AsFuncType()) )
+	if ( ! id->Type()->AsFuncType() )
 		{
 		debug_msg("Function %s not declared.\n", id->Name());
 		plr.type = plrUnknown;
@@ -343,7 +343,7 @@ vector<ParseLocationRec> parse_location_string(const string& s)
 				plr.type = plrUnknown;
 
 			FILE* throwaway = search_for_file(filename.c_str(), "bro",
-								&full_filename);
+								&full_filename, true, 0);
 			if ( ! throwaway )
 				{
 				debug_msg("No such policy file: %s.\n", filename.c_str());
@@ -362,7 +362,7 @@ vector<ParseLocationRec> parse_location_string(const string& s)
 		{
 		Filemap* map = g_dbgfilemaps.Lookup(loc_filename);
 		if ( ! map )
-			internal_error("Policy file %s should have been loaded\n",
+			reporter->InternalError("Policy file %s should have been loaded\n",
 					loc_filename);
 
 		if ( plr.line > how_many_lines_in(loc_filename) )
@@ -603,7 +603,7 @@ int dbg_execute_command(const char* cmd)
 #endif
 
 	if ( int(cmd_code) >= num_debug_cmds() )
-		internal_error("Assertion failed: %s", "int(cmd_code) < num_debug_cmds()");
+		reporter->InternalError("Assertion failed: int(cmd_code) < num_debug_cmds()");
 
 	// Dispatch to the op-specific handler (with args).
 	int retcode = dbg_dispatch_cmd(cmd_code, arguments);
@@ -612,7 +612,7 @@ int dbg_execute_command(const char* cmd)
 
 	const DebugCmdInfo* info = get_debug_cmd_info(cmd_code);
 	if ( ! info  )
-		internal_error("Assertion failed: %s", "info");
+		reporter->InternalError("Assertion failed: info");
 
 	if ( ! info )
 		return -2;	// ### yuck, why -2?
@@ -721,9 +721,8 @@ static char* get_prompt(bool reset_counter = false)
 
 string get_context_description(const Stmt* stmt, const Frame* frame)
 	{
-	char buf[1024];
 	ODesc d;
-	const BroFunc* func = frame->GetFunction();
+	const BroFunc* func = frame ? frame->GetFunction() : 0;
 
 	if ( func )
 		func->DescribeDebug(&d, frame->GetFuncArgs());
@@ -735,14 +734,18 @@ string get_context_description(const Stmt* stmt, const Frame* frame)
 		loc = *stmt->GetLocationInfo();
 	else
 		{
-		loc.filename = "<no filename>";
+		loc.filename = copy_string("<no filename>");
 		loc.last_line = 0;
 		}
 
-	safe_snprintf(buf, sizeof(buf), "In %s at %s:%d",
+	size_t buf_size = strlen(d.Description()) + strlen(loc.filename) + 1024;
+	char* buf = new char[buf_size];
+	safe_snprintf(buf, buf_size, "In %s at %s:%d",
 		      d.Description(), loc.filename, loc.last_line);
 
-	return string(buf);
+	string retval(buf);
+	delete [] buf;
+	return retval;
 	}
 
 int dbg_handle_debug_input()
@@ -760,13 +763,13 @@ int dbg_handle_debug_input()
 	Frame* curr_frame = g_frame_stack.back();
 	const BroFunc* func = curr_frame->GetFunction();
 	if ( func )
-		current_module = func->GetID()->ModuleName();
+		current_module = extract_module_name(func->Name());
 	else
 		current_module = GLOBAL_MODULE_NAME;
 
 	const Stmt* stmt = curr_frame->GetNextStmt();
 	if ( ! stmt )
-		internal_error("Assertion failed: %s", "stmt != 0");
+		reporter->InternalError("Assertion failed: stmt != 0");
 
 	const Location loc = *stmt->GetLocationInfo();
 
@@ -797,7 +800,7 @@ int dbg_handle_debug_input()
 		input_line = (char*) safe_malloc(1024);
 		input_line[1023] = 0;
 		// ### Maybe it's not always stdin.
-		fgets(input_line, 1023, stdin);
+		input_line = fgets(input_line, 1023, stdin);
 #endif
 
 		// ### Maybe not stdin; maybe do better cleanup.
@@ -872,7 +875,7 @@ bool pre_execute_stmt(Stmt* stmt, Frame* f)
 		p = g_debugger_state.breakpoint_map.equal_range(stmt);
 
 		if ( p.first == p.second )
-			internal_error("Breakpoint count nonzero, but no matching breakpoints");
+			reporter->InternalError("Breakpoint count nonzero, but no matching breakpoints");
 
 		for ( BPMapType::iterator i = p.first; i != p.second; ++i )
 			{
@@ -924,6 +927,8 @@ bool post_execute_stmt(Stmt* stmt, Frame* f, Val* result, stmt_flow_type* flow)
 // Evaluates the given expression in the context of the currently selected
 // frame.  Returns the resulting value, or nil if none (or there was an error).
 Expr* g_curr_debug_expr = 0;
+const char* g_curr_debug_error = 0;
+bool in_debug = false;
 
 // ### fix this hardwired access to external variables etc.
 struct yy_buffer_state;
@@ -943,11 +948,11 @@ Val* dbg_eval_expr(const char* expr)
 		(g_frame_stack.size() - 1) - g_debugger_state.curr_frame_idx;
 
 	if ( ! (frame_idx >= 0 && (unsigned) frame_idx < g_frame_stack.size())  )
-		internal_error("Assertion failed: %s", "frame_idx >= 0 && (unsigned) frame_idx < g_frame_stack.size()");
+		reporter->InternalError("Assertion failed: frame_idx >= 0 && (unsigned) frame_idx < g_frame_stack.size()");
 
 	Frame* frame = g_frame_stack[frame_idx];
 	if ( ! (frame)  )
-		internal_error("Assertion failed: %s", "frame");
+		reporter->InternalError("Assertion failed: frame");
 
 	const BroFunc* func = frame->GetFunction();
 	if ( func )
@@ -969,6 +974,11 @@ Val* dbg_eval_expr(const char* expr)
 	Val* result = 0;
 	if ( yyparse() )
 		{
+		if ( g_curr_debug_error )
+			debug_msg("Parsing expression '%s' failed: %s\n", expr, g_curr_debug_error);
+		else
+			debug_msg("Parsing expression '%s' failed\n", expr);
+
 		if ( g_curr_debug_expr )
 			{
 			delete g_curr_debug_expr;
@@ -983,6 +993,9 @@ Val* dbg_eval_expr(const char* expr)
 
 	delete g_curr_debug_expr;
 	g_curr_debug_expr = 0;
+	delete [] g_curr_debug_error;
+	g_curr_debug_error = 0;
+	in_debug = false;
 
 	return result;
 	}
