@@ -1,5 +1,3 @@
-// $Id: EventHandler.cc 5911 2008-07-03 22:59:01Z vern $
-
 #include "Event.h"
 #include "EventHandler.h"
 #include "Func.h"
@@ -12,7 +10,7 @@ EventHandler::EventHandler(const char* arg_name)
 	used = false;
 	local = 0;
 	type = 0;
-	group = 0;
+	error_handler = false;
 	enabled = true;
 	}
 
@@ -20,7 +18,11 @@ EventHandler::~EventHandler()
 	{
 	Unref(local);
 	delete [] name;
-	delete [] group;
+	}
+
+EventHandler::operator bool() const
+	{
+	return enabled && ((local && local->HasBodies()) || receivers.length());
 	}
 
 FuncType* EventHandler::FType()
@@ -64,6 +66,7 @@ void EventHandler::Call(val_list* vl, bool no_remote)
 		}
 
 	if ( local )
+		// No try/catch here; we pass exceptions upstream.
 		Unref(local->Call(vl));
 	else
 		{
@@ -91,7 +94,7 @@ EventHandler* EventHandler::Unserialize(UnserialInfo* info)
 	{
 	char* name;
 	if ( ! UNSERIALIZE_STR(&name, 0) )
-		return false;
+		return 0;
 
 	EventHandler* h = event_registry->Lookup(name);
 	if ( ! h )
