@@ -1,15 +1,13 @@
-// $Id: File.cc 7094 2010-10-03 21:18:32Z vern $
-//
 // See the file "COPYING" in the main distribution directory for copyright.
 
 #include "config.h"
 
 #include <sys/types.h>
-#if TIME_WITH_SYS_TIME
+#ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
 #else
-# if HAVE_SYS_TIME_H
+# ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>
 # else
 #  include <time.h>
@@ -20,6 +18,8 @@
 #include <errno.h>
 #include <unistd.h>
 
+#include <algorithm>
+
 #include "File.h"
 #include "Type.h"
 #include "Timer.h"
@@ -28,6 +28,7 @@
 #include "Net.h"
 #include "Serializer.h"
 #include "Event.h"
+#include "Reporter.h"
 
 // Timer which on dispatching rotates the file.
 class RotateTimer : public Timer {
@@ -73,9 +74,8 @@ void RotateTimer::Dispatch(double t, int is_expire)
 
 // The following could in principle be part of a "file manager" object.
 
-#define MAX_FILE_CACHE_SIZE 32
+#define MAX_FILE_CACHE_SIZE 512
 static int num_files_in_cache = 0;
-static int max_files_in_cache = 0;
 static BroFile* head = 0;
 static BroFile* tail = 0;
 
@@ -86,12 +86,9 @@ double BroFile::default_rotation_size = 0;
 // that we should use for the cache.
 static int maximize_num_fds()
 	{
-#ifdef NO_HAVE_SETRLIMIT
-	return MAX_FILE_CACHE_SIZE;
-#else
 	struct rlimit rl;
 	if ( getrlimit(RLIMIT_NOFILE, &rl) < 0 )
-		internal_error("maximize_num_fds(): getrlimit failed");
+		reporter->FatalError("maximize_num_fds(): getrlimit failed");
 
 	if ( rl.rlim_max == RLIM_INFINITY )
 		{
@@ -107,10 +104,9 @@ static int maximize_num_fds()
 	rl.rlim_cur = rl.rlim_max;
 
 	if ( setrlimit(RLIMIT_NOFILE, &rl) < 0 )
-		internal_error("maximize_num_fds(): setrlimit failed");
+		reporter->FatalError("maximize_num_fds(): setrlimit failed");
 
 	return rl.rlim_cur / 2;
-#endif
 	}
 
 
@@ -142,13 +138,24 @@ BroFile::BroFile(FILE* arg_f, const char* arg_name, const char* arg_access)
 BroFile::BroFile(const char* arg_name, const char* arg_access, BroType* arg_t)
 	{
 	Init();
-
+	f = 0;
 	name = copy_string(arg_name);
 	access = copy_string(arg_access);
 	t = arg_t ? arg_t : base_type(TYPE_STRING);
-	if ( ! Open() )
+
+	if ( streq(name, "/dev/stdin") )
+		f = stdin;
+	else if ( streq(name, "/dev/stdout") )
+		f = stdout;
+	else if ( streq(name, "/dev/stderr") )
+		f = stderr;
+
+	if ( f )
+		is_open = 1;
+
+	else if ( ! Open() )
 		{
-		error(fmt("cannot open %s: %s", name, strerror(errno)));
+		reporter->Error("cannot open %s: %s", name, strerror(errno));
 		is_open = 0;
 		okay_to_manage = 0;
 		}
@@ -171,7 +178,7 @@ const char* BroFile::Name() const
 	return 0;
 	}
 
-bool BroFile::Open(FILE* file)
+bool BroFile::Open(FILE* file, const char* mode)
 	{
 	open_time = network_time ? network_time : current_time();
 
@@ -196,10 +203,14 @@ bool BroFile::Open(FILE* file)
 
 	if ( ! f )
 		{
-		f = fopen(name, access);
-		SetBuf(buffered);
+		if ( ! mode )
+			f = fopen(name, access);
+		else
+			f = fopen(name, mode);
 		}
 
+	SetBuf(buffered);
+
 	if ( f )
 		{
 		// These are the only files we manage, because we open them
@@ -230,12 +241,9 @@ BroFile::~BroFile()
 
 	delete [] name;
 	delete [] access;
-
-#ifdef USE_OPENSSL
 	delete [] cipher_buffer;
-#endif
 
-#ifdef USE_PERFTOOLS
+#ifdef USE_PERFTOOLS_DEBUG
 	heap_checker->UnIgnoreObject(this);
 #endif
 	}
@@ -254,14 +262,11 @@ void BroFile::Init()
 	print_hook = true;
 	raw_output = false;
 	t = 0;
-
-#ifdef USE_OPENSSL
 	pub_key = 0;
 	cipher_ctx = 0;
 	cipher_buffer = 0;
-#endif
 
-#ifdef USE_PERFTOOLS
+#ifdef USE_PERFTOOLS_DEBUG
 	heap_checker->IgnoreObject(this);
 #endif
 	}
@@ -276,8 +281,10 @@ FILE* BroFile::File()
 
 FILE* BroFile::BringIntoCache()
 	{
+	char buf[256];
+
 	if ( f )
-		internal_error("BroFile non-nil non-open file");
+		reporter->InternalError("BroFile non-nil non-open file");
 
 	if ( num_files_in_cache >= max_files_in_cache )
 		PurgeCache();
@@ -291,22 +298,30 @@ FILE* BroFile::BringIntoCache()
 
 	if ( ! f )
 		{
-		run_time("can't open %s", this);
+		strerror_r(errno, buf, sizeof(buf));
+		reporter->Error("can't open %s: %s", name, buf);
 
 		f = fopen("/dev/null", "w");
 
-		if ( ! f )
-			internal_error("out of file descriptors");
+		if ( f )
+			{
+			okay_to_manage = 0;
+			return f;
+			}
 
-		okay_to_manage = 0;
-		return f;
+		strerror_r(errno, buf, sizeof(buf));
+		reporter->Error("can't open /dev/null: %s", buf);
+		return 0;
 		}
 
 	RaiseOpenEvent();
 	UpdateFileSize();
 
 	if ( fseek(f, position, SEEK_SET) < 0 )
-		run_time("reopen seek failed", this);
+		{
+		strerror_r(errno, buf, sizeof(buf));
+		reporter->Error("reopen seek failed: %s", buf);
+		}
 
 	InsertAtBeginning();
 
@@ -319,7 +334,7 @@ FILE* BroFile::Seek(long new_position)
 		return 0;
 
 	if ( fseek(f, new_position, SEEK_SET) < 0 )
-		run_time("seek failed", this);
+		reporter->Error("seek failed");
 
 	return f;
 	}
@@ -330,7 +345,7 @@ void BroFile::SetBuf(bool arg_buffered)
 		return;
 
 	if ( setvbuf(f, NULL, arg_buffered ? _IOFBF : _IOLBF, 0) != 0 )
-		run_time("setvbuf failed", this);
+		reporter->Error("setvbuf failed");
 
 	buffered = arg_buffered;
 	}
@@ -346,12 +361,10 @@ int BroFile::Close()
 	if ( ! is_open )
 		return 1;
 
-#ifdef USE_OPENSSL
 	FinishEncrypt();
-#endif
 
-	// Do not close stdout/stderr.
-	if ( f == stdout || f == stderr )
+	// Do not close stdin/stdout/stderr.
+	if ( f == stdin || f == stdout || f == stderr )
 		return 0;
 
 	if ( is_in_cache )
@@ -383,18 +396,21 @@ int BroFile::Close()
 void BroFile::Suspend()
 	{
 	if ( ! is_in_cache )
-		internal_error("BroFile::Suspend() called for non-cached file");
+		reporter->InternalError("BroFile::Suspend() called for non-cached file");
+
 	if ( ! is_open )
-		internal_error("BroFile::Suspend() called for non-open file");
+		reporter->InternalError("BroFile::Suspend() called for non-open file");
 
 	Unlink();
 
 	if ( ! f )
-		internal_error("BroFile::Suspend() called for nil file");
+		reporter->InternalError("BroFile::Suspend() called for nil file");
 
 	if ( (position = ftell(f)) < 0 )
 		{
-		run_time("ftell failed", this);
+		char buf[256];
+		strerror_r(errno, buf, sizeof(buf));
+		reporter->Error("ftell failed: %s", buf);
 		position = 0;
 		}
 
@@ -404,10 +420,13 @@ void BroFile::Suspend()
 
 void BroFile::PurgeCache()
 	{
-	if ( ! tail )
-		internal_error("BroFile purge of empty cache");
+	if ( tail )
+		{
+		tail->Suspend();
+		return;
+		}
 
-	tail->Suspend();
+	reporter->InternalWarning("BroFile purge of empty cache");
 	}
 
 void BroFile::Unlink()
@@ -425,13 +444,13 @@ void BroFile::Unlink()
 			Next()->SetPrev(prev);
 
 		if ( (head || tail) && ! (head && tail) )
-			internal_error("BroFile link list botch");
+			reporter->InternalError("BroFile link list botch");
 
 		is_in_cache = 0;
 		prev = next = 0;
 
 		if ( --num_files_in_cache < 0 )
-			internal_error("BroFile underflow of file cache");
+			reporter->InternalError("BroFile underflow of file cache");
 		}
 	}
 
@@ -451,7 +470,7 @@ void BroFile::InsertAtBeginning()
 		}
 
 	if ( ++num_files_in_cache > max_files_in_cache )
-		internal_error("BroFile overflow of file cache");
+		reporter->InternalError("BroFile overflow of file cache");
 
 	is_in_cache = 1;
 	}
@@ -462,7 +481,7 @@ void BroFile::MoveToBeginning()
 		return;	// already at the beginning
 
 	if ( ! is_in_cache || ! prev )
-		internal_error("BroFile inconsistency in MoveToBeginning()");
+		reporter->InternalError("BroFile inconsistency in MoveToBeginning()");
 
 	Unlink();
 	InsertAtBeginning();
@@ -511,12 +530,9 @@ void BroFile::SetAttrs(Attributes* arg_attrs)
 			InitEncrypt(log_encryption_key->AsString()->CheckString());
 		}
 
-	if ( attrs->FindAttr(ATTR_DISABLE_PRINT_HOOK) )
-		DisablePrintHook();
-
 	if ( attrs->FindAttr(ATTR_RAW_OUTPUT) )
 		EnableRawOutput();
-	
+
 	InstallRotateTimer();
 	}
 
@@ -531,6 +547,10 @@ RecordVal* BroFile::Rotate()
 	if ( ! is_open )
 		return 0;
 
+	// Do not rotate stdin/stdout/stderr.
+	if ( f == stdin || f == stdout || f == stderr )
+		return 0;
+
 	if ( okay_to_manage && ! is_in_cache )
 		BringIntoCache();
 
@@ -580,8 +600,9 @@ void BroFile::InstallRotateTimer()
 			const char* base_time = log_rotate_base_time ?
 				log_rotate_base_time->AsString()->CheckString() : 0;
 
+			double base = parse_rotate_base_time(base_time);
 			double delta_t =
-				calc_next_rotate(rotate_interval, base_time);
+				calc_next_rotate(network_time, rotate_interval, base);
 			rotate_timer = new RotateTimer(network_time + delta_t,
 							this, true);
 			}
@@ -638,19 +659,6 @@ void BroFile::CloseCachedFiles()
 		}
 	}
 
-#ifndef USE_OPENSSL
-
-void BroFile::InitEncrypt(const char* keyfile)
-	{
-	if ( keyfile )
-		{
-		error("file encryption requested, but OpenSSL support not compiled in.");
-		Close();
-		}
-	}
-
-#else
-
 void BroFile::InitEncrypt(const char* keyfile)
 	{
 	if ( ! (pub_key || keyfile) )
@@ -662,7 +670,7 @@ void BroFile::InitEncrypt(const char* keyfile)
 
 		if ( ! key )
 			{
-			error(fmt("can't open key file %s: %s", keyfile, strerror(errno)));
+			reporter->Error("can't open key file %s: %s", keyfile, strerror(errno));
 			Close();
 			return;
 			}
@@ -670,8 +678,8 @@ void BroFile::InitEncrypt(const char* keyfile)
 		pub_key = PEM_read_PUBKEY(key, 0, 0, 0);
 		if ( ! pub_key )
 			{
-			error(fmt("can't read key from %s: %s", keyfile,
-					ERR_error_string(ERR_get_error(), 0)));
+			reporter->Error("can't read key from %s: %s", keyfile,
+					ERR_error_string(ERR_get_error(), 0));
 			Close();
 			return;
 			}
@@ -684,7 +692,7 @@ void BroFile::InitEncrypt(const char* keyfile)
 
 	unsigned char secret[EVP_PKEY_size(pub_key)];
 	unsigned char* psecret = secret;
-	unsigned long secret_len;
+	unsigned int secret_len;
 
 	int iv_len = EVP_CIPHER_iv_length(cipher_type);
 	unsigned char iv[iv_len];
@@ -692,8 +700,8 @@ void BroFile::InitEncrypt(const char* keyfile)
 	if ( ! EVP_SealInit(cipher_ctx, cipher_type, &psecret,
 				(int*) &secret_len, iv, &pub_key, 1) )
 		{
-		error(fmt("can't init cipher context for %s: %s", keyfile,
-				ERR_error_string(ERR_get_error(), 0)));
+		reporter->Error("can't init cipher context for %s: %s", keyfile,
+				ERR_error_string(ERR_get_error(), 0));
 		Close();
 		return;
 		}
@@ -705,8 +713,8 @@ void BroFile::InitEncrypt(const char* keyfile)
 		fwrite(secret, ntohl(secret_len), 1, f) &&
 		fwrite(iv, iv_len, 1, f)) )
 		{
-		error(fmt("can't write header to log file %s: %s",
-				name, strerror(errno)));
+		reporter->Error("can't write header to log file %s: %s",
+				name, strerror(errno));
 		Close();
 		return;
 		}
@@ -714,14 +722,12 @@ void BroFile::InitEncrypt(const char* keyfile)
 	int buf_size = MIN_BUFFER_SIZE + EVP_CIPHER_block_size(cipher_type);
 	cipher_buffer = new unsigned char[buf_size];
 	}
-#endif
 
 void BroFile::FinishEncrypt()
 	{
 	if ( ! is_open )
 		return;
 
-#ifdef USE_OPENSSL
 	if ( ! pub_key )
 		return;
 
@@ -732,15 +738,14 @@ void BroFile::FinishEncrypt()
 
 		if ( outl && ! fwrite(cipher_buffer, outl, 1, f) )
 			{
-			run_time(fmt("write error for %s: %s",
-					name, strerror(errno)));
+			reporter->Error("write error for %s: %s",
+					name, strerror(errno));
 			return;
 			}
 
 		delete cipher_ctx;
 		cipher_ctx = 0;
 		}
-#endif
 	}
 
 
@@ -755,28 +760,27 @@ int BroFile::Write(const char* data, int len)
 	if ( ! len )
 		len = strlen(data);
 
-#ifdef USE_OPENSSL
 	if ( cipher_ctx )
 		{
 		while ( len )
 			{
 			int outl;
-			int inl = min(MIN_BUFFER_SIZE, len);
+			int inl = min(+MIN_BUFFER_SIZE, len);
 
 			if ( ! EVP_SealUpdate(cipher_ctx, cipher_buffer, &outl,
 						(unsigned char*)data, inl) )
 				{
-				run_time(fmt("encryption error for %s: %s",
+				reporter->Error("encryption error for %s: %s",
 					name,
-					ERR_error_string(ERR_get_error(), 0)));
+					ERR_error_string(ERR_get_error(), 0));
 				Close();
 				return 0;
 				}
 
 			if ( outl && ! fwrite(cipher_buffer, outl, 1, f) )
 				{
-				run_time(fmt("write error for %s: %s",
-						name, strerror(errno)));
+				reporter->Error("write error for %s: %s",
+						name, strerror(errno));
 				Close();
 				return 0;
 				}
@@ -787,7 +791,6 @@ int BroFile::Write(const char* data, int len)
 
 		return 1;
 		}
-#endif
 
 	len = fwrite(data, 1, len, f);
 	if ( len <= 0 )
@@ -824,7 +827,7 @@ void BroFile::UpdateFileSize()
 	struct stat s;
 	if ( fstat(fileno(f), &s) < 0 )
 		{
-		run_time(fmt("can't stat fd for %s: %s", name, strerror(errno)));
+		reporter->Error("can't stat fd for %s: %s", name, strerror(errno));
 		current_size = 0;
 		return;
 		}
@@ -872,8 +875,8 @@ BroFile* BroFile::Unserialize(UnserialInfo* info)
 			}
 		}
 
-	// Otherwise, open.
-	if ( ! file->Open() )
+	// Otherwise, open, but don't clobber.
+	if ( ! file->Open(0, "a") )
 		{
 		info->s->Error(fmt("cannot open %s: %s",
 					file->name, strerror(errno)));
