@@ -1,5 +1,3 @@
-// $Id: Func.h 6916 2009-09-24 20:48:36Z vern $
-//
 // See the file "COPYING" in the main distribution directory for copyright.
 
 #ifndef func_h
@@ -15,19 +13,19 @@ class FuncType;
 class Stmt;
 class Frame;
 class ID;
+class CallExpr;
 
 class Func : public BroObj {
 public:
 
 	enum Kind { BRO_FUNC, BUILTIN_FUNC };
 
-	Func(Kind arg_kind)
-		{ scope = 0; kind = arg_kind; id = 0; return_value = 0; }
+	Func(Kind arg_kind);
 
 	virtual ~Func();
 
 	virtual int IsPure() const = 0;
-	int IsEvent() const	{ return FType()->IsEvent(); }
+	function_flavor Flavor() const	{ return FType()->Flavor(); }
 
 	struct Body {
 		Stmt* stmts;
@@ -36,7 +34,8 @@ public:
 			{ return priority > other.priority; } // reverse sort
 	};
 
-	virtual const vector<Body>& GetBodies() const	{ return bodies; }
+	const vector<Body>& GetBodies() const	{ return bodies; }
+	bool HasBodies() const	{ return bodies.size(); }
 
 	// virtual Val* Call(ListExpr* args) const = 0;
 	virtual Val* Call(val_list* args, Frame* parent = 0) const = 0;
@@ -48,15 +47,11 @@ public:
 	virtual void SetScope(Scope* newscope)	{ scope = newscope; }
 	virtual Scope* GetScope() const		{ return scope; }
 
-	virtual FuncType* FType() const
-		{
-		return (FuncType*) id->Type()->AsFuncType();
-		}
+	virtual FuncType* FType() const { return type->AsFuncType(); }
 
 	Kind GetKind() const	{ return kind; }
 
-	const ID* GetID() const { return id; }
-	void SetID(ID *arg_id);
+	const char* Name() const { return name.c_str(); }
 
 	virtual void Describe(ODesc* d) const = 0;
 	virtual void DescribeDebug(ODesc* d, const val_list* args) const;
@@ -65,25 +60,30 @@ public:
 	bool Serialize(SerialInfo* info) const;
 	static Func* Unserialize(UnserialInfo* info);
 
-	ID* GetReturnValueID() const;
 	virtual TraversalCode Traverse(TraversalCallback* cb) const;
 
+	uint32 GetUniqueFuncID() const { return unique_id; }
+	static Func* GetFuncPtrByID(uint32 id)
+		{ return id >= unique_ids.size() ? 0 : unique_ids[id]; }
+
 protected:
-	Func()	{ scope = 0; id = 0; return_value = 0; }
+	Func();
 
 	DECLARE_ABSTRACT_SERIAL(Func);
 
 	vector<Body> bodies;
 	Scope* scope;
 	Kind kind;
-	ID* id;
-	ID* return_value;
+	BroType* type;
+	string name;
+	uint32 unique_id;
+	static vector<Func*> unique_ids;
 };
 
 
 class BroFunc : public Func {
 public:
-	BroFunc(ID* id, Stmt* body, id_list* inits, int frame_size);
+	BroFunc(ID* id, Stmt* body, id_list* inits, int frame_size, int priority);
 	~BroFunc();
 
 	int IsPure() const;
@@ -114,23 +114,21 @@ public:
 
 	int IsPure() const;
 	Val* Call(val_list* args, Frame* parent) const;
-	const char* Name() const	{ return name; }
 	built_in_func TheFunc() const	{ return func; }
 
 	void Describe(ODesc* d) const;
 
 protected:
-	BuiltinFunc()	{ func = 0; name = 0; is_pure = 0; }
+	BuiltinFunc()	{ func = 0; is_pure = 0; }
 
 	DECLARE_SERIAL(BuiltinFunc);
 
 	built_in_func func;
-	const char* name;
 	int is_pure;
 };
 
 
-extern void builtin_run_time(const char* msg, BroObj* arg = 0);
+extern void builtin_error(const char* msg, BroObj* arg = 0);
 extern void init_builtin_funcs();
 
 extern bool check_built_in_call(BuiltinFunc* f, CallExpr* call);
