@@ -1,15 +1,13 @@
-// $Id: Net.cc 6915 2009-09-22 05:04:17Z vern $
-//
 // See the file "COPYING" in the main distribution directory for copyright.
 
 #include "config.h"
 
 #include <sys/types.h>
-#if TIME_WITH_SYS_TIME
+#ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
 #else
-# if HAVE_SYS_TIME_H
+# ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>
 # else
 #  include <time.h>
@@ -26,18 +24,13 @@
 #include "Event.h"
 #include "Timer.h"
 #include "Var.h"
-#include "Logger.h"
+#include "Reporter.h"
 #include "Net.h"
-#include "TCP_Rewriter.h"
 #include "Anon.h"
 #include "PacketSort.h"
 #include "Serializer.h"
 #include "PacketDumper.h"
 
-#ifdef USE_DAG
-#include "PktDagSrc.h"
-#endif
-
 extern "C" {
 #include "setsignal.h"
 };
@@ -49,21 +42,12 @@ extern int select(int, fd_set *, fd_set *, fd_set *, struct timeval *);
 PList(PktSrc) pkt_srcs;
 
 // FIXME: We should really merge PktDumper and PacketDumper.
-// It's on my to-do [Robin].
 PktDumper* pkt_dumper = 0;
-PktDumper* pkt_transformed_dumper = 0;
-
-// For trace of rewritten packets
-PacketDumper* transformed_pkt_dump = 0;
-// For trace of original packets from selected connections
-PacketDumper* source_pkt_dump = 0;
-int transformed_pkt_dump_MTU = 1514;
 
 int reading_live = 0;
 int reading_traces = 0;
 int have_pending_timers = 0;
 double pseudo_realtime = 0.0;
-char* user_pcap_filter = 0;
 bool using_communication = false;
 
 double network_time = 0.0;	// time according to last packet timestamp
@@ -84,6 +68,8 @@ double current_timestamp = 0.0;
 PktSrc* current_pktsrc = 0;
 IOSource* current_iosrc;
 
+std::list<ScannedFile> files_scanned;
+std::vector<string> sig_files;
 
 RETSIGTYPE watchdog(int /* signo */)
 	{
@@ -121,15 +107,6 @@ RETSIGTYPE watchdog(int /* signo */)
 			int frac_pst =
 				int((processing_start_time - int_pst) * 1e6);
 
-			char msg[512];
-			safe_snprintf(msg, sizeof(msg),
-				      "**watchdog timer expired, t = %d.%06d, start = %d.%06d, dispatched = %d",
-				      int_ct, frac_ct, int_pst, frac_pst,
-				      current_dispatched);
-
-			bro_logger->Log(msg);
-			run_time("watchdog timer expired");
-
 			if ( current_hdr )
 				{
 				if ( ! pkt_dumper )
@@ -141,7 +118,8 @@ RETSIGTYPE watchdog(int /* signo */)
 					pkt_dumper = new PktDumper("watchdog-pkt.pcap");
 					if ( pkt_dumper->IsError() )
 						{
-						fprintf(stderr, "watchdog: can't open watchdog-pkt.pcap for writing\n");
+						reporter->Error("watchdog: can't open watchdog-pkt.pcap for writing\n");
+						delete pkt_dumper;
 						pkt_dumper = 0;
 						}
 					}
@@ -153,8 +131,10 @@ RETSIGTYPE watchdog(int /* signo */)
 			net_get_final_stats();
 			net_finish(0);
 
-			abort();
-			exit(1);
+			reporter->FatalErrorWithCore(
+			          "**watchdog timer expired, t = %d.%06d, start = %d.%06d, dispatched = %d",
+				      int_ct, frac_ct, int_pst, frac_pst,
+					  current_dispatched);
 			}
 		}
 
@@ -164,11 +144,10 @@ RETSIGTYPE watchdog(int /* signo */)
 	return RETSIGVAL;
 	}
 
-void net_init(name_list& interfaces, name_list& readfiles, 
+void net_init(name_list& interfaces, name_list& readfiles,
 	      name_list& netflows, name_list& flowfiles,
-	        const char* writefile, const char* transformed_writefile,
-	        const char* filter, const char* secondary_filter,
-		int do_watchdog)
+	        const char* writefile, const char* filter,
+			const char* secondary_filter, int do_watchdog)
 	{
 	init_net_var();
 
@@ -182,11 +161,8 @@ void net_init(name_list& interfaces, name_list& readfiles,
 			PktFileSrc* ps = new PktFileSrc(readfiles[i], filter);
 
 			if ( ! ps->IsOpen() )
-				{
-				fprintf(stderr, "%s: problem with trace file %s - %s\n",
+				reporter->FatalError("%s: problem with trace file %s - %s\n",
 					prog, readfiles[i], ps->ErrorMsg());
-				exit(1);
-				}
 			else
 				{
 				pkt_srcs.append(ps);
@@ -202,12 +178,9 @@ void net_init(name_list& interfaces, name_list& readfiles,
 							TYPE_FILTER_SECONDARY);
 
 				if ( ! ps->IsOpen() )
-					{
-					fprintf(stderr, "%s: problem with trace file %s - %s\n",
+					reporter->FatalError("%s: problem with trace file %s - %s\n",
 						prog, readfiles[i],
 						ps->ErrorMsg());
-					exit(1);
-					}
 				else
 					{
 					pkt_srcs.append(ps);
@@ -223,11 +196,8 @@ void net_init(name_list& interfaces, name_list& readfiles,
 			FlowFileSrc* fs = new FlowFileSrc(flowfiles[i]);
 
 			if ( ! fs->IsOpen() )
-				{
-				fprintf(stderr, "%s: problem with netflow file %s - %s\n",
+				reporter->FatalError("%s: problem with netflow file %s - %s\n",
 					prog, flowfiles[i], fs->ErrorMsg());
-				exit(1);
-				}
 			else
 				{
 				io_sources.Register(fs);
@@ -243,19 +213,11 @@ void net_init(name_list& interfaces, name_list& readfiles,
 		for ( int i = 0; i < interfaces.length(); ++i )
 			{
 			PktSrc* ps;
-#ifdef USE_DAG
-			if ( strncmp(interfaces[i], "dag", 3) == 0 )
-				ps = new PktDagSrc(interfaces[i], filter);
-			else
-#endif
-				ps = new PktInterfaceSrc(interfaces[i], filter);
+			ps = new PktInterfaceSrc(interfaces[i], filter);
 
 			if ( ! ps->IsOpen() )
-				{
-				fprintf(stderr, "%s: problem with interface %s - %s\n",
+				reporter->FatalError("%s: problem with interface %s - %s\n",
 					prog, interfaces[i], ps->ErrorMsg());
-				exit(1);
-				}
 			else
 				{
 				pkt_srcs.append(ps);
@@ -265,22 +227,13 @@ void net_init(name_list& interfaces, name_list& readfiles,
 			if ( secondary_filter )
 				{
 				PktSrc* ps;
-#ifdef USE_DAG
-				if ( strncmp(interfaces[i], "dag", 3) == 0 )
-					ps = new PktDagSrc(interfaces[i],
-						filter, TYPE_FILTER_SECONDARY);
-				else
-#endif
-					ps = new PktInterfaceSrc(interfaces[i],
-						filter, TYPE_FILTER_SECONDARY);
+				ps = new PktInterfaceSrc(interfaces[i],
+					filter, TYPE_FILTER_SECONDARY);
 
 				if ( ! ps->IsOpen() )
-					{
-					fprintf(stderr, "%s: problem with interface %s - %s\n",
+					reporter->Error("%s: problem with interface %s - %s\n",
 						prog, interfaces[i],
 						ps->ErrorMsg());
-					exit(1);
-					}
 				else
 					{
 					pkt_srcs.append(ps);
@@ -297,14 +250,13 @@ void net_init(name_list& interfaces, name_list& readfiles,
 
 			if ( ! fs->IsOpen() )
 				{
-				fprintf(stderr, "%s: problem with netflow socket %s - %s\n",
+				reporter->Error("%s: problem with netflow socket %s - %s\n",
 					prog, netflows[i], fs->ErrorMsg());
-				exit(1);
+				delete fs;
 				}
+
 			else
-				{
 				io_sources.Register(fs);
-				}
 			}
 
 		}
@@ -322,50 +274,17 @@ void net_init(name_list& interfaces, name_list& readfiles,
 		// interfaces with different-lengthed media.
 		pkt_dumper = new PktDumper(writefile);
 		if ( pkt_dumper->IsError() )
-			{
-			fprintf(stderr, "%s: can't open write file \"%s\" - %s\n",
+			reporter->FatalError("%s: can't open write file \"%s\" - %s\n",
 				prog, writefile, pkt_dumper->ErrorMsg());
-			exit(1);
-			}
 
 		ID* id = global_scope()->Lookup("trace_output_file");
 		if ( ! id )
-			run_time("trace_output_file not defined in bro.init");
+			reporter->Error("trace_output_file not defined in bro.init");
 		else
 			id->SetVal(new StringVal(writefile));
 		}
 
-	if ( transformed_writefile )
-		{
-		pkt_transformed_dumper = new PktDumper(transformed_writefile);
-		if ( pkt_transformed_dumper->IsError() )
-			{
-			fprintf(stderr, "%s: can't open trace transformation write file \"%s\" - %s\n",
-				prog, writefile,
-				pkt_transformed_dumper->ErrorMsg());
-			exit(1);
-			}
-
-		transformed_pkt_dump =
-			new PacketDumper(pkt_transformed_dumper->PcapDumper());
-
-		// If both -A and -w are specified, -A will be the transformed
-		// trace file and -w will be the source packet trace file.
-		// Otherwise the packets will go to the same file.
-		if ( pkt_dumper )
-			source_pkt_dump =
-				new PacketDumper(pkt_dumper->PcapDumper());
-		}
-
-	else if ( pkt_dumper )
-		transformed_pkt_dump =
-			new PacketDumper(pkt_dumper->PcapDumper());
-
-	if ( anonymize_ip_addr )
-		init_ip_addr_anonymizers();
-	else
-		for ( int i = 0; i < NUM_ADDR_ANONYMIZATION_METHODS; ++i )
-			ip_anonymizer[i] = 0;
+	init_ip_addr_anonymizers();
 
 	if ( packet_sort_window > 0 )
 		packet_sorter = new PacketSortGlobalPQ();
@@ -421,7 +340,7 @@ void net_packet_dispatch(double t, const struct pcap_pkthdr* hdr,
 		if ( load_freq == 0 )
 			load_freq = uint32(0xffffffff) / uint32(load_sample_freq);
 
-		if ( uint32(random() & 0xffffffff) < load_freq )
+		if ( uint32(bro_random() & 0xffffffff) < load_freq )
 			{
 			// Drain the queued timer events so they're not
 			// charged against this sample.
@@ -503,7 +422,8 @@ void net_run()
 	set_processing_status("RUNNING", "net_run");
 
 	while ( io_sources.Size() ||
-		(packet_sorter && ! packet_sorter->Empty()) )
+		(packet_sorter && ! packet_sorter->Empty()) ||
+		(BifConst::exit_only_after_terminate && ! terminating) )
 		{
 		double ts;
 		IOSource* src = io_sources.FindSoonest(&ts);
@@ -538,6 +458,7 @@ void net_run()
 				// date on timers and events.
 				network_time = ct;
 				expire_timers();
+				usleep(1); // Just yield.
 				}
 			}
 
@@ -567,6 +488,8 @@ void net_run()
 			// since Bro timers are not high-precision anyway.)
 			if ( ! using_communication )
 				usleep(100000);
+			else
+				usleep(1000);
 
 			// Flawfinder says about usleep:
 			//
@@ -620,7 +543,7 @@ void net_get_final_stats()
 			{
 			struct PktSrc::Stats s;
 			ps->Statistics(&s);
-			fprintf(stderr, "%d packets received on interface %s, %d dropped\n",
+			reporter->Info("%d packets received on interface %s, %d dropped\n",
 					s.received, ps->Interface(), s.dropped);
 			}
 		}
@@ -642,9 +565,6 @@ void net_finish(int drain_events)
 		}
 
 	delete pkt_dumper;
-	delete pkt_transformed_dumper;
-
-	// fprintf(stderr, "uhash: %d/%d\n", hash_cnt_uhash, hash_cnt_all);
 
 #ifdef DEBUG
 	extern int reassem_seen_bytes, reassem_copied_bytes;
@@ -663,11 +583,6 @@ void net_delete()
 	delete sessions;
 	delete packet_sorter;
 
-	// Can't put this in net_finish() because packets might be
-	// dumped when connections are deleted.
-	if ( transformed_pkt_dump )
-		delete transformed_pkt_dump;
-
 	for ( int i = 0; i < NUM_ADDR_ANONYMIZATION_METHODS; ++i )
 		delete ip_anonymizer[i];
 	}
@@ -702,7 +617,7 @@ static double suspend_start = 0;
 void net_suspend_processing()
 	{
 	if ( _processing_suspended == 0 )
-		bro_logger->Log("processing suspended");
+		reporter->Info("processing suspended");
 
 	++_processing_suspended;
 	}
@@ -711,7 +626,7 @@ void net_continue_processing()
 	{
 	if ( _processing_suspended == 1 )
 		{
-		bro_logger->Log("processing continued");
+		reporter->Info("processing continued");
 		loop_over_list(pkt_srcs, i)
 			pkt_srcs[i]->ContinueAfterSuspend();
 		}
