@@ -1,5 +1,3 @@
-// $Id: NetVar.cc 6887 2009-08-20 05:17:33Z vern $
-//
 // See the file "COPYING" in the main distribution directory for copyright.
 
 #include "config.h"
@@ -11,6 +9,7 @@ RecordType* conn_id;
 RecordType* endpoint;
 RecordType* endpoint_stats;
 RecordType* connection_type;
+RecordType* fa_file_type;
 RecordType* icmp_conn;
 RecordType* icmp_context;
 RecordType* SYN_packet;
@@ -18,11 +17,11 @@ RecordType* pcap_packet;
 RecordType* signature_state;
 EnumType* transport_proto;
 TableType* string_set;
-
-RecordType* net_stats;
+TableType* string_array;
+TableType* count_set;
+VectorType* string_vec;
 
 int watchdog_interval;
-double heartbeat_interval;
 
 int max_timer_expires;
 int max_remote_events_processed;
@@ -33,7 +32,6 @@ int tcp_SYN_ack_ok;
 int tcp_match_undelivered;
 
 int encap_hdr_size;
-int udp_tunnel_port;
 
 double frag_timeout;
 
@@ -49,17 +47,10 @@ int tcp_max_initial_window;
 int tcp_max_above_hole_without_any_acks;
 int tcp_excessive_data_without_further_acks;
 
-int ssl_compare_cipherspecs;
-int ssl_analyze_certificates;
-int ssl_store_certificates;
-int ssl_verify_certificates;
-int ssl_store_key_material;
-int ssl_max_cipherspec_size;
-StringVal* ssl_store_cert_path;
-StringVal* x509_trusted_cert_path;
-TableType* cipher_suites_list;
 RecordType* x509_type;
 
+RecordType* socks_address;
+
 double non_analyzed_lifetime;
 double tcp_inactivity_timeout;
 double udp_inactivity_timeout;
@@ -103,18 +94,11 @@ RecordType* http_stats_rec;
 RecordType* http_message_stat;
 int truncate_http_URI;
 
-int pm_request;
 RecordType* pm_mapping;
 TableType* pm_mappings;
 RecordType* pm_port_request;
 RecordType* pm_callit_request;
 
-RecordType* nfs3_attrs;
-RecordType* nfs3_opt_attrs;
-RecordType* nfs3_lookup_args;
-RecordType* nfs3_lookup_reply;
-RecordType* nfs3_fsstat;
-
 RecordType* ntp_msg;
 
 TableVal* samba_cmds;
@@ -126,6 +110,8 @@ TableType* smb_negotiate;
 
 RecordType* geo_location;
 
+RecordType* entropy_test_result;
+
 TableType* dhcp_router_list;
 RecordType* dhcp_msg;
 
@@ -182,6 +168,7 @@ TableVal* preserve_orig_addr;
 TableVal* preserve_resp_addr;
 TableVal* preserve_other_addr;
 
+int max_files_in_cache;
 double log_rotate_interval;
 double log_max_size;
 RecordType* rotate_info;
@@ -198,10 +185,6 @@ StringVal* ssl_ca_certificate;
 StringVal* ssl_private_key;
 StringVal* ssl_passphrase;
 
-StringVal* x509_crl_file;
-TableType* x509_extension;
-TableType* SSL_sessionID;
-
 Val* profiling_file;
 double profiling_interval;
 int expensive_profiling_multiple;
@@ -221,16 +204,10 @@ int sig_max_group_size;
 
 int enable_syslog;
 
-int use_connection_compressor;
-int cc_handle_resets;
-int cc_handle_only_syns;
-int cc_instantiate_on_data;
-
 TableType* irc_join_list;
 RecordType* irc_join_info;
 TableVal* irc_servers;
 
-TableVal* dpd_config;
 int dpd_reassemble_first_packets;
 int dpd_buffer_size;
 int dpd_match_only_beginning;
@@ -247,7 +224,6 @@ int dump_used_event_handlers;
 int suppress_local_output;
 
 double timer_mgr_inactivity_timeout;
-double expected_connection_timeout;
 
 int time_machine_profiling;
 
@@ -257,9 +233,21 @@ int record_all_packets;
 
 RecordType* script_id;
 TableType* id_table;
+RecordType* record_field;
+TableType* record_field_table;
+
+StringVal* cmd_line_bpf_filter;
+
+StringVal* global_hash_seed;
+
+bro_uint_t bits_per_uid;
 
 #include "const.bif.netvar_def"
+#include "types.bif.netvar_def"
 #include "event.bif.netvar_def"
+#include "logging.bif.netvar_def"
+#include "input.bif.netvar_def"
+#include "reporter.bif.netvar_def"
 
 void init_event_handlers()
 	{
@@ -275,6 +263,7 @@ void init_general_global_var()
 	state_dir = internal_val("state_dir")->AsStringVal();
 	state_write_delay = opt_internal_double("state_write_delay");
 
+	max_files_in_cache = opt_internal_int("max_files_in_cache");
 	log_rotate_interval = opt_internal_double("log_rotate_interval");
 	log_max_size = opt_internal_double("log_max_size");
 	rotate_info = internal_type("rotate_info")->AsRecordType();
@@ -295,7 +284,7 @@ void init_general_global_var()
 	ssl_passphrase = internal_val("ssl_passphrase")->AsStringVal();
 
 	packet_filter_default = opt_internal_int("packet_filter_default");
-	
+
 	sig_max_group_size = opt_internal_int("sig_max_group_size");
 	enable_syslog = opt_internal_int("enable_syslog");
 
@@ -309,16 +298,28 @@ void init_general_global_var()
 	trace_output_file = internal_val("trace_output_file")->AsStringVal();
 
 	record_all_packets = opt_internal_int("record_all_packets");
+
+	cmd_line_bpf_filter =
+		internal_val("cmd_line_bpf_filter")->AsStringVal();
+
+	global_hash_seed = opt_internal_string("global_hash_seed");
+
+	bits_per_uid = opt_internal_unsigned("bits_per_uid");
 	}
 
 void init_net_var()
 	{
 #include "const.bif.netvar_init"
+#include "types.bif.netvar_init"
+#include "logging.bif.netvar_init"
+#include "input.bif.netvar_init"
+#include "reporter.bif.netvar_init"
 
 	conn_id = internal_type("conn_id")->AsRecordType();
 	endpoint = internal_type("endpoint")->AsRecordType();
 	endpoint_stats = internal_type("endpoint_stats")->AsRecordType();
 	connection_type = internal_type("connection")->AsRecordType();
+	fa_file_type = internal_type("fa_file")->AsRecordType();
 	icmp_conn = internal_type("icmp_conn")->AsRecordType();
 	icmp_context = internal_type("icmp_context")->AsRecordType();
 	signature_state = internal_type("signature_state")->AsRecordType();
@@ -326,6 +327,8 @@ void init_net_var()
 	pcap_packet = internal_type("pcap_packet")->AsRecordType();
 	transport_proto = internal_type("transport_proto")->AsEnumType();
 	string_set = internal_type("string_set")->AsTableType();
+	string_array = internal_type("string_array")->AsTableType();
+	string_vec = internal_type("string_vec")->AsVectorType();
 
 	ignore_checksums = opt_internal_int("ignore_checksums");
 	partial_connection_ok = opt_internal_int("partial_connection_ok");
@@ -334,8 +337,6 @@ void init_net_var()
 
 	encap_hdr_size = opt_internal_int("encap_hdr_size");
 
-	udp_tunnel_port = opt_internal_int("udp_tunnel_port") & ~UDP_PORT_MASK;
-
 	frag_timeout = opt_internal_double("frag_timeout");
 
 	tcp_SYN_timeout = opt_internal_double("tcp_SYN_timeout");
@@ -352,20 +353,9 @@ void init_net_var()
 	tcp_excessive_data_without_further_acks =
 		opt_internal_int("tcp_excessive_data_without_further_acks");
 
-	ssl_compare_cipherspecs  = opt_internal_int("ssl_compare_cipherspecs");
-	ssl_analyze_certificates = opt_internal_int("ssl_analyze_certificates");
-	ssl_store_certificates   = opt_internal_int("ssl_store_certificates");
-	ssl_verify_certificates  = opt_internal_int("ssl_verify_certificates");
-	ssl_store_key_material = opt_internal_int("ssl_store_key_material");
-	ssl_max_cipherspec_size  = opt_internal_int("ssl_max_cipherspec_size");
-
-	x509_trusted_cert_path = opt_internal_string("X509_trusted_cert_path");
-	ssl_store_cert_path = opt_internal_string("ssl_store_cert_path");
 	x509_type = internal_type("X509")->AsRecordType();
-	cipher_suites_list = internal_type("cipher_suites_list")->AsTableType();
-	x509_crl_file = opt_internal_string("X509_crl_file");
-	x509_extension = internal_type("X509_extension")->AsTableType();
-	SSL_sessionID = internal_type("SSL_sessionID")->AsTableType();
+
+	socks_address = internal_type("SOCKS::Address")->AsRecordType();
 
 	non_analyzed_lifetime = opt_internal_double("non_analyzed_lifetime");
 	tcp_inactivity_timeout = opt_internal_double("tcp_inactivity_timeout");
@@ -403,10 +393,7 @@ void init_net_var()
 	ntp_session_timeout = opt_internal_double("ntp_session_timeout");
 	rpc_timeout = opt_internal_double("rpc_timeout");
 
-	net_stats = internal_type("net_stats")->AsRecordType();
-
 	watchdog_interval = int(opt_internal_double("watchdog_interval"));
-	heartbeat_interval = opt_internal_double("heartbeat_interval");
 
 	max_timer_expires = opt_internal_int("max_timer_expires");
 	max_remote_events_processed =
@@ -430,25 +417,11 @@ void init_net_var()
 	http_message_stat = internal_type("http_message_stat")->AsRecordType();
 	truncate_http_URI = opt_internal_int("truncate_http_URI");
 
-	pm_request = pm_request_null || pm_request_set ||
-		pm_request_unset || pm_request_getport ||
-		pm_request_dump || pm_request_callit ||
-		pm_attempt_null || pm_attempt_set ||
-		pm_attempt_unset || pm_attempt_getport ||
-		pm_attempt_dump || pm_attempt_callit ||
-		pm_bad_port;
-
 	pm_mapping = internal_type("pm_mapping")->AsRecordType();
 	pm_mappings = internal_type("pm_mappings")->AsTableType();
 	pm_port_request = internal_type("pm_port_request")->AsRecordType();
 	pm_callit_request = internal_type("pm_callit_request")->AsRecordType();
 
-	nfs3_attrs = internal_type("nfs3_attrs")->AsRecordType();
-	nfs3_opt_attrs = internal_type("nfs3_opt_attrs")->AsRecordType();
-	nfs3_lookup_args = internal_type("nfs3_lookup_args")->AsRecordType();
-	nfs3_lookup_reply = internal_type("nfs3_lookup_reply")->AsRecordType();
-	nfs3_fsstat = internal_type("nfs3_fsstat")->AsRecordType();
-
 	ntp_msg = internal_type("ntp_msg")->AsRecordType();
 
 	samba_cmds = internal_val("samba_cmds")->AsTableVal();
@@ -460,6 +433,8 @@ void init_net_var()
 
 	geo_location = internal_type("geo_location")->AsRecordType();
 
+	entropy_test_result = internal_type("entropy_test_result")->AsRecordType();
+
 	dhcp_router_list = internal_type("dhcp_router_list")->AsTableType();
 	dhcp_msg = internal_type("dhcp_msg")->AsRecordType();
 
@@ -529,12 +504,6 @@ void init_net_var()
 
 	gap_report_freq = opt_internal_double("gap_report_freq");
 
-	use_connection_compressor =
-		opt_internal_int("use_connection_compressor");
-	cc_handle_resets = opt_internal_int("cc_handle_resets");
-	cc_handle_only_syns = opt_internal_int("cc_handle_only_syns");
-	cc_instantiate_on_data = opt_internal_int("cc_instantiate_on_data");
-
 	irc_join_info = internal_type("irc_join_info")->AsRecordType();
 	irc_join_list = internal_type("irc_join_list")->AsTableType();
 	irc_servers = internal_val("irc_servers")->AsTableVal();
@@ -543,7 +512,6 @@ void init_net_var()
 		opt_internal_double("remote_trace_sync_interval");
 	remote_trace_sync_peers = opt_internal_int("remote_trace_sync_peers");
 
-	dpd_config = internal_val("dpd_config")->AsTableVal();
 	dpd_reassemble_first_packets =
 		opt_internal_int("dpd_reassemble_first_packets");
 	dpd_buffer_size = opt_internal_int("dpd_buffer_size");
@@ -554,10 +522,10 @@ void init_net_var()
 
 	timer_mgr_inactivity_timeout =
 		opt_internal_double("timer_mgr_inactivity_timeout");
-	expected_connection_timeout =
-		opt_internal_double("expected_connection_timeout");
 	time_machine_profiling = opt_internal_int("time_machine_profiling");
 
 	script_id = internal_type("script_id")->AsRecordType();
 	id_table = internal_type("id_table")->AsTableType();
+	record_field = internal_type("record_field")->AsRecordType();
+	record_field_table = internal_type("record_field_table")->AsTableType();
 	}
