@@ -1,5 +1,3 @@
-// $Id: NetVar.h 6887 2009-08-20 05:17:33Z vern $
-//
 // See the file "COPYING" in the main distribution directory for copyright.
 
 #ifndef netvar_h
@@ -14,6 +12,7 @@ extern RecordType* conn_id;
 extern RecordType* endpoint;
 extern RecordType* endpoint_stats;
 extern RecordType* connection_type;
+extern RecordType* fa_file_type;
 extern RecordType* icmp_conn;
 extern RecordType* icmp_context;
 extern RecordType* signature_state;
@@ -21,11 +20,11 @@ extern RecordType* SYN_packet;
 extern RecordType* pcap_packet;
 extern EnumType* transport_proto;
 extern TableType* string_set;
-
-extern RecordType* net_stats;
+extern TableType* string_array;
+extern TableType* count_set;
+extern VectorType* string_vec;
 
 extern int watchdog_interval;
-extern double heartbeat_interval;
 
 extern int max_timer_expires;
 extern int max_remote_events_processed;
@@ -36,7 +35,6 @@ extern int tcp_SYN_ack_ok;
 extern int tcp_match_undelivered;
 
 extern int encap_hdr_size;
-extern int udp_tunnel_port;
 
 extern double frag_timeout;
 
@@ -52,20 +50,9 @@ extern int tcp_max_initial_window;
 extern int tcp_max_above_hole_without_any_acks;
 extern int tcp_excessive_data_without_further_acks;
 
-// see policy/ssl.bro for details
-extern int ssl_compare_cipherspecs;
-extern int ssl_analyze_certificates;
-extern int ssl_store_certificates;
-extern int ssl_verify_certificates;
-extern int ssl_store_key_material;
-extern int ssl_max_cipherspec_size;
-extern StringVal* ssl_store_cert_path;
-extern StringVal* x509_trusted_cert_path;
-extern TableType* cipher_suites_list;
 extern RecordType* x509_type;
-extern StringVal* x509_crl_file;
-extern TableType* x509_extension;
-extern TableType* SSL_sessionID;
+
+extern RecordType* socks_address;
 
 extern double non_analyzed_lifetime;
 extern double tcp_inactivity_timeout;
@@ -110,18 +97,11 @@ extern RecordType* http_stats_rec;
 extern RecordType* http_message_stat;
 extern int truncate_http_URI;
 
-extern int pm_request;
 extern RecordType* pm_mapping;
 extern TableType* pm_mappings;
 extern RecordType* pm_port_request;
 extern RecordType* pm_callit_request;
 
-extern RecordType* nfs3_attrs;
-extern RecordType* nfs3_opt_attrs;
-extern RecordType* nfs3_lookup_args;
-extern RecordType* nfs3_lookup_reply;
-extern RecordType* nfs3_fsstat;
-
 extern RecordType* ntp_msg;
 
 extern TableVal* samba_cmds;
@@ -133,6 +113,8 @@ extern TableType* smb_negotiate;
 
 extern RecordType* geo_location;
 
+extern RecordType* entropy_test_result;
+
 extern TableType* dhcp_router_list;
 extern RecordType* dhcp_msg;
 
@@ -189,6 +171,7 @@ extern double connection_status_update_interval;
 extern StringVal* state_dir;
 extern double state_write_delay;
 
+extern int max_files_in_cache;
 extern double log_rotate_interval;
 extern double log_max_size;
 extern RecordType* rotate_info;
@@ -225,16 +208,10 @@ extern int sig_max_group_size;
 
 extern int enable_syslog;
 
-extern int use_connection_compressor;
-extern int cc_handle_resets;
-extern int cc_handle_only_syns;
-extern int cc_instantiate_on_data;
-
 extern TableType* irc_join_list;
 extern RecordType* irc_join_info;
 extern TableVal* irc_servers;
 
-extern TableVal* dpd_config;
 extern int dpd_reassemble_first_packets;
 extern int dpd_buffer_size;
 extern int dpd_match_only_beginning;
@@ -251,7 +228,6 @@ extern int dump_used_event_handlers;
 extern int suppress_local_output;
 
 extern double timer_mgr_inactivity_timeout;
-extern double expected_connection_timeout;
 
 extern int time_machine_profiling;
 
@@ -261,6 +237,14 @@ extern int record_all_packets;
 
 extern RecordType* script_id;
 extern TableType* id_table;
+extern RecordType* record_field;
+extern TableType* record_field_table;
+
+extern StringVal* cmd_line_bpf_filter;
+
+extern StringVal* global_hash_seed;
+
+extern bro_uint_t bits_per_uid;
 
 // Initializes globals that don't pertain to network/event analysis.
 extern void init_general_global_var();
@@ -269,6 +253,10 @@ extern void init_event_handlers();
 extern void init_net_var();
 
 #include "const.bif.netvar_h"
+#include "types.bif.netvar_h"
 #include "event.bif.netvar_h"
+#include "logging.bif.netvar_h"
+#include "input.bif.netvar_h"
+#include "reporter.bif.netvar_h"
 
 #endif
