@@ -1,5 +1,3 @@
-// $Id: OSFinger.cc 5857 2008-06-26 23:00:03Z vern $
-
 /*
   Taken with permission from:
 
@@ -65,15 +63,16 @@ OSFingerprint::OSFingerprint(FingerprintMode arg_mode)
     }
 }
 
-bool OSFingerprint::CacheMatch(uint32 addr, int id)
+bool OSFingerprint::CacheMatch(const IPAddr& addr, int id)
   {
-  HashKey key = HashKey(&addr, 1);
+  HashKey* key = addr.GetHashKey();
   int* pid = new int;
   *pid=id;
-  int* prev = os_matches.Insert(&key, pid);
+  int* prev = os_matches.Insert(key, pid);
   bool ret = (prev ? *prev != id : 1);
   if (prev)
     delete prev;
+  delete key;
   return ret;
   }
 
@@ -87,8 +86,8 @@ void OSFingerprint::collide(uint32 id)
   if (sig[id].ttl % 32 && sig[id].ttl != 255 && sig[id].ttl % 30)
     {
     problems=1;
-    warn(fmt("OS fingerprinting: [!] Unusual TTL (%d) for signature '%s %s' (line %d).",
-          sig[id].ttl,sig[id].os,sig[id].desc,sig[id].line));
+    reporter->Warning("OS fingerprinting: [!] Unusual TTL (%d) for signature '%s %s' (line %d).",
+          sig[id].ttl,sig[id].os,sig[id].desc,sig[id].line);
     }
 
   for (i=0;i<id;i++)
@@ -96,8 +95,8 @@ void OSFingerprint::collide(uint32 id)
     if (!strcmp(sig[i].os,sig[id].os) &&
         !strcmp(sig[i].desc,sig[id].desc)) {
       problems=1;
-      warn(fmt("OS fingerprinting: [!] Duplicate signature name: '%s %s' (line %d and %d).",
-            sig[i].os,sig[i].desc,sig[i].line,sig[id].line));
+      reporter->Warning("OS fingerprinting: [!] Duplicate signature name: '%s %s' (line %d and %d).",
+            sig[i].os,sig[i].desc,sig[i].line,sig[id].line);
     }
 
     /* If TTLs are sufficiently away from each other, the risk of
@@ -279,10 +278,10 @@ do_const:
       if (sig[id].opt[j] ^ sig[i].opt[j]) goto reloop;
 
     problems=1;
-    warn(fmt("OS fingerprinting: [!] Signature '%s %s' (line %d)\n"
+    reporter->Warning("OS fingerprinting: [!] Signature '%s %s' (line %d)\n"
           "    is already covered by '%s %s' (line %d).",
           sig[id].os,sig[id].desc,sig[id].line,sig[i].os,sig[i].desc,
-          sig[i].line));
+          sig[i].line);
 
 reloop:
     ;
@@ -295,7 +294,7 @@ void OSFingerprint::load_config(const char* file)
   uint32 ln=0;
   char buf[MAXLINE];
   char* p;
-  FILE* c = search_for_file( file, "osf", 0);
+  FILE* c = search_for_file(file, "osf", 0, false, 0);
 
   if (!c)
     {
