@@ -1,17 +1,15 @@
-// $Id: Reassem.h 6703 2009-05-13 22:27:44Z vern $
-//
 // See the file "COPYING" in the main distribution directory for copyright.
 
 #ifndef reassem_h
 #define reassem_h
 
-#include "Active.h"
 #include "Obj.h"
+#include "IPAddr.h"
 
 class DataBlock {
 public:
 	DataBlock(const u_char* data, int size, int seq,
-			DataBlock* next, DataBlock* prev);
+			DataBlock* prev, DataBlock* next);
 
 	~DataBlock();
 
@@ -28,8 +26,7 @@ enum ReassemblerType { REASSEM_IP, REASSEM_TCP };
 
 class Reassembler : public BroObj {
 public:
-	Reassembler(int init_seq, const uint32* ip_addr,
-			ReassemblerType arg_type);
+	Reassembler(int init_seq, ReassemblerType arg_type);
 	virtual ~Reassembler();
 
 	void NewBlock(double t, int seq, int len, const u_char* data);
@@ -74,8 +71,6 @@ protected:
 	int last_reassem_seq;
 	int trim_seq;	// how far we've trimmed
 
-	ReassemblyPolicy policy;
-
 	static unsigned int total_size;
 };
 
