@@ -1,5 +1,3 @@
-// $Id: SerialTypes.h 6752 2009-06-14 04:24:52Z vern $
-
 #ifndef serialtypes_h
 #define serialtypes_h
 
@@ -51,6 +49,10 @@ SERIAL_IS(STATE_ACCESS, 0x1100)
 SERIAL_IS_BO(CASE, 0x1200)
 SERIAL_IS(LOCATION, 0x1300)
 SERIAL_IS(RE_MATCHER, 0x1400)
+SERIAL_IS(BITVECTOR, 0x1500)
+SERIAL_IS(COUNTERVECTOR, 0x1600)
+SERIAL_IS(BLOOMFILTER, 0x1700)
+SERIAL_IS(HASHER, 0x1800)
 
 // These are the externally visible types.
 const SerialType SER_NONE = 0;
@@ -91,16 +93,24 @@ SERIAL_VAL(VAL, 1)
 SERIAL_VAL(INTERVAL_VAL,  2)
 SERIAL_VAL(PORT_VAL, 3)
 SERIAL_VAL(ADDR_VAL, 4)
-SERIAL_VAL(NET_VAL, 5)
-SERIAL_VAL(SUBNET_VAL, 6)
-SERIAL_VAL(STRING_VAL, 7)
-SERIAL_VAL(PATTERN_VAL, 8)
-SERIAL_VAL(LIST_VAL, 9)
-SERIAL_VAL(TABLE_VAL, 10)
-SERIAL_VAL(RECORD_VAL, 11)
-SERIAL_VAL(ENUM_VAL, 12)
-SERIAL_VAL(VECTOR_VAL, 13)
-SERIAL_VAL(MUTABLE_VAL, 14)
+SERIAL_VAL(SUBNET_VAL, 5)
+SERIAL_VAL(STRING_VAL, 6)
+SERIAL_VAL(PATTERN_VAL, 7)
+SERIAL_VAL(LIST_VAL, 8)
+SERIAL_VAL(TABLE_VAL, 9)
+SERIAL_VAL(RECORD_VAL, 10)
+SERIAL_VAL(ENUM_VAL, 11)
+SERIAL_VAL(VECTOR_VAL, 12)
+SERIAL_VAL(MUTABLE_VAL, 13)
+SERIAL_VAL(OPAQUE_VAL, 14)
+SERIAL_VAL(HASH_VAL, 15)
+SERIAL_VAL(MD5_VAL, 16)
+SERIAL_VAL(SHA1_VAL, 17)
+SERIAL_VAL(SHA256_VAL, 18)
+SERIAL_VAL(ENTROPY_VAL, 19)
+SERIAL_VAL(TOPK_VAL, 20)
+SERIAL_VAL(BLOOMFILTER_VAL, 21)
+SERIAL_VAL(CARDINALITY_VAL, 22)
 
 #define SERIAL_EXPR(name, val) SERIAL_CONST(name, val, EXPR)
 SERIAL_EXPR(EXPR, 1)
@@ -128,7 +138,7 @@ SERIAL_EXPR(FIELD_EXPR, 22)
 SERIAL_EXPR(HAS_FIELD_EXPR, 23)
 SERIAL_EXPR(RECORD_CONSTRUCTOR_EXPR, 24)
 SERIAL_EXPR(FIELD_ASSIGN_EXPR, 25)
-SERIAL_EXPR(RECORD_MATCH_EXPR, 26)
+// There used to be a SERIAL_EXPR(RECORD_MATCH_EXPR, 26) here
 SERIAL_EXPR(ARITH_COERCE_EXPR, 27)
 SERIAL_EXPR(RECORD_COERCE_EXPR, 28)
 SERIAL_EXPR(FLATTEN_EXPR, 29)
@@ -146,11 +156,12 @@ SERIAL_EXPR(TABLE_CONSTRUCTOR_EXPR, 40)
 SERIAL_EXPR(SET_CONSTRUCTOR_EXPR, 41)
 SERIAL_EXPR(VECTOR_CONSTRUCTOR_EXPR, 42)
 SERIAL_EXPR(TABLE_COERCE_EXPR, 43)
+SERIAL_EXPR(VECTOR_COERCE_EXPR, 44)
 
 #define SERIAL_STMT(name, val) SERIAL_CONST(name, val, STMT)
 SERIAL_STMT(STMT, 1)
 SERIAL_STMT(EXPR_LIST_STMT, 2)
-SERIAL_STMT(ALARM_STMT, 3)
+// There used to be ALARM_STMT (3) here.
 SERIAL_STMT(PRINT_STMT, 4)
 SERIAL_STMT(EXPR_STMT, 5)
 SERIAL_STMT(IF_STMT, 6)
@@ -167,6 +178,7 @@ SERIAL_STMT(EVENT_BODY_LIST, 16)
 SERIAL_STMT(INIT_STMT, 17)
 SERIAL_STMT(NULL_STMT, 18)
 SERIAL_STMT(WHEN_STMT, 19)
+SERIAL_STMT(FALLTHROUGH_STMT, 20)
 
 #define SERIAL_TYPE(name, val) SERIAL_CONST(name, val, BRO_TYPE)
 SERIAL_TYPE(BRO_TYPE, 1)
@@ -180,6 +192,7 @@ SERIAL_TYPE(SUBNET_TYPE, 8)
 SERIAL_TYPE(FILE_TYPE, 9)
 SERIAL_TYPE(ENUM_TYPE, 10)
 SERIAL_TYPE(VECTOR_TYPE, 11)
+SERIAL_TYPE(OPAQUE_TYPE, 12)
 
 SERIAL_CONST2(ATTRIBUTES)
 SERIAL_CONST2(EVENT_HANDLER)
@@ -191,10 +204,22 @@ SERIAL_FUNC(BRO_FUNC, 2)
 SERIAL_FUNC(DEBUG_FUNC, 3)
 SERIAL_FUNC(BUILTIN_FUNC, 4)
 
+#define SERIAL_BLOOMFILTER(name, val) SERIAL_CONST(name, val, BLOOMFILTER)
+SERIAL_BLOOMFILTER(BLOOMFILTER, 1)
+SERIAL_BLOOMFILTER(BASICBLOOMFILTER, 2)
+SERIAL_BLOOMFILTER(COUNTINGBLOOMFILTER, 3)
+
+#define SERIAL_HASHER(name, val) SERIAL_CONST(name, val, HASHER)
+SERIAL_HASHER(HASHER, 1)
+SERIAL_HASHER(DEFAULTHASHER, 2)
+SERIAL_HASHER(DOUBLEHASHER, 3)
+
 SERIAL_CONST2(ID)
 SERIAL_CONST2(STATE_ACCESS)
 SERIAL_CONST2(CASE)
 SERIAL_CONST2(LOCATION)
 SERIAL_CONST2(RE_MATCHER)
+SERIAL_CONST2(BITVECTOR)
+SERIAL_CONST2(COUNTERVECTOR)
 
 #endif
