@@ -1,14 +1,14 @@
-// $Id: SerializationFormat.cc 5873 2008-06-28 19:25:03Z vern $
-
 #include <ctype.h>
 
 #include "net_util.h"
 #include "SerializationFormat.h"
 #include "Serializer.h"
+#include "Reporter.h"
 
 SerializationFormat::SerializationFormat()
+    : output(), output_size(), output_pos(), input(), input_len(), input_pos(),
+      bytes_written(), bytes_read()
 	{
-	output = 0;
 	}
 
 SerializationFormat::~SerializationFormat()
@@ -21,6 +21,7 @@ void SerializationFormat::StartRead(char* data, uint32 arg_len)
 	input = data;
 	input_len = arg_len;
 	input_pos = 0;
+	bytes_read = 0;
 	}
 
 void SerializationFormat::EndRead()
@@ -57,13 +58,15 @@ bool SerializationFormat::ReadData(void* b, size_t count)
 	{
 	if ( input_pos + count > input_len )
 		{
-		error("data underflow during read in binary format");
+		reporter->Error("data underflow during read in binary format");
 		abort();
 		return false;
 		}
 
 	memcpy(b, input + input_pos, count);
 	input_pos += count;
+	bytes_read += count;
+
 	return true;
 	}
 
@@ -201,7 +204,7 @@ bool BinarySerializationFormat::Read(char** str, int* len, const char* tag)
 		for ( int i = 0; i < l; i++ )
 			if ( ! s[i] )
 				{
-				error("binary Format: string contains null; replaced by '_'");
+				reporter->Error("binary Format: string contains null; replaced by '_'");
 				s[i] = '_';
 				}
 		}
@@ -214,6 +217,85 @@ bool BinarySerializationFormat::Read(char** str, int* len, const char* tag)
 	return true;
 	}
 
+bool BinarySerializationFormat::Read(string* v, const char* tag)
+	{
+	char* buffer;
+	int len;
+
+	if ( ! Read(&buffer, &len, tag) )
+		return false;
+
+	*v = string(buffer, len);
+
+	delete [] buffer;
+	return true;
+	}
+
+bool BinarySerializationFormat::Read(IPAddr* addr, const char* tag)
+	{
+	int n = 0;
+	if ( ! Read(&n, "addr-len") )
+		return false;
+
+	if ( n != 1 && n != 4 )
+		return false;
+
+	uint32_t raw[4];
+
+	for ( int i = 0; i < n; ++i )
+		{
+		if ( ! Read(&raw[i], "addr-part") )
+			return false;
+
+		raw[i] = htonl(raw[i]);
+		}
+
+	if ( n == 1 )
+		*addr = IPAddr(IPv4, raw, IPAddr::Network);
+	else
+		*addr = IPAddr(IPv6, raw, IPAddr::Network);
+
+	return true;
+	}
+
+bool BinarySerializationFormat::Read(IPPrefix* prefix, const char* tag)
+	{
+	IPAddr addr;
+	int len;
+
+	if ( ! (Read(&addr, "prefix") && Read(&len, "width")) )
+		return false;
+
+	*prefix = IPPrefix(addr, len);
+	return true;
+	}
+
+bool BinarySerializationFormat::Read(struct in_addr* addr, const char* tag)
+	{
+	uint32_t* bytes = (uint32_t*) &addr->s_addr;
+
+	if ( ! Read(&bytes[0], "addr4") )
+		return false;
+
+	bytes[0] = htonl(bytes[0]);
+	return true;
+	}
+
+bool BinarySerializationFormat::Read(struct in6_addr* addr, const char* tag)
+	{
+	uint32_t* bytes = (uint32_t*) &addr->s6_addr;
+
+	for ( int i = 0; i < 4; ++i )
+		{
+		if ( ! Read(&bytes[i], "addr6-part") )
+			return false;
+
+		bytes[i] = htonl(bytes[i]);
+		}
+
+	return true;
+	}
+
 bool BinarySerializationFormat::Write(char v, const char* tag)
 	{
 	DBG_LOG(DBG_SERIAL, "Write char %s [%s]", fmt_bytes(&v, 1), tag);
@@ -278,6 +360,58 @@ bool BinarySerializationFormat::Write(const char* s, const char* tag)
 	return Write(s, strlen(s), tag);
 	}
 
+bool BinarySerializationFormat::Write(const string& s, const char* tag)
+	{
+	return Write(s.data(), s.size(), tag);
+	}
+
+bool BinarySerializationFormat::Write(const IPAddr& addr, const char* tag)
+	{
+	const uint32_t* raw;
+	int n = addr.GetBytes(&raw);
+
+	assert(n == 1 || n == 4);
+
+	if ( ! Write(n, "addr-len") )
+		return false;
+
+	for ( int i = 0; i < n; ++i )
+		{
+		if ( ! Write(ntohl(raw[i]), "addr-part") )
+			return false;
+		}
+
+	return true;
+	}
+
+bool BinarySerializationFormat::Write(const IPPrefix& prefix, const char* tag)
+	{
+	return Write(prefix.Prefix(), "prefix") && Write(prefix.Length(), "width");
+	}
+
+bool BinarySerializationFormat::Write(const struct in_addr& addr, const char* tag)
+	{
+	const uint32_t* bytes = (uint32_t*) &addr.s_addr;
+
+	if ( ! Write(ntohl(bytes[0]), "addr4") )
+		return false;
+
+	return true;
+	}
+
+bool BinarySerializationFormat::Write(const struct in6_addr& addr, const char* tag)
+	{
+	const uint32_t* bytes = (uint32_t*) &addr.s6_addr;
+
+	for ( int i = 0; i < 4; ++i )
+		{
+		if ( ! Write(ntohl(bytes[i]), "addr6-part") )
+			return false;
+		}
+
+	return true;
+	}
+
 bool BinarySerializationFormat::WriteOpenTag(const char* tag)
 	{
 	return true;
@@ -310,55 +444,85 @@ XMLSerializationFormat::~XMLSerializationFormat()
 
 bool XMLSerializationFormat::Read(int* v, const char* tag)
 	{
-	internal_error("no reading of xml");
+	reporter->InternalError("no reading of xml");
 	return false;
 	}
 
 bool XMLSerializationFormat::Read(uint16* v, const char* tag)
 	{
-	internal_error("no reading of xml");
+	reporter->InternalError("no reading of xml");
 	return false;
 	}
 
 bool XMLSerializationFormat::Read(uint32* v, const char* tag)
 	{
-	internal_error("no reading of xml");
+	reporter->InternalError("no reading of xml");
 	return false;
 	}
 
 bool XMLSerializationFormat::Read(int64* v, const char* tag)
 	{
-	internal_error("no reading of xml");
+	reporter->InternalError("no reading of xml");
 	return false;
 	}
 
 bool XMLSerializationFormat::Read(uint64* v, const char* tag)
 	{
-	internal_error("no reading of xml");
+	reporter->InternalError("no reading of xml");
 	return false;
 	}
 
 bool XMLSerializationFormat::Read(bool* v, const char* tag)
 	{
-	internal_error("no reading of xml");
+	reporter->InternalError("no reading of xml");
 	return false;
 	}
 
 bool XMLSerializationFormat::Read(double* d, const char* tag)
 	{
-	internal_error("no reading of xml");
+	reporter->InternalError("no reading of xml");
 	return false;
 	}
 
 bool XMLSerializationFormat::Read(char* v, const char* tag)
 	{
-	internal_error("no reading of xml");
+	reporter->InternalError("no reading of xml");
 	return false;
 	}
 
 bool XMLSerializationFormat::Read(char** str, int* len, const char* tag)
 	{
-	internal_error("no reading of xml");
+	reporter->InternalError("no reading of xml");
+	return false;
+	}
+
+bool XMLSerializationFormat::Read(string* s, const char* tag)
+	{
+	reporter->InternalError("no reading of xml");
+	return false;
+	}
+
+bool XMLSerializationFormat::Read(IPAddr* addr, const char* tag)
+	{
+	reporter->InternalError("no reading of xml");
+	return false;
+	}
+
+bool XMLSerializationFormat::Read(IPPrefix* prefix, const char* tag)
+	{
+	reporter->InternalError("no reading of xml");
+	return false;
+	}
+
+bool XMLSerializationFormat::Read(struct in_addr* addr, const char* tag)
+	{
+	reporter->InternalError("no reading of xml");
+	return false;
+	}
+
+bool XMLSerializationFormat::Read(struct in6_addr* addr, const char* tag)
+	{
+	reporter->InternalError("no reading of xml");
 	return false;
 	}
 
@@ -369,25 +533,25 @@ bool XMLSerializationFormat::Write(char v, const char* tag)
 
 bool XMLSerializationFormat::Write(uint16 v, const char* tag)
 	{
-	const char* tmp = fmt("%u", v);
+	const char* tmp = fmt("%" PRIu16, v);
 	return WriteElem(tag, "uint16", tmp, strlen(tmp));
 	}
 
 bool XMLSerializationFormat::Write(uint32 v, const char* tag)
 	{
-	const char* tmp = fmt("%u", v);
+	const char* tmp = fmt("%"PRIu32, v);
 	return WriteElem(tag, "uint32", tmp, strlen(tmp));
 	}
 
 bool XMLSerializationFormat::Write(uint64 v, const char* tag)
 	{
-	const char* tmp = fmt("%llu", v);
+	const char* tmp = fmt("%"PRIu64, v);
 	return WriteElem(tag, "uint64", tmp, strlen(tmp));
 	}
 
 bool XMLSerializationFormat::Write(int64 v, const char* tag)
 	{
-	const char* tmp = fmt("%lld", v);
+	const char* tmp = fmt("%"PRId64, v);
 	return WriteElem(tag, "int64", tmp, strlen(tmp));
 	}
 
@@ -416,6 +580,11 @@ bool XMLSerializationFormat::Write(const char* s, const char* tag)
 	return WriteElem(tag, "string", s, strlen(s));
 	}
 
+bool XMLSerializationFormat::Write(const string& s, const char* tag)
+	{
+	return WriteElem(tag, "string", s.data(), s.size());
+	}
+
 bool XMLSerializationFormat::WriteOpenTag(const char* tag)
 	{
 	return WriteData("<", 1) && WriteData(tag, strlen(tag) && WriteData(">", 1));
@@ -437,6 +606,30 @@ bool XMLSerializationFormat::Write(const char* buf, int len, const char* tag)
 	return WriteElem(tag, "string", buf, len);
 	}
 
+bool XMLSerializationFormat::Write(const IPAddr& addr, const char* tag)
+	{
+	reporter->InternalError("XML output of addresses not implemented");
+	return false;
+	}
+
+bool XMLSerializationFormat::Write(const IPPrefix& prefix, const char* tag)
+	{
+	reporter->InternalError("XML output of prefixes not implemented");
+	return false;
+	}
+
+bool XMLSerializationFormat::Write(const struct in_addr& addr, const char* tag)
+	{
+	reporter->InternalError("XML output of in_addr not implemented");
+	return false;
+	}
+
+bool XMLSerializationFormat::Write(const struct in6_addr& addr, const char* tag)
+	{
+	reporter->InternalError("XML output of in6_addr not implemented");
+	return false;
+	}
+
 bool XMLSerializationFormat::WriteEncodedString(const char* s, int len)
 	{
 	while ( len-- )
