@@ -1,5 +1,3 @@
-// $Id: Serializer.cc 6752 2009-06-14 04:24:52Z vern $
-
 #include <fcntl.h>
 #include <unistd.h>
 #include <errno.h>
@@ -12,7 +10,7 @@
 #include "Serializer.h"
 #include "Scope.h"
 #include "Stmt.h"
-#include "Logger.h"
+#include "Reporter.h"
 #include "Func.h"
 #include "Event.h"
 #include "EventRegistry.h"
@@ -72,12 +70,8 @@ bool Serializer::StartSerialization(SerialInfo* info, const char* descr,
 
 bool Serializer::EndSerialization(SerialInfo* info)
 	{
-	ChunkedIO::Chunk* chunk = new ChunkedIO::Chunk;
-	chunk->len = format->EndWrite(&chunk->data);
-
 	if ( info->chunk )
 		{
-
 		if ( ! io->Write(info->chunk) )
 			{
 			Error(io->Error());
@@ -85,6 +79,9 @@ bool Serializer::EndSerialization(SerialInfo* info)
 			}
 		}
 
+	ChunkedIO::Chunk* chunk = new ChunkedIO::Chunk;
+	chunk->len = format->EndWrite(&chunk->data);
+
 	if ( ! io->Write(chunk) )
 		{
 		Error(io->Error());
@@ -139,7 +136,14 @@ bool Serializer::Serialize(SerialInfo* info, const char* func, val_list* args)
 	Write(network_time, "time");
 	Write(a, "len");
 
-	loop_over_list(*args, i) (*args)[i]->Serialize(info);
+	loop_over_list(*args, i)
+		{
+		if ( ! (*args)[i]->Serialize(info) )
+			{
+			Error("failed");
+			return false;
+			}
+		}
 
 	WriteCloseTag("call");
 	WriteSeparator();
@@ -368,6 +372,7 @@ bool Serializer::UnserializeCall(UnserialInfo* info)
 		if ( ! v )
 			{
 			delete [] name;
+			delete_vals(args);
 			return false;
 			}
 
@@ -380,7 +385,7 @@ bool Serializer::UnserializeCall(UnserialInfo* info)
 				ignore = true;
 				}
 
-			if ( info->print && types && ! ignore )
+			if ( info->print && ! ignore )
 				v->Describe(&d);
 			}
 
@@ -391,23 +396,35 @@ bool Serializer::UnserializeCall(UnserialInfo* info)
 		{
 		if ( info->print )
 			fprintf(info->print, "%s [%.06f] %s(%s)\n",
-				functype->IsEvent() ? "Event" : "Function call",
+				functype->FlavorString().c_str(),
 				time, name, types ? d.Description() : "<ignored>");
 
-		if ( functype->IsEvent() )
+		switch ( functype->Flavor() ) {
+
+		case FUNC_FLAVOR_EVENT:
 			{
 			EventHandler* handler = event_registry->Lookup(name);
 			assert(handler);
+
 			if ( ! info->ignore_callbacks )
 				GotEvent(name, time, handler, args);
+
+			break;
 			}
-		else
+
+		case FUNC_FLAVOR_FUNCTION:
+		case FUNC_FLAVOR_HOOK:
 			if ( ! info->ignore_callbacks )
 				GotFunctionCall(name, time, id->ID_Val()->AsFunc(), args);
+			break;
+
+		default:
+			reporter->InternalError("unserialized call for invalid function flavor");
+			break;
+		}
 
 		if ( info->ignore_callbacks )
 			delete_vals(args);
-
 		}
 	else
 		delete_vals(args);
@@ -744,10 +761,11 @@ FileSerializer::~FileSerializer()
 		io->Flush();
 
 	delete [] file;
-	delete io;
 
-	if ( fd >= 0 )
-		close(fd);
+	if ( io )
+		delete io;  // destructor will call close() on fd
+	else if ( fd >= 0 )
+		safe_close(fd);
 	}
 
 bool FileSerializer::Open(const char* file, bool pure)
@@ -810,8 +828,8 @@ void FileSerializer::CloseFile()
 	if ( io )
 		io->Flush();
 
-	if ( fd >= 0 )
-		close(fd);
+	if ( fd >= 0 && ! io ) // destructor of io calls close() on fd
+		safe_close(fd);
 	fd = -1;
 
 	delete [] file;
@@ -911,7 +929,7 @@ bool FileSerializer::Read(UnserialInfo* info, const char* file, bool header)
 
 void FileSerializer::ReportError(const char* str)
 	{
-	run_time(str);
+	reporter->Error("%s", str);
 	}
 
 void FileSerializer::GotID(ID* id, Val* val)
@@ -971,7 +989,7 @@ ConversionSerializer::~ConversionSerializer()
 
 bool ConversionSerializer::Convert(const char* file_in, const char* file_out)
 	{
-	internal_error("Error: Printing as XML is broken.");
+	reporter->InternalError("Error: Printing as XML is broken.");
 
 	if ( ! serout->Open(file_out, true) )
 		return false;
@@ -1004,23 +1022,24 @@ void ConversionSerializer::GotFunctionCall(const char* name, double time,
 
 void ConversionSerializer::GotID(ID* id, Val* val)
 	{
-	warn("ConversionSerializer::GotID not implemented");
+	reporter->Warning("ConversionSerializer::GotID not implemented");
 	Unref(id);
 	}
 
 void ConversionSerializer::GotStateAccess(StateAccess* s)
 	{
-	warn("ConversionSerializer::GotID not implemented");
+	reporter->Warning("ConversionSerializer::GotID not implemented");
 	delete s;
 	}
 
 void ConversionSerializer::GotPacket(Packet* p)
 	{
-	warn("ConversionSerializer::GotPacket not implemented");
+	reporter->Warning("ConversionSerializer::GotPacket not implemented");
 	delete p;
 	}
 
 EventPlayer::EventPlayer(const char* file)
+    : stream_time(), replay_time(), ne_time(), ne_handler(), ne_args()
 	{
 	if ( ! OpenFile(file, true) || fd < 0 )
 		Error(fmt("event replayer: cannot open %s", file));
@@ -1105,9 +1124,9 @@ void EventPlayer::Process()
 void Packet::Describe(ODesc* d) const
 	{
 	const IP_Hdr ip = IP();
-	d->Add(dotted_addr(ip.SrcAddr()));
+	d->Add(ip.SrcAddr());
 	d->Add("->");
-	d->Add(dotted_addr(ip.DstAddr()));
+	d->Add(ip.DstAddr());
 	}
 
 bool Packet::Serialize(SerialInfo* info) const
@@ -1137,7 +1156,11 @@ Packet* Packet::Unserialize(UnserialInfo* info)
 		UNSERIALIZE(&tv_usec) &&
 		UNSERIALIZE(&len) &&
 		UNSERIALIZE(&p->link_type)) )
+		{
+		delete p;
+		delete hdr;
 		return 0;
+		}
 
 	hdr->ts.tv_sec = tv_sec;
 	hdr->ts.tv_usec = tv_usec;
@@ -1145,19 +1168,25 @@ Packet* Packet::Unserialize(UnserialInfo* info)
 
 	char* tag;
 	if ( ! info->s->Read((char**) &tag, 0, "tag") )
+		{
+		delete p;
+		delete hdr;
 		return 0;
+		}
 
-	u_char* pkt;
+	char* pkt;
 	int caplen;
 	if ( ! info->s->Read((char**) &pkt, &caplen, "data") )
 		{
+		delete p;
+		delete hdr;
 		delete [] tag;
 		return 0;
 		}
 
 	hdr->caplen = uint32(caplen);
 	p->hdr = hdr;
-	p->pkt = pkt;
+	p->pkt = (u_char*) pkt;
 	p->tag = tag;
 	p->hdr_size = get_link_header_size(p->link_type);
 
