@@ -1,5 +1,3 @@
-// $Id: StmtEnums.h 6219 2008-10-01 05:39:07Z vern $
-//
 // See the file "COPYING" in the main distribution directory for copyright.
 
 
@@ -9,7 +7,8 @@
 // These are in a separate file to break circular dependences 
 typedef enum {
 	STMT_ANY = -1,
-	STMT_ALARM, STMT_PRINT, STMT_EVENT,
+	STMT_ALARM, // Does no longer exist but kept to create enums consistent.
+	STMT_PRINT, STMT_EVENT,
 	STMT_EXPR,
 	STMT_IF, STMT_WHEN, STMT_SWITCH,
 	STMT_FOR, STMT_NEXT, STMT_BREAK,
@@ -17,6 +16,7 @@ typedef enum {
 	STMT_ADD, STMT_DELETE,
 	STMT_LIST, STMT_EVENT_BODY_LIST,
 	STMT_INIT,
+	STMT_FALLTHROUGH,
 	STMT_NULL
 #define NUM_STMTS (int(STMT_NULL) + 1)
 } BroStmtTag;
@@ -25,7 +25,8 @@ typedef enum {
 	FLOW_NEXT,		// continue on to next statement
 	FLOW_LOOP,		// go to top of loop
 	FLOW_BREAK,		// break out of loop
-	FLOW_RETURN		// return from function
+	FLOW_RETURN,		// return from function
+	FLOW_FALLTHROUGH	// fall through to next switch case
 } stmt_flow_type;
 
 extern const char* stmt_name(BroStmtTag t);
