@@ -1,5 +1,3 @@
-// $Id: Val.cc 6945 2009-11-27 19:25:10Z vern $
-//
 // See the file "COPYING" in the main distribution directory for copyright.
 
 #include "config.h"
@@ -26,12 +24,13 @@
 #include "RemoteSerializer.h"
 #include "PrefixTable.h"
 #include "Conn.h"
-#include "Logger.h"
-
+#include "Reporter.h"
+#include "IPAddr.h"
 
 Val::Val(Func* f)
 	{
 	val.func_val = f;
+	::Ref(val.func_val);
 	type = f->FType()->Ref();
 	attribs = 0;
 #ifdef DEBUG
@@ -61,12 +60,15 @@ Val::~Val()
 	if ( type->InternalType() == TYPE_INTERNAL_STRING )
 		delete val.string_val;
 
+	else if ( type->Tag() == TYPE_FUNC )
+		Unref(val.func_val);
+
 	else if ( type->Tag() == TYPE_FILE )
 		Unref(val.file_val);
 
 	Unref(type);
 #ifdef DEBUG
-	Unref(bound_id);
+	delete [] bound_id;
 #endif
 	}
 
@@ -79,7 +81,9 @@ Val* Val::Clone() const
 	SerialInfo sinfo(&ss);
 	sinfo.cache = false;
 
-	this->Serialize(&sinfo);
+	if ( ! this->Serialize(&sinfo) )
+		return 0;
+
 	char* data;
 	uint32 len = form->EndWrite(&data);
 	form->StartRead(data, len);
@@ -207,29 +211,10 @@ bool Val::DoSerialize(SerialInfo* info) const
 				val.string_val->Len());
 
 	case TYPE_INTERNAL_ADDR:
-		return SERIALIZE(NUM_ADDR_WORDS)
-#ifdef BROv6
-			&& SERIALIZE(uint32(ntohl(val.addr_val[0])))
-			&& SERIALIZE(uint32(ntohl(val.addr_val[1])))
-			&& SERIALIZE(uint32(ntohl(val.addr_val[2])))
-			&& SERIALIZE(uint32(ntohl(val.addr_val[3])));
-#else
-			&& SERIALIZE(uint32(ntohl(val.addr_val)));
-#endif
+		return SERIALIZE(*val.addr_val);
 
 	case TYPE_INTERNAL_SUBNET:
-		return info->s->WriteOpenTag("subnet")
-			&& SERIALIZE(NUM_ADDR_WORDS)
-#ifdef BROv6
-			&& SERIALIZE(uint32(ntohl(val.subnet_val.net[0])))
-			&& SERIALIZE(uint32(ntohl(val.subnet_val.net[1])))
-			&& SERIALIZE(uint32(ntohl(val.subnet_val.net[2])))
-			&& SERIALIZE(uint32(ntohl(val.subnet_val.net[3])))
-#else
-			&& SERIALIZE(uint32(ntohl(val.subnet_val.net)))
-#endif
-			&& SERIALIZE(val.subnet_val.width)
-			&& info->s->WriteCloseTag("subnet");
+		return SERIALIZE(*val.subnet_val);
 
 	case TYPE_INTERNAL_OTHER:
 		// Derived classes are responsible for this.
@@ -253,7 +238,6 @@ bool Val::DoSerialize(SerialInfo* info) const
 		return false;
 	}
 
-	internal_error("should not be reached");
 	return false;
 	}
 
@@ -296,94 +280,15 @@ bool Val::DoUnserialize(UnserialInfo* info)
 
 	case TYPE_INTERNAL_ADDR:
 		{
-		int num_words;
-		if ( ! UNSERIALIZE(&num_words) )
-			return false;
-
-		if ( num_words != 1 && num_words != 4 )
-			{
-			info->s->Error("bad address type");
-			return false;
-			}
-
-		uint32 a[4];	// big enough to hold either
-
-		for ( int i = 0; i < num_words; ++i )
-			{
-			if ( ! UNSERIALIZE(&a[i]) )
-				return false;
-
-			a[i] = htonl(a[i]);
-			}
-
-#ifndef BROv6
-		if ( num_words == 4 )
-			{
-			if ( a[0] || a[1] || a[2] )
-				info->s->Warning("received IPv6 address, ignoring");
-			((AddrVal*) this)->Init(a[3]);
-			}
-		else
-			((AddrVal*) this)->Init(a[0]);
-#else
-		if ( num_words == 1 )
-			((AddrVal*) this)->Init(a[0]);
-		else
-			((AddrVal*) this)->Init(a);
-#endif
+		val.addr_val = new IPAddr();
+		return UNSERIALIZE(val.addr_val);
 		}
-		return true;
 
 	case TYPE_INTERNAL_SUBNET:
 		{
-		int num_words;
-		if ( ! UNSERIALIZE(&num_words) )
-			return false;
-
-		if ( num_words != 1 && num_words != 4 )
-			{
-			info->s->Error("bad subnet type");
-			return false;
-			}
-
-		uint32 a[4];	// big enough to hold either
-
-		for ( int i = 0; i < num_words; ++i )
-			{
-			if ( ! UNSERIALIZE(&a[i]) )
-				return false;
-
-			a[i] = htonl(a[i]);
-			}
-
-		int width;
-		if ( ! UNSERIALIZE(&width) )
-			return false;
-
-#ifdef BROv6
-		if ( num_words == 1 )
-			{
-			a[3] = a[0];
-			a[0] = a[1] = a[2] = 0;
-			}
-
-		((SubNetVal*) this)->Init(a, width);
-
-#else
-		if ( num_words == 4 )
-			{
-			if ( a[0] || a[1] || a[2] )
-				info->s->Warning("received IPv6 subnet, ignoring");
-			a[0] = a[3];
-
-			if ( width > 32 )
-				width -= 96;
-			}
-
-		((SubNetVal*) this)->Init(a[0], width);
-#endif
+		val.subnet_val = new IPPrefix();
+		return UNSERIALIZE(val.subnet_val);
 		}
-		return true;
 
 	case TYPE_INTERNAL_OTHER:
 		// Derived classes are responsible for this.
@@ -410,19 +315,9 @@ bool Val::DoUnserialize(UnserialInfo* info)
 		return false;
 	}
 
-	internal_error("should not be reached");
 	return false;
 	}
 
-RecordVal* Val::GetAttribs(bool instantiate)
-	{
-	if ( ! instantiate || attribs )
-		return attribs;
-
-	attribs = new RecordVal(type->AttributesType());
-	return attribs;
-	}
-
 int Val::IsZero() const
 	{
 	switch ( type->InternalType() ) {
@@ -524,7 +419,12 @@ Val* Val::SizeVal() const
 	{
 	switch ( type->InternalType() ) {
 	case TYPE_INTERNAL_INT:
-		return new Val(abs(val.int_val), TYPE_COUNT);
+		// Return abs value. However abs() only works on ints and llabs
+		// doesn't work on Mac OS X 10.5. So we do it by hand
+		if ( val.int_val < 0 )
+			return new Val(-val.int_val, TYPE_COUNT);
+		else
+			return new Val(val.int_val, TYPE_COUNT);
 
 	case TYPE_INTERNAL_UNSIGNED:
 		return new Val(val.uint_val, TYPE_COUNT);
@@ -578,6 +478,11 @@ void Val::Describe(ODesc* d) const
 		Val::ValDescribe(d);
 	}
 
+void Val::DescribeReST(ODesc* d) const
+	{
+	ValDescribeReST(d);
+	}
+
 void Val::ValDescribe(ODesc* d) const
 	{
 	if ( d->IsReadable() && type->Tag() == TYPE_BOOL )
@@ -591,12 +496,10 @@ void Val::ValDescribe(ODesc* d) const
 	case TYPE_INTERNAL_UNSIGNED:	d->Add(val.uint_val); break;
 	case TYPE_INTERNAL_DOUBLE:	d->Add(val.double_val); break;
 	case TYPE_INTERNAL_STRING:	d->AddBytes(val.string_val); break;
-	case TYPE_INTERNAL_ADDR:	d->Add(dotted_addr(val.addr_val)); break;
+	case TYPE_INTERNAL_ADDR:	d->Add(val.addr_val->AsString().c_str()); break;
 
 	case TYPE_INTERNAL_SUBNET:
-		d->Add(dotted_addr(val.subnet_val.net));
-		d->Add("/");
-		d->Add(val.subnet_val.width);
+		d->Add(val.subnet_val->AsString().c_str());
 		break;
 
 	case TYPE_INTERNAL_ERROR:	d->AddCS("error"); break;
@@ -614,8 +517,23 @@ void Val::ValDescribe(ODesc* d) const
 		break;
 
 	default:
-		// Don't call Internal(), that'll loop!
-		internal_error("Val description unavailable");
+		reporter->InternalWarning("Val description unavailable");
+		d->Add("<value description unavailable>");
+		break;
+	}
+	}
+
+void Val::ValDescribeReST(ODesc* d) const
+	{
+	switch ( type->InternalType() ) {
+	case TYPE_INTERNAL_OTHER:
+		Describe(d);
+		break;
+
+	default:
+		d->Add("``");
+		ValDescribe(d);
+		d->Add("``");
 	}
 	}
 
@@ -623,14 +541,16 @@ MutableVal::~MutableVal()
 	{
 	for ( list<ID*>::iterator i = aliases.begin(); i != aliases.end(); ++i )
 		{
-		global_scope()->Remove((*i)->Name());
+		if ( global_scope() )
+			global_scope()->Remove((*i)->Name());
 		(*i)->ClearVal();	// just to make sure.
 		Unref((*i));
 		}
 
 	if ( id )
 		{
-		global_scope()->Remove(id->Name());
+		if ( global_scope() )
+			global_scope()->Remove(id->Name());
 		id->ClearVal(); // just to make sure.
 		Unref(id);
 		}
@@ -691,7 +611,8 @@ ID* MutableVal::Bind() const
 			ip = htonl(0x7f000001);	// 127.0.0.1
 
 		safe_snprintf(name, MAX_NAME_SIZE, "#%s#%d#",
-			      dotted_addr(ip), getpid());
+			      IPAddr(IPv4, &ip, IPAddr::Network)->AsString().c_str(),
+			      getpid());
 #else
 		safe_snprintf(name, MAX_NAME_SIZE, "#%s#%d#", host, getpid());
 #endif
@@ -942,92 +863,41 @@ bool PortVal::DoUnserialize(UnserialInfo* info)
 
 AddrVal::AddrVal(const char* text) : Val(TYPE_ADDR)
 	{
-	const char* colon = strchr(text, ':');
-
-	if ( colon )
-		{
-#ifdef BROv6
-		Init(dotted_to_addr6(text));
-#else
-		error("bro wasn't compiled with IPv6 support");
-		Init(uint32(0));
-#endif
-		}
-
-	else
-		Init(dotted_to_addr(text));
+	val.addr_val = new IPAddr(text);
 	}
 
 AddrVal::AddrVal(uint32 addr) : Val(TYPE_ADDR)
 	{
 	// ### perhaps do gethostbyaddr here?
-	Init(addr);
+	val.addr_val = new IPAddr(IPv4, &addr, IPAddr::Network);
 	}
 
-AddrVal::AddrVal(const uint32* addr) : Val(TYPE_ADDR)
+AddrVal::AddrVal(const uint32 addr[4]) : Val(TYPE_ADDR)
 	{
-	Init(addr);
+	val.addr_val = new IPAddr(IPv6, addr, IPAddr::Network);
+	}
+
+AddrVal::AddrVal(const IPAddr& addr) : Val(TYPE_ADDR)
+	{
+	val.addr_val = new IPAddr(addr);
 	}
 
 AddrVal::~AddrVal()
 	{
-#ifdef BROv6
-	delete [] val.addr_val;
-#endif
-	}
-
-Val* AddrVal::SizeVal() const
-	{
-	uint32 addr;
-
-#ifdef BROv6
-	if ( ! is_v4_addr(val.addr_val) )
-		{
-		RunTime("|addr| for IPv6 addresses not supported");
-		return new Val(0, TYPE_COUNT);
-		}
-
-	addr = to_v4_addr(val.addr_val);
-#else
-	addr = val.addr_val;
-#endif
-
-	addr = ntohl(addr);
-
-	return new Val(addr, TYPE_COUNT);
-	}
-
-void AddrVal::Init(uint32 addr)
-	{
-#ifdef BROv6
-	val.addr_val = new uint32[4];
-	val.addr_val[0] = val.addr_val[1] = val.addr_val[2] = 0;
-	val.addr_val[3] = addr;
-#else
-	val.addr_val = addr;
-#endif
-	}
-
-void AddrVal::Init(const uint32* addr)
-	{
-#ifdef BROv6
-	val.addr_val = new uint32[4];
-	val.addr_val[0] = addr[0];
-	val.addr_val[1] = addr[1];
-	val.addr_val[2] = addr[2];
-	val.addr_val[3] = addr[3];
-#else
-	val.addr_val = addr[0];
-#endif
+	delete val.addr_val;
 	}
 
 unsigned int AddrVal::MemoryAllocation() const
 	{
-#ifdef BROv6
-		return padded_sizeof(*this) + pad_size(4 * sizeof(uint32));
-#else
-		return padded_sizeof(*this);
-#endif
+	return padded_sizeof(*this) + val.addr_val->MemoryAllocation();
+	}
+
+Val* AddrVal::SizeVal() const
+	{
+	if ( val.addr_val->GetFamily() == IPv4 )
+		return new Val(32, TYPE_COUNT);
+	else
+		return new Val(128, TYPE_COUNT);
 	}
 
 IMPLEMENT_SERIAL(AddrVal, SER_ADDR_VAL);
@@ -1044,287 +914,114 @@ bool AddrVal::DoUnserialize(UnserialInfo* info)
 	return true;
 	}
 
-static uint32 parse_dotted(const char* text, int& dots)
-	{
-	int addr[4];
-	uint32 a = 0;
-	dots = 0;
-
-	if ( sscanf(text, "%d.%d.%d.%d", addr+0, addr+1, addr+2, addr+3) == 4 )
-		{
-		a = (addr[0] << 24) | (addr[1] << 16) |
-			(addr[2] << 8) | addr[3];
-		dots = 3;
-		}
-
-	else if ( sscanf(text, "%d.%d.%d", addr+0, addr+1, addr+2) == 3 )
-		{
-		a = (addr[0] << 24) | (addr[1] << 16) | (addr[2] << 8);
-		dots = 2;
-		}
-
-	else if ( sscanf(text, "%d.%d", addr+0, addr+1) == 2 )
-		{
-		a = (addr[0] << 24) | (addr[1] << 16);
-		dots = 1;
-		}
-
-	else
-		internal_error("scanf failed in parse_dotted()");
-
-	for ( int i = 0; i <= dots; ++i )
-		{
-		if ( addr[i] < 0 || addr[i] > 255 )
-			{
-			error("bad dotted address", text);
-			break;
-			}
-		}
-
-	return a;
-	}
-
-NetVal::NetVal(const char* text) : AddrVal(TYPE_NET)
-	{
-	int dots;
-	uint32 a = parse_dotted(text, dots);
-
-	if ( addr_to_net(a) != a )
-		error("bad net address", text);
-
-	Init(uint32(htonl(a)));
-	}
-
-NetVal::NetVal(uint32 addr) : AddrVal(TYPE_NET)
-	{
-	Init(addr);
-	}
-
-#ifdef BROv6
-NetVal::NetVal(const uint32* addr) : AddrVal(TYPE_NET)
-	{
-	Init(addr);
-	}
-#endif
-
-Val* NetVal::SizeVal() const
-	{
-	uint32 addr;
-
-#ifdef BROv6
-	if ( ! is_v4_addr(val.addr_val) )
-		{
-		RunTime("|net| for IPv6 addresses not supported");
-		return new Val(0.0, TYPE_DOUBLE);
-		}
-
-	addr = to_v4_addr(val.addr_val);
-#else
-	addr = val.addr_val;
-#endif
-	addr = ntohl(addr);
-
-	if ( (addr & 0xFFFFFFFF) == 0L )
-		return new Val(4294967296.0, TYPE_DOUBLE);
-
-	if ( (addr & 0x00FFFFFF) == 0L )
-		return new Val(double(0xFFFFFF + 1), TYPE_DOUBLE);
-
-	if ( (addr & 0x0000FFFF) == 0L )
-		return new Val(double(0xFFFF + 1), TYPE_DOUBLE);
-
-	if ( (addr & 0x000000FF) == 0L )
-		return new Val(double(0xFF + 1), TYPE_DOUBLE);
-
-	return new Val(1.0, TYPE_DOUBLE);
-	}
-
-void NetVal::ValDescribe(ODesc* d) const
-	{
-#ifdef BROv6
-	d->Add(dotted_net6(val.addr_val));
-#else
-	d->Add(dotted_net(val.addr_val));
-#endif
-	}
-
-IMPLEMENT_SERIAL(NetVal, SER_NET_VAL);
-
-bool NetVal::DoSerialize(SerialInfo* info) const
-	{
-	DO_SERIALIZE(SER_NET_VAL, AddrVal);
-	return true;
-	}
-
-bool NetVal::DoUnserialize(UnserialInfo* info)
-	{
-	DO_UNSERIALIZE(AddrVal);
-	return true;
-	}
-
 SubNetVal::SubNetVal(const char* text) : Val(TYPE_SUBNET)
 	{
-	const char* sep = strchr(text, '/');
-	if ( ! sep )
-		Internal("separator missing in SubNetVal::SubNetVal");
+	string s(text);
+	size_t slash_loc = s.find('/');
 
-	Init(text, atoi(sep+1));
+	if ( slash_loc == string::npos )
+		{
+		reporter->Error("Bad string in SubNetVal ctor: %s", text);
+		val.subnet_val = new IPPrefix();
+		}
+	else
+		{
+		val.subnet_val = new IPPrefix(s.substr(0, slash_loc),
+		                              atoi(s.substr(slash_loc + 1).c_str()));
+		}
 	}
 
 SubNetVal::SubNetVal(const char* text, int width) : Val(TYPE_SUBNET)
 	{
-	Init(text, width);
+	val.subnet_val = new IPPrefix(text, width);
 	}
 
 SubNetVal::SubNetVal(uint32 addr, int width) : Val(TYPE_SUBNET)
 	{
-	Init(addr, width);
+	IPAddr a(IPv4, &addr, IPAddr::Network);
+	val.subnet_val = new IPPrefix(a, width);
 	}
 
-#ifdef BROv6
 SubNetVal::SubNetVal(const uint32* addr, int width) : Val(TYPE_SUBNET)
 	{
-	Init(addr, width);
-	}
-#endif
-
-void SubNetVal::Init(const char* text, int width)
-	{
-#ifdef BROv6
-	if ( width <= 0 || width > 128 )
-#else
-	if ( width <= 0 || width > 32 )
-#endif
-		Error("bad subnet width");
-
-	int dots;
-	uint32 a = parse_dotted(text, dots);
-
-	Init(uint32(htonl(a)), width);
+	IPAddr a(IPv6, addr, IPAddr::Network);
+	val.subnet_val = new IPPrefix(a, width);
 	}
 
-
-void SubNetVal::Init(uint32 addr, int width)
+SubNetVal::SubNetVal(const IPAddr& addr, int width) : Val(TYPE_SUBNET)
 	{
-#ifdef BROv6
-	Internal("SubNetVal::Init called on 4-byte address w/ BROv6");
-#else
-	val.subnet_val.net = mask_addr(addr, uint32(width));
-	val.subnet_val.width = width;
-#endif
+	val.subnet_val = new IPPrefix(addr, width);
 	}
 
-void SubNetVal::Init(const uint32* addr, int width)
+SubNetVal::SubNetVal(const IPPrefix& prefix) : Val(TYPE_SUBNET)
 	{
-#ifdef BROv6
-	const uint32* a = mask_addr(addr, uint32(width));
+	val.subnet_val = new IPPrefix(prefix);
+	}
 
-	val.subnet_val.net[0] = a[0];
-	val.subnet_val.net[1] = a[1];
-	val.subnet_val.net[2] = a[2];
-	val.subnet_val.net[3] = a[3];
+SubNetVal::~SubNetVal()
+	{
+	delete val.subnet_val;
+	}
 
-	if ( is_v4_addr(addr) && width <= 32 )
-		val.subnet_val.width = width + 96;
-	else
-		val.subnet_val.width = width;
-#else
-	Internal("SubNetVal::Init called on 16-byte address w/o BROv6");
-#endif
+const IPAddr& SubNetVal::Prefix() const
+	{
+	return val.subnet_val->Prefix();
+	}
+
+int SubNetVal::Width() const
+	{
+	return val.subnet_val->Length();
+	}
+
+unsigned int SubNetVal::MemoryAllocation() const
+	{
+	return padded_sizeof(*this) + val.subnet_val->MemoryAllocation();
 	}
 
 Val* SubNetVal::SizeVal() const
 	{
-	int retained;
-#ifdef BROv6
-	retained = 128 - Width();
-#else
-	retained = 32 - Width();
-#endif
-
+	int retained = 128 - val.subnet_val->LengthIPv6();
 	return new Val(pow(2.0, double(retained)), TYPE_DOUBLE);
 	}
 
 void SubNetVal::ValDescribe(ODesc* d) const
 	{
-	d->Add(dotted_addr(val.subnet_val.net, d->Style() == ALTERNATIVE_STYLE));
-	d->Add("/");
-#ifdef BROv6
-	if ( is_v4_addr(val.subnet_val.net) )
-		d->Add(val.subnet_val.width - 96);
-	else
-#endif
-	d->Add(val.subnet_val.width);
+	d->Add(string(*val.subnet_val).c_str());
 	}
 
-addr_type SubNetVal::Mask() const
+IPAddr SubNetVal::Mask() const
 	{
-	if ( val.subnet_val.width == 0 )
+	if ( val.subnet_val->Length() == 0 )
 		{
 		// We need to special-case a mask width of zero, since
 		// the compiler doesn't guarantee that 1 << 32 yields 0.
-#ifdef BROv6
-		uint32* m = new uint32[4];
-		for ( int i = 0; i < 4; ++i )
+		uint32 m[4];
+		for ( unsigned int i = 0; i < 4; ++i )
 			m[i] = 0;
-
-		return m;
-#else
-		return 0;
-#endif
+		IPAddr rval(IPv6, m, IPAddr::Host);
+		return rval;
 		}
 
-#ifdef BROv6
-	uint32* m = new uint32[4];
+	uint32 m[4];
 	uint32* mp = m;
 
 	uint32 w;
-	for ( w = val.subnet_val.width; w >= 32; w -= 32 )
-		*(mp++) = 0xffffffff;
+	for ( w = val.subnet_val->Length(); w >= 32; w -= 32 )
+		   *(mp++) = 0xffffffff;
 
 	*mp = ~((1 << (32 - w)) - 1);
 
 	while ( ++mp < m + 4 )
-		*mp = 0;
+		   *mp = 0;
 
-	return m;
-
-#else
-	return ~((1 << (32 - val.subnet_val.width)) - 1);
-#endif
+	IPAddr rval(IPv6, m, IPAddr::Host);
+	return rval;
 	}
 
-bool SubNetVal::Contains(const uint32 addr) const
+bool SubNetVal::Contains(const IPAddr& addr) const
 	{
-#ifdef BROv6
-	Internal("SubNetVal::Contains called on 4-byte address w/ BROv6");
-	return false;
-#else
-	return ntohl(val.subnet_val.net) == (ntohl(addr) & Mask());
-#endif
-	}
-
-bool SubNetVal::Contains(const uint32* addr) const
-	{
-#ifdef BROv6
-	const uint32* net = val.subnet_val.net;
-	const uint32* a = addr;
-	uint32 m;
-
-	for ( m = val.subnet_val.width; m > 32; m -= 32 )
-		{
-		if ( *net != *a )
-			return false;
-
-		++net;
-		++a;
-		}
-
-	uint32 mask = ~((1 << (32 - m)) - 1);
-	return ntohl(*net) == (ntohl(*a) & mask);
-#else
-	return Contains(addr[3]);
-#endif
+	IPAddr a(addr);
+	return val.subnet_val->Contains(a);
 	}
 
 IMPLEMENT_SERIAL(SubNetVal, SER_SUBNET_VAL);
@@ -1357,6 +1054,11 @@ StringVal::StringVal(const char* s) : Val(TYPE_STRING)
 	val.string_val = new BroString(s);
 	}
 
+StringVal::StringVal(const string& s) : Val(TYPE_STRING)
+	{
+	val.string_val = new BroString(s.c_str());
+	}
+
 StringVal* StringVal::ToUpper()
 	{
 	val.string_val->ToUpper();
@@ -1526,6 +1228,7 @@ TableVal* ListVal::ConvertToSet() const
 	loop_over_list(vals, i)
 		t->Assign(vals[i], 0);
 
+	Unref(s);
 	return t;
 	}
 
@@ -1645,6 +1348,7 @@ TableVal::TableVal(TableType* t, Attributes* a) : MutableVal(t)
 
 void TableVal::Init(TableType* t)
 	{
+	::Ref(t);
 	table_type = t;
 	expire_expr = 0;
 	expire_time = 0;
@@ -1667,6 +1371,7 @@ TableVal::~TableVal()
 	if ( timer )
 		timer_mgr->Cancel(timer);
 
+	Unref(table_type);
 	delete table_hash;
 	delete AsTable();
 	delete subnets;
@@ -1738,7 +1443,7 @@ void TableVal::CheckExpireAttr(attr_tag at)
 			a->AttrExpr()->Error("value of timeout not fixed");
 			return;
 			}
-		
+
 		expire_time = timeout->AsInterval();
 
 		if ( timer )
@@ -1952,10 +1657,10 @@ int TableVal::RemoveFrom(Val* val) const
 	IterCookie* c = tbl->InitForIteration();
 
 	HashKey* k;
-	TableEntryVal* v;
-	while ( (v = tbl->NextEntry(k, c)) )
+	while ( tbl->NextEntry(k, c) )
 		{
 		Val* index = RecoverIndex(k);
+
 		Unref(index);
 		Unref(t->Delete(k));
 		delete k;
@@ -1983,7 +1688,7 @@ int TableVal::ExpandAndInit(Val* index, Val* new_val)
 	if ( iv->BaseTag() != TYPE_ANY )
 		{
 		if ( table_type->Indices()->Types()->length() != 1 )
-			internal_error("bad singleton list index");
+			reporter->InternalError("bad singleton list index");
 
 		for ( int i = 0; i < iv->Length(); ++i )
 			if ( ! ExpandAndInit(iv->Index(i), new_val ? new_val->Ref() : 0) )
@@ -2027,17 +1732,33 @@ Val* TableVal::Default(Val* index)
 		return 0;
 
 	if ( ! def_val )
-		def_val = def_attr->AttrExpr()->Eval(0);
+		{
+		BroType* ytype = Type()->YieldType();
+		BroType* dtype = def_attr->AttrExpr()->Type();
+
+		if ( dtype->Tag() == TYPE_RECORD && ytype->Tag() == TYPE_RECORD &&
+		     ! same_type(dtype, ytype) &&
+		     record_promotion_compatible(dtype->AsRecordType(),
+						 ytype->AsRecordType()) )
+			{
+			Expr* coerce = new RecordCoerceExpr(def_attr->AttrExpr(), ytype->AsRecordType());
+			def_val = coerce->Eval(0);
+			Unref(coerce);
+			}
+
+		else
+			def_val = def_attr->AttrExpr()->Eval(0);
+		}
 
 	if ( ! def_val )
 		{
-		RunTime("non-constant default attribute");
+		Error("non-constant default attribute");
 		return 0;
 		}
 
 	if ( def_val->Type()->Tag() != TYPE_FUNC ||
 	     same_type(def_val->Type(), Type()->YieldType()) )
-		return def_val->Ref();
+		return def_attr->AttrExpr()->IsConst() ? def_val->Ref() : def_val->Clone();
 
 	const Func* f = def_val->AsFunc();
 	val_list* vl = new val_list();
@@ -2051,12 +1772,21 @@ Val* TableVal::Default(Val* index)
 	else
 		vl->append(index->Ref());
 
-	Val* result = f->Call(vl);
+	Val* result = 0;
+
+	try
+		{
+		result = f->Call(vl);
+		}
+
+	catch ( InterpreterException& e )
+		{ /* Already reported. */ }
+
 	delete vl;
 
 	if ( ! result )
 		{
-		RunTime("no value returned from &default function");
+		Error("no value returned from &default function");
 		return 0;
 		}
 
@@ -2162,7 +1892,7 @@ Val* TableVal::Delete(const Val* index)
 	Val* va = v ? (v->Value() ? v->Value() : this->Ref()) : 0;
 
 	if ( subnets && ! subnets->Remove(index) )
-		internal_error( "index not in prefix table" );
+		reporter->InternalWarning("index not in prefix table");
 
 	if ( LoggingAccess() )
 		{
@@ -2204,7 +1934,7 @@ Val* TableVal::Delete(const HashKey* k)
 		{
 		Val* index = table_hash->RecoverVals(k);
 		if ( ! subnets->Remove(index) )
-			internal_error( "index not in prefix table" );
+			reporter->InternalWarning("index not in prefix table");
 		Unref(index);
 		}
 
@@ -2225,8 +1955,7 @@ ListVal* TableVal::ConvertToList(TypeTag t) const
 	IterCookie* c = tbl->InitForIteration();
 
 	HashKey* k;
-	TableEntryVal* v;
-	while ( (v = tbl->NextEntry(k, c)) )
+	while ( tbl->NextEntry(k, c) )
 		{
 		ListVal* index = table_hash->RecoverVals(k);
 
@@ -2253,7 +1982,10 @@ ListVal* TableVal::ConvertToPureList() const
 	{
 	type_list* tl = table_type->Indices()->Types();
 	if ( tl->length() != 1 )
+		{
 		InternalWarning("bad index type in TableVal::ConvertToPureList");
+		return 0;
+		}
 
 	return ConvertToList((*tl)[0]->Tag());
 	}
@@ -2285,7 +2017,7 @@ void TableVal::Describe(ODesc* d) const
 		TableEntryVal* v = tbl->NextEntry(k, c);
 
 		if ( ! v )
-			internal_error("hash table underflow in TableVal::Describe");
+			reporter->InternalError("hash table underflow in TableVal::Describe");
 
 		ListVal* vl = table_hash->RecoverVals(k);
 		int dim = vl->Length();
@@ -2336,7 +2068,7 @@ void TableVal::Describe(ODesc* d) const
 		}
 
 	if ( tbl->NextEntry(c) )
-		internal_error("hash table overflow in TableVal::Describe");
+		reporter->InternalError("hash table overflow in TableVal::Describe");
 
 	if ( d->IsPortable() || d->IsReadable() )
 		{
@@ -2350,7 +2082,7 @@ int TableVal::ExpandCompoundAndInit(val_list* vl, int k, Val* new_val)
 	Val* ind_k_v = (*vl)[k];
 	ListVal* ind_k = ind_k_v->Type()->IsSet() ?
 				ind_k_v->AsTableVal()->ConvertToList() :
-				ind_k_v->AsListVal();
+				ind_k_v->Ref()->AsListVal();
 
 	for ( int i = 0; i < ind_k->Length(); ++i )
 		{
@@ -2368,12 +2100,13 @@ int TableVal::ExpandCompoundAndInit(val_list* vl, int k, Val* new_val)
 		Unref(expd);
 
 		if ( ! success )
+			{
+			Unref(ind_k);
 			return 0;
+			}
 		}
 
-	if ( ind_k_v->Type()->IsSet() )
-		Unref(ind_k);
-
+	Unref(ind_k);
 	return 1;
 	}
 
@@ -2418,14 +2151,17 @@ void TableVal::DoExpire(double t)
 			 (v = tbl->NextEntry(k, expire_cookie)); ++i )
 		{
 		if ( v->ExpireAccessTime() == 0 )
+			{
 			// This happens when we insert val while network_time
-			// hasn't been initialized yet (e.g. in bro_init()).
-			// We correct the timestamp now.
-			v->SetExpireAccess(network_time);
+			// hasn't been initialized yet (e.g. in bro_init()), and
+			// also when bro_start_network_time hasn't been initialized
+			// (e.g. before first packet).  The expire_access_time is
+			// correct, so we just need to wait.
+			}
 
 		else if ( v->ExpireAccessTime() + expire_time < t )
 			{
-			Val* val = v ? v->Value() : 0;
+			Val* val = v->Value();
 
 			if ( expire_expr )
 				{
@@ -2458,7 +2194,7 @@ void TableVal::DoExpire(double t)
 				{
 				Val* index = RecoverIndex(k);
 				if ( ! subnets->Remove(index) )
-					internal_error( "index not in prefix table" );
+					reporter->InternalWarning("index not in prefix table");
 				Unref(index);
 				}
 
@@ -2507,10 +2243,20 @@ double TableVal::CallExpireFunc(Val* idx)
 
 	vl->append(idx);
 
-	Val* vs = expire_expr->Eval(0)->AsFunc()->Call(vl);
-	double secs = vs->AsInterval();
-	Unref(vs);
-	delete vl;
+	double secs;
+
+	try
+		{
+		Val* vs = expire_expr->Eval(0)->AsFunc()->Call(vl);
+		secs = vs->AsInterval();
+		Unref(vs);
+		delete vl;
+		}
+
+	catch ( InterpreterException& e )
+		{
+		secs = 0;
+		}
 
 	return secs;
 	}
@@ -2541,7 +2287,7 @@ bool TableVal::DoSerialize(SerialInfo* info) const
 		IterCookie* c;
 		TableEntryVal* v;	// current value
 		bool did_index;	// already wrote the val's index
-	}* state;
+	}* state = 0;
 
 	PDict(TableEntryVal)* tbl =
 		const_cast<TableVal*>(this)->AsNonConstTable();
@@ -2579,9 +2325,9 @@ bool TableVal::DoSerialize(SerialInfo* info) const
 		state = (State*) info->cont.RestoreState();
 		}
 	else
-		internal_error("unknown continuation state");
+		reporter->InternalError("unknown continuation state");
 
-	HashKey* k;
+	HashKey* k = 0;
 	int count = 0;
 
 	assert((!info->cont.ChildSuspended()) || state->v);
@@ -2594,12 +2340,21 @@ bool TableVal::DoSerialize(SerialInfo* info) const
 			if ( ! state->c )
 				{
 				// No next one.
-				SERIALIZE(false);
+				if ( ! SERIALIZE(false) )
+					{
+					delete k;
+					return false;
+					}
+
 				break;
 				}
 
 			// There's a value coming.
-			SERIALIZE(true);
+			if ( ! SERIALIZE(true) )
+				{
+				delete k;
+				return false;
+				}
 
 			if ( state->v->Value() )
 				state->v->Ref();
@@ -2664,7 +2419,7 @@ bool TableVal::DoSerialize(SerialInfo* info) const
 			{
 			info->cont.SaveState(state);
 			info->cont.Suspend();
-			bro_logger->Log("TableVals serialization suspended right in the middle.");
+			reporter->Info("TableVals serialization suspended right in the middle.");
 			return true;
 			}
 		}
@@ -2721,7 +2476,11 @@ bool TableVal::DoUnserialize(UnserialInfo* info)
 
 		if ( ! UNSERIALIZE(&entry_val->last_access_time) ||
 		     ! UNSERIALIZE(&eat) )
+			{
+			entry_val->Unref();
+			delete entry_val;
 			return false;
+			}
 
 		entry_val->SetExpireAccess(eat);
 
@@ -2812,6 +2571,7 @@ unsigned int TableVal::MemoryAllocation() const
 
 RecordVal::RecordVal(RecordType* t) : MutableVal(t)
 	{
+	origin = 0;
 	record_type = t;
 	int n = record_type->NumFields();
 	val_list* vl = val.val_list_val = new val_list(n);
@@ -2823,10 +2583,22 @@ RecordVal::RecordVal(RecordType* t) : MutableVal(t)
 		Attributes* a = record_type->FieldDecl(i)->attrs;
 		Attr* def_attr = a ? a->FindAttr(ATTR_DEFAULT) : 0;
 		Val* def = def_attr ? def_attr->AttrExpr()->Eval(0) : 0;
+		BroType* type = record_type->FieldDecl(i)->type;
+
+		if ( def && type->Tag() == TYPE_RECORD &&
+		     def->Type()->Tag() == TYPE_RECORD &&
+		     ! same_type(def->Type(), type) )
+			{
+			Val* tmp = def->AsRecordVal()->CoerceTo(type->AsRecordType());
+			if ( tmp )
+				{
+				Unref(def);
+				def = tmp;
+				}
+			}
 
 		if ( ! def && ! (a && a->FindAttr(ATTR_OPTIONAL)) )
 			{
-			BroType* type = record_type->FieldDecl(i)->type;
 			TypeTag tag = type->Tag();
 
 			if ( tag == TYPE_RECORD )
@@ -2835,7 +2607,7 @@ RecordVal::RecordVal(RecordType* t) : MutableVal(t)
 			else if ( tag == TYPE_TABLE )
 				def = new TableVal(type->AsTableType(), a);
 
-			else if ( t->Tag() == TYPE_VECTOR )
+			else if ( tag == TYPE_VECTOR )
 				def = new VectorVal(type->AsVectorType());
 			}
 
@@ -2852,7 +2624,7 @@ RecordVal::~RecordVal()
 
 void RecordVal::Assign(int field, Val* new_val, Opcode op)
 	{
-	if ( Lookup(field) &&
+	if ( new_val && Lookup(field) &&
 	     record_type->FieldType(field)->Tag() == TYPE_TABLE &&
 	     new_val->AsTableVal()->FindAttr(ATTR_MERGEABLE) )
 		{
@@ -2897,6 +2669,99 @@ Val* RecordVal::Lookup(int field) const
 	return (*AsRecord())[field];
 	}
 
+Val* RecordVal::LookupWithDefault(int field) const
+	{
+	Val* val = (*AsRecord())[field];
+
+	if ( val )
+		return val->Ref();
+
+	return record_type->FieldDefault(field);
+	}
+
+Val* RecordVal::Lookup(const char* field, bool with_default) const
+	{
+	int idx = record_type->FieldOffset(field);
+
+	if ( idx < 0 )
+		reporter->InternalError("missing record field: %s", field);
+
+	return with_default ? LookupWithDefault(idx) : Lookup(idx);
+	}
+
+RecordVal* RecordVal::CoerceTo(const RecordType* t, Val* aggr, bool allow_orphaning) const
+	{
+	if ( ! record_promotion_compatible(t->AsRecordType(), Type()->AsRecordType()) )
+		return 0;
+
+	if ( ! aggr )
+		aggr = new RecordVal(const_cast<RecordType*>(t->AsRecordType()));
+
+	RecordVal* ar = aggr->AsRecordVal();
+	RecordType* ar_t = aggr->Type()->AsRecordType();
+
+	const RecordType* rv_t = Type()->AsRecordType();
+
+	int i;
+	for ( i = 0; i < rv_t->NumFields(); ++i )
+		{
+		int t_i = ar_t->FieldOffset(rv_t->FieldName(i));
+
+		if ( t_i < 0 )
+			{
+			if ( allow_orphaning )
+				continue;
+
+			char buf[512];
+			safe_snprintf(buf, sizeof(buf),
+					"orphan field \"%s\" in initialization",
+					rv_t->FieldName(i));
+			Error(buf);
+			break;
+			}
+
+		Val* v = Lookup(i);
+
+		if ( ! v )
+			// Check for allowable optional fields is outside the loop, below.
+			continue;
+
+		if ( ar_t->FieldType(t_i)->Tag() == TYPE_RECORD
+				&& ! same_type(ar_t->FieldType(t_i), v->Type()) )
+			{
+			Expr* rhs = new ConstExpr(v->Ref());
+			Expr* e = new RecordCoerceExpr(rhs, ar_t->FieldType(t_i)->AsRecordType());
+			ar->Assign(t_i, e->Eval(0));
+			continue;
+			}
+
+		ar->Assign(t_i, v->Ref());
+		}
+
+	for ( i = 0; i < ar_t->NumFields(); ++i )
+		if ( ! ar->Lookup(i) &&
+			 ! ar_t->FieldDecl(i)->FindAttr(ATTR_OPTIONAL) )
+			{
+			char buf[512];
+			safe_snprintf(buf, sizeof(buf),
+					"non-optional field \"%s\" missing in initialization", ar_t->FieldName(i));
+			Error(buf);
+			}
+
+	return ar;
+	}
+
+RecordVal* RecordVal::CoerceTo(RecordType* t, bool allow_orphaning)
+	{
+	if ( same_type(Type(), t) )
+		{
+		this->Ref();
+		return this;
+		}
+
+	return CoerceTo(t, 0, allow_orphaning);
+	}
+
 void RecordVal::Describe(ODesc* d) const
 	{
 	const val_list* vl = AsRecord();
@@ -2933,6 +2798,34 @@ void RecordVal::Describe(ODesc* d) const
 		d->Add("]");
 	}
 
+void RecordVal::DescribeReST(ODesc* d) const
+	{
+	const val_list* vl = AsRecord();
+	int n = vl->length();
+
+	d->Add("{");
+	d->PushIndent();
+
+	loop_over_list(*vl, i)
+		{
+		if ( i > 0 )
+			d->NL();
+
+		d->Add(record_type->FieldName(i));
+		d->Add("=");
+
+		Val* v = (*vl)[i];
+
+		if ( v )
+			v->Describe(d);
+		else
+			d->Add("<uninitialized>");
+		}
+
+	d->PopIndent();
+	d->Add("}");
+	}
+
 IMPLEMENT_SERIAL(RecordVal, SER_RECORD_VAL);
 
 bool RecordVal::DoSerialize(SerialInfo* info) const
@@ -3027,14 +2920,13 @@ unsigned int RecordVal::MemoryAllocation() const
 	{
 	unsigned int size = 0;
 
-	for ( int i = 0; i < type->AsRecordType()->NumFields(); ++i )
-		{
-		Val* v = (*val.val_list_val)[i];
+	const val_list* vl = AsRecord();
 
-		// v might be nil for records that don't wind
-		// up being set to a value.
+	loop_over_list(*vl, i)
+		{
+		Val* v = (*vl)[i];
 		if ( v )
-			size += v->MemoryAllocation();
+		    size += v->MemoryAllocation();
 		}
 
 	return size + padded_sizeof(*this) + val.val_list_val->MemoryAllocation();
@@ -3047,10 +2939,6 @@ void EnumVal::ValDescribe(ODesc* d) const
 	if ( ! ename )
 		ename = "<undefined>";
 
-	const char* module_offset = strstr(ename, "::");
-	if ( module_offset )
-		ename = module_offset + 2;
-
 	d->Add(ename);
 	}
 
@@ -3084,8 +2972,7 @@ VectorVal::~VectorVal()
 	delete val.vector_val;
 	}
 
-bool VectorVal::Assign(unsigned int index, Val* element, const Expr* assigner,
-			Opcode op)
+bool VectorVal::Assign(unsigned int index, Val* element, Opcode op)
 	{
 	if ( element &&
 	     ! same_type(element->Type(), vector_type->YieldType(), 0) )
@@ -3094,15 +2981,6 @@ bool VectorVal::Assign(unsigned int index, Val* element, const Expr* assigner,
 		return false;
 		}
 
-	if ( index == 0 || index > (1 << 30) )
-		{
-		if ( assigner )
-			assigner->Error(fmt("index (%d) must be positive",
-						index));
-		Unref(element);
-		return true;	// true = "no fatal error"
-		}
-
 	BroType* yt = Type()->AsVectorType()->YieldType();
 
 	if ( yt && yt->Tag() == TYPE_TABLE &&
@@ -3117,7 +2995,7 @@ bool VectorVal::Assign(unsigned int index, Val* element, const Expr* assigner,
 				Val* ival = new Val(index, TYPE_COUNT);
 				StateAccess::Log(new StateAccess(OP_ASSIGN_IDX,
 						this, ival, element,
-						(*val.vector_val)[index - 1]));
+						(*val.vector_val)[index]));
 				Unref(ival);
 				}
 
@@ -3127,10 +3005,10 @@ bool VectorVal::Assign(unsigned int index, Val* element, const Expr* assigner,
 			}
 		}
 
-	if ( index <= val.vector_val->size() )
-		Unref((*val.vector_val)[index - 1]);
+	if ( index < val.vector_val->size() )
+		Unref((*val.vector_val)[index]);
 	else
-		val.vector_val->resize(index);
+		val.vector_val->resize(index + 1);
 
 	if ( LoggingAccess() && op != OP_NONE )
 		{
@@ -3141,26 +3019,26 @@ bool VectorVal::Assign(unsigned int index, Val* element, const Expr* assigner,
 
 		StateAccess::Log(new StateAccess(op == OP_INCR ?
 				OP_INCR_IDX : OP_ASSIGN_IDX,
-				this, ival, element, (*val.vector_val)[index - 1]));
+				this, ival, element, (*val.vector_val)[index]));
 		Unref(ival);
 		}
 
 	// Note: we do *not* Ref() the element, if any, at this point.
 	// AssignExpr::Eval() already does this; other callers must remember
 	// to do it similarly.
-	(*val.vector_val)[index - 1] = element;
+	(*val.vector_val)[index] = element;
 
 	Modified();
 	return true;
 	}
 
 bool VectorVal::AssignRepeat(unsigned int index, unsigned int how_many,
-				Val* element, const Expr* assigner)
+				Val* element)
 	{
-	ResizeAtLeast(index + how_many - 1);
+	ResizeAtLeast(index + how_many);
 
 	for ( unsigned int i = index; i < index + how_many; ++i )
-		if ( ! Assign(i, element, assigner) )
+		if ( ! Assign(i, element ) )
 			return false;
 
 	return true;
@@ -3169,10 +3047,10 @@ bool VectorVal::AssignRepeat(unsigned int index, unsigned int how_many,
 
 Val* VectorVal::Lookup(unsigned int index) const
 	{
-	if ( index == 0 || index > val.vector_val->size() )
+	if ( index >= val.vector_val->size() )
 		return 0;
 
-	return (*val.vector_val)[index - 1];
+	return (*val.vector_val)[index];
 	}
 
 unsigned int VectorVal::Resize(unsigned int new_num_elements)
@@ -3261,7 +3139,7 @@ bool VectorVal::DoUnserialize(UnserialInfo* info)
 		{
 		Val* v;
 		UNSERIALIZE_OPTIONAL(v, Val::Unserialize(info, TYPE_ANY));
-		Assign(i + VECTOR_MIN, v, 0);
+		Assign(i, v);
 		}
 
 	return true;
@@ -3286,9 +3164,33 @@ void VectorVal::ValDescribe(ODesc* d) const
 	d->Add("]");
 	}
 
+OpaqueVal::OpaqueVal(OpaqueType* t) : Val(t)
+	{
+	}
+
+OpaqueVal::~OpaqueVal()
+	{
+	}
+
+IMPLEMENT_SERIAL(OpaqueVal, SER_OPAQUE_VAL);
+
+bool OpaqueVal::DoSerialize(SerialInfo* info) const
+	{
+	DO_SERIALIZE(SER_OPAQUE_VAL, Val);
+	return true;
+	}
+
+bool OpaqueVal::DoUnserialize(UnserialInfo* info)
+	{
+	DO_UNSERIALIZE(Val);
+	return true;
+	}
 
 Val* check_and_promote(Val* v, const BroType* t, int is_init)
 	{
+	if ( ! v )
+		return 0;
+
 	BroType* vt = v->Type();
 
 	vt = flatten_type(vt);
@@ -3297,6 +3199,10 @@ Val* check_and_promote(Val* v, const BroType* t, int is_init)
 	TypeTag t_tag = t->Tag();
 	TypeTag v_tag = vt->Tag();
 
+	// More thought definitely needs to go into this.
+	if ( t_tag == TYPE_ANY || v_tag == TYPE_ANY )
+		return v;
+
 	if ( ! EitherArithmetic(t_tag, v_tag) ||
 	     /* allow sets as initializers */
 	     (is_init && v_tag == TYPE_TABLE) )
@@ -3357,7 +3263,7 @@ Val* check_and_promote(Val* v, const BroType* t, int is_init)
 		break;
 
 	default:
-		internal_error("bad internal type in check_and_promote()");
+		reporter->InternalError("bad internal type in check_and_promote()");
 		Unref(v);
 		return 0;
 	}
@@ -3368,23 +3274,13 @@ Val* check_and_promote(Val* v, const BroType* t, int is_init)
 
 int same_val(const Val* /* v1 */, const Val* /* v2 */)
 	{
-	internal_error("same_val not implemented");
+	reporter->InternalError("same_val not implemented");
 	return 0;
 	}
 
 bool is_atomic_val(const Val* v)
 	{
-	switch ( v->Type()->InternalType() ) {
-	case TYPE_INTERNAL_INT:
-	case TYPE_INTERNAL_UNSIGNED:
-	case TYPE_INTERNAL_DOUBLE:
-	case TYPE_INTERNAL_STRING:
-	case TYPE_INTERNAL_ADDR:
-	case TYPE_INTERNAL_SUBNET:
-		return true;
-	default:
-		return false;
-	}
+	return is_atomic_type(v->Type());
 	}
 
 int same_atomic_val(const Val* v1, const Val* v2)
@@ -3403,23 +3299,13 @@ int same_atomic_val(const Val* v1, const Val* v2)
 		return v1->InternalDouble() == v2->InternalDouble();
 	case TYPE_INTERNAL_STRING:
 		return Bstr_eq(v1->AsString(), v2->AsString());
-
 	case TYPE_INTERNAL_ADDR:
-		{
-		const addr_type& a1 = v1->AsAddr();
-		const addr_type& a2 = v2->AsAddr();
-#ifdef BROv6
-		return addr_eq(a1, a2);
-#else
-		return addr_eq(&a1, &a2);
-#endif
-		}
-
+		return v1->AsAddr() == v2->AsAddr();
 	case TYPE_INTERNAL_SUBNET:
-		return subnet_eq(v1->AsSubNet(), v2->AsSubNet());
+		return v1->AsSubNet() == v2->AsSubNet();
 
 	default:
-		internal_error("same_atomic_val called for non-atomic value");
+		reporter->InternalWarning("same_atomic_val called for non-atomic value");
 		return 0;
 	}
 
@@ -3436,7 +3322,7 @@ void describe_vals(const val_list* vals, ODesc* d, int offset)
 
 	for ( int i = offset; i < vals->length(); ++i )
 		{
-		if ( i > offset && d->IsReadable() )
+		if ( i > offset && d->IsReadable() && d->Style() != RAW_STYLE )
 			d->Add(", ");
 
 		(*vals)[i]->Describe(d);
