@@ -1,5 +1,3 @@
-// $Id: Val.h 6916 2009-09-24 20:48:36Z vern $
-//
 // See the file "COPYING" in the main distribution directory for copyright.
 
 #ifndef val_h
@@ -20,6 +18,7 @@
 #include "ID.h"
 #include "Scope.h"
 #include "StateAccess.h"
+#include "IPAddr.h"
 
 class Val;
 class Func;
@@ -30,7 +29,6 @@ class SerialInfo;
 
 class PortVal;
 class AddrVal;
-class NetVal;
 class SubNetVal;
 
 class IntervalVal;
@@ -39,6 +37,7 @@ class TableVal;
 class RecordVal;
 class ListVal;
 class StringVal;
+class EnumVal;
 class MutableVal;
 
 class StateAccess;
@@ -55,11 +54,11 @@ typedef union {
 	// Used for count, counter, port, subnet.
 	bro_uint_t uint_val;
 
-	// Used for addr, net
-	addr_type addr_val;
+	// Used for addr
+	IPAddr* addr_val;
 
 	// Used for subnet
-	subnet_type subnet_val;
+	IPPrefix* subnet_val;
 
 	// Used for double, time, interval.
 	double double_val;
@@ -87,7 +86,7 @@ public:
 #endif
 		}
 
-	Val(int i, TypeTag t)
+	Val(int32 i, TypeTag t)
 		{
 		val.int_val = bro_int_t(i);
 		type = base_type(t);
@@ -97,17 +96,7 @@ public:
 #endif
 		}
 
-	Val(long i, TypeTag t)
-		{
-		val.int_val = bro_int_t(i);
-		type = base_type(t);
-		attribs = 0;
-#ifdef DEBUG
-		bound_id = 0;
-#endif
-		}
-
-	Val(unsigned int u, TypeTag t)
+	Val(uint32 u, TypeTag t)
 		{
 		val.uint_val = bro_uint_t(u);
 		type = base_type(t);
@@ -117,17 +106,6 @@ public:
 #endif
 		}
 
-	Val(unsigned long u, TypeTag t)
-		{
-		val.uint_val = bro_uint_t(u);
-		type = base_type(t);
-		attribs = 0;
-#ifdef DEBUG
-		bound_id = 0;
-#endif
-		}
-
-#ifdef USE_INT64
 	Val(int64 i, TypeTag t)
 		{
 		val.int_val = i;
@@ -147,7 +125,6 @@ public:
 		bound_id = 0;
 #endif
 		}
-#endif // USE_INT64
 
 	Val(double d, TypeTag t)
 		{
@@ -165,6 +142,15 @@ public:
 	// class has ref'd it.
 	Val(BroFile* f);
 
+	Val(BroType* t, bool type_type) // Extra arg to differentiate from protected version.
+		{
+		type = new TypeType(t->Ref());
+		attribs = 0;
+#ifdef DEBUG
+		bound_id = 0;
+#endif
+		}
+
 	Val()
 		{
 		val.int_val = 0;
@@ -180,13 +166,6 @@ public:
 	Val* Ref()			{ ::Ref(this); return this; }
 	virtual Val* Clone() const;
 
-	RecordVal* GetAttribs(bool instantiate);
-	void SetAttribs(RecordVal* arg_attribs)
-		{
-		Unref((Val*) attribs);
-		attribs = arg_attribs;
-		}
-
 	int IsZero() const;
 	int IsOne() const;
 
@@ -248,18 +227,23 @@ public:
 	CONST_ACCESSOR(TYPE_PATTERN, RE_Matcher*, re_val, AsPattern)
 	CONST_ACCESSOR(TYPE_VECTOR, vector<Val*>*, vector_val, AsVector)
 
-	const subnet_type* AsSubNet() const
+	const IPPrefix& AsSubNet() const
 		{
 		CHECK_TAG(type->Tag(), TYPE_SUBNET, "Val::SubNet", type_name)
-		return &val.subnet_val;
+		return *val.subnet_val;
 		}
 
-	// ... in network byte order
-	const addr_type AsAddr() const
+	BroType* AsType() const
 		{
-		if ( type->Tag() != TYPE_ADDR && type->Tag() != TYPE_NET )
+		CHECK_TAG(type->Tag(), TYPE_TYPE, "Val::Type", type_name)
+		return type;
+		}
+
+	const IPAddr& AsAddr() const
+		{
+		if ( type->Tag() != TYPE_ADDR )
 			BadTag("Val::AsAddr", type_name(type->Tag()));
-		return val.addr_val;
+		return *val.addr_val;
 		}
 
 #define ACCESSOR(tag, ctype, accessor, name) \
@@ -277,10 +261,17 @@ public:
 	ACCESSOR(TYPE_PATTERN, RE_Matcher*, re_val, AsPattern)
 	ACCESSOR(TYPE_VECTOR, vector<Val*>*, vector_val, AsVector)
 
-	subnet_type* AsSubNet()
+	const IPPrefix& AsSubNet()
 		{
 		CHECK_TAG(type->Tag(), TYPE_SUBNET, "Val::SubNet", type_name)
-		return &val.subnet_val;
+		return *val.subnet_val;
+		}
+
+	const IPAddr& AsAddr()
+		{
+		if ( type->Tag() != TYPE_ADDR )
+			BadTag("Val::AsAddr", type_name(type->Tag()));
+		return *val.addr_val;
 		}
 
 	// Gives fast access to the bits of something that is one of
@@ -297,13 +288,14 @@ public:
 
 	CONVERTER(TYPE_PATTERN, PatternVal*, AsPatternVal)
 	CONVERTER(TYPE_PORT, PortVal*, AsPortVal)
-	CONVERTER(TYPE_NET, NetVal*, AsNetVal)
 	CONVERTER(TYPE_SUBNET, SubNetVal*, AsSubNetVal)
+	CONVERTER(TYPE_ADDR, AddrVal*, AsAddrVal)
 	CONVERTER(TYPE_TABLE, TableVal*, AsTableVal)
 	CONVERTER(TYPE_RECORD, RecordVal*, AsRecordVal)
 	CONVERTER(TYPE_LIST, ListVal*, AsListVal)
 	CONVERTER(TYPE_STRING, StringVal*, AsStringVal)
 	CONVERTER(TYPE_VECTOR, VectorVal*, AsVectorVal)
+	CONVERTER(TYPE_ENUM, EnumVal*, AsEnumVal)
 
 #define CONST_CONVERTER(tag, ctype, name) \
 	const ctype name() const \
@@ -314,8 +306,8 @@ public:
 
 	CONST_CONVERTER(TYPE_PATTERN, PatternVal*, AsPatternVal)
 	CONST_CONVERTER(TYPE_PORT, PortVal*, AsPortVal)
-	CONST_CONVERTER(TYPE_NET, NetVal*, AsNetVal)
 	CONST_CONVERTER(TYPE_SUBNET, SubNetVal*, AsSubNetVal)
+	CONST_CONVERTER(TYPE_ADDR, AddrVal*, AsAddrVal)
 	CONST_CONVERTER(TYPE_TABLE, TableVal*, AsTableVal)
 	CONST_CONVERTER(TYPE_RECORD, RecordVal*, AsRecordVal)
 	CONST_CONVERTER(TYPE_LIST, ListVal*, AsListVal)
@@ -342,6 +334,7 @@ public:
 		}
 
 	void Describe(ODesc* d) const;
+	virtual void DescribeReST(ODesc* d) const;
 
 	bool Serialize(SerialInfo* info) const;
 	static Val* Unserialize(UnserialInfo* info, TypeTag type = TYPE_ANY)
@@ -354,13 +347,15 @@ public:
 #ifdef DEBUG
 	// For debugging, we keep a reference to the global ID to which a
 	// value has been bound *last*.
-	ID* GetID() const	{ return bound_id; }
+	ID* GetID() const
+		{
+		return bound_id ? global_scope()->Lookup(bound_id) : 0;
+		}
+
 	void SetID(ID* id)
 		{
-		if ( bound_id )
-			::Unref(bound_id);
-		bound_id = id;
-		::Ref(bound_id);
+		delete [] bound_id;
+		bound_id = id ? copy_string(id->Name()) : 0;
 		}
 #endif
 
@@ -376,6 +371,7 @@ protected:
 		}
 
 	virtual void ValDescribe(ODesc* d) const;
+	virtual void ValDescribeReST(ODesc* d) const;
 
 	Val(TypeTag t)
 		{
@@ -407,8 +403,8 @@ protected:
 	RecordVal* attribs;
 
 #ifdef DEBUG
-	// For debugging, we keep the ID to which a Val is bound.
-	ID* bound_id;
+	// For debugging, we keep the name of the ID to which a Val is bound.
+	const char* bound_id;
 #endif
 
 };
@@ -470,7 +466,7 @@ public:
 protected:
 	MutableVal(BroType* t) : Val(t)
 		{ props = 0; id = 0; last_modified = SerialObj::ALWAYS; }
-	MutableVal()	{ id = 0; last_modified = SerialObj::ALWAYS; }
+	MutableVal()	{ props = 0; id = 0; last_modified = SerialObj::ALWAYS; }
 	~MutableVal();
 
 	friend class ID;
@@ -515,13 +511,9 @@ protected:
 #define NUM_PORT_SPACES 4
 #define PORT_SPACE_MASK 0x30000
 
-#define TCP_PORT_MASK  0x10000
-#define UDP_PORT_MASK  0x20000
-#define ICMP_PORT_MASK 0x30000
-
-typedef enum {
-	TRANSPORT_UNKNOWN, TRANSPORT_TCP, TRANSPORT_UDP, TRANSPORT_ICMP,
-} TransportProto;
+#define TCP_PORT_MASK	0x10000
+#define UDP_PORT_MASK	0x20000
+#define ICMP_PORT_MASK	0x30000
 
 class PortVal : public Val {
 public:
@@ -568,8 +560,9 @@ public:
 	Val* SizeVal() const;
 
 	// Constructor for address already in network order.
-	AddrVal(uint32 addr);
-	AddrVal(const uint32* addr);
+	AddrVal(uint32 addr);          // IPv4.
+	AddrVal(const uint32 addr[4]); // IPv6.
+	AddrVal(const IPAddr& addr);
 
 	unsigned int MemoryAllocation() const;
 
@@ -579,57 +572,33 @@ protected:
 	AddrVal(TypeTag t) : Val(t)	{ }
 	AddrVal(BroType* t) : Val(t)	{ }
 
-	void Init(uint32 addr);
-	void Init(const uint32* addr);
-
 	DECLARE_SERIAL(AddrVal);
 };
 
-class NetVal : public AddrVal {
-public:
-	NetVal(const char* text);
-	NetVal(uint32 addr);
-	NetVal(const uint32* addr);
-
-	Val* SizeVal() const;
-
-protected:
-	friend class Val;
-	NetVal()	{}
-
-	void ValDescribe(ODesc* d) const;
-
-	DECLARE_SERIAL(NetVal);
-};
-
 class SubNetVal : public Val {
 public:
 	SubNetVal(const char* text);
 	SubNetVal(const char* text, int width);
-	SubNetVal(uint32 addr, int width);	// for address already massaged
-	SubNetVal(const uint32* addr, int width);	// ditto
+	SubNetVal(uint32 addr, int width); // IPv4.
+	SubNetVal(const uint32 addr[4], int width); // IPv6.
+	SubNetVal(const IPAddr& addr, int width);
+	SubNetVal(const IPPrefix& prefix);
+	~SubNetVal();
 
 	Val* SizeVal() const;
 
-	int Width() const	{ return val.subnet_val.width; }
-	addr_type Mask() const; // returns host byte order
+	const IPAddr& Prefix() const;
+	int Width() const;
+	IPAddr Mask() const;
 
-	bool Contains(const uint32 addr) const;
-	bool Contains(const uint32* addr) const;
+	bool Contains(const IPAddr& addr) const;
 
-	unsigned int MemoryAllocation() const
-		{
-		return Val::MemoryAllocation() + padded_sizeof(*this) - padded_sizeof(Val);
-		}
+	unsigned int MemoryAllocation() const;
 
 protected:
 	friend class Val;
 	SubNetVal()	{}
 
-	void Init(const char* text, int width);
-	void Init(uint32 addr, int width);
-	void Init(const uint32 *addr, int width);
-
 	void ValDescribe(ODesc* d) const;
 
 	DECLARE_SERIAL(SubNetVal);
@@ -639,6 +608,7 @@ class StringVal : public Val {
 public:
 	StringVal(BroString* s);
 	StringVal(const char* s);
+	StringVal(const string& s);
 	StringVal(int length, const char* s);
 
 	Val* SizeVal() const
@@ -873,6 +843,9 @@ public:
 			timer = 0;
 		}
 
+	HashKey* ComputeHash(const Val* index) const
+		{ return table_hash->ComputeHash(index, 1); }
+
 protected:
 	friend class Val;
 	friend class StateAccess;
@@ -883,8 +856,6 @@ protected:
 	void CheckExpireAttr(attr_tag at);
 	int ExpandCompoundAndInit(val_list* vl, int k, Val* new_val);
 	int CheckAndAssign(Val* index, Val* new_val, Opcode op = OP_ASSIGN);
-	HashKey* ComputeHash(const Val* index) const
-		{ return table_hash->ComputeHash(index, 1); }
 
 	bool AddProperties(Properties arg_state);
 	bool RemoveProperties(Properties arg_state);
@@ -901,7 +872,7 @@ protected:
 
 	DECLARE_SERIAL(TableVal);
 
-	const TableType* table_type;
+	TableType* table_type;
 	CompositeHash* table_hash;
 	Attributes* attrs;
 	double expire_time;
@@ -921,7 +892,19 @@ public:
 		{ return new Val(record_type->NumFields(), TYPE_COUNT); }
 
 	void Assign(int field, Val* new_val, Opcode op = OP_ASSIGN);
-	Val* Lookup(int field) const;
+	Val* Lookup(int field) const;	// Does not Ref() value.
+	Val* LookupWithDefault(int field) const;	// Does Ref() value.
+
+	/**
+	 * Looks up the value of a field by field name.  If the field doesn't
+	 * exist in the record type, it's an internal error: abort.
+	 * @param field name of field to lookup.
+	 * @param with_default whether to rely on field's &default attribute when
+	 * the field has yet to be initialized.
+	 * @return the value in field \a field.  It is Ref()'d only if
+	 * \a with_default is true.
+	 */
+	Val* Lookup(const char* field, bool with_default = false) const;
 
 	void Describe(ODesc* d) const;
 
@@ -930,7 +913,22 @@ public:
 	void SetOrigin(BroObj* o)	{ origin = o; }
 	BroObj* GetOrigin() const	{ return origin; }
 
+	// Returns a new value representing the value coerced to the given
+	// type. If coercion is not possible, returns 0. The non-const
+	// version may return the current value ref'ed if its type matches
+	// directly.
+	//
+	// *aggr* is optional; if non-zero, we add to it. See
+	// Expr::InitVal(). We leave it out in the non-const version to make
+	// the choice unambigious.
+	//
+	// The *allow_orphaning* parameter allows for a record to be demoted
+	// down to a record type that contains less fields.
+	RecordVal* CoerceTo(const RecordType* other, Val* aggr, bool allow_orphaning = false) const;
+	RecordVal* CoerceTo(RecordType* other, bool allow_orphaning = false);
+
 	unsigned int MemoryAllocation() const;
+	void DescribeReST(ODesc* d) const;
 
 protected:
 	friend class Val;
@@ -966,9 +964,6 @@ protected:
 };
 
 
-// The minimum index for vectors (0 or 1).
-const int VECTOR_MIN = 1;
-
 class VectorVal : public MutableVal {
 public:
 	VectorVal(VectorType* t);
@@ -985,18 +980,16 @@ public:
 	// Note: does NOT Ref() the element! Remember to do so unless
 	//       the element was just created and thus has refcount 1.
 	//
-	bool Assign(unsigned int index, Val* element, const Expr* assigner,
-			Opcode op = OP_ASSIGN);
-	bool Assign(Val* index, Val* element, const Expr* assigner,
-			Opcode op = OP_ASSIGN)
+	bool Assign(unsigned int index, Val* element, Opcode op = OP_ASSIGN);
+	bool Assign(Val* index, Val* element, Opcode op = OP_ASSIGN)
 		{
 		return Assign(index->AsListVal()->Index(0)->CoerceToUnsigned(),
-				element, assigner, op);
+				element, op);
 		}
 
 	// Assigns the value to how_many locations starting at index.
 	bool AssignRepeat(unsigned int index, unsigned int how_many,
-			  Val* element, const Expr* assigner);
+			  Val* element);
 
 	// Returns nil if no element was at that value.
 	// Lookup does NOT grow the vector to this size.
@@ -1030,6 +1023,20 @@ protected:
 	VectorType* vector_type;
 };
 
+// Base class for values with types that are managed completely internally,
+// with no further script-level operators provided (other than bif
+// functions). See OpaqueVal.h for derived classes.
+class OpaqueVal : public Val {
+public:
+	OpaqueVal(OpaqueType* t);
+	virtual ~OpaqueVal();
+
+protected:
+	friend class Val;
+	OpaqueVal() { }
+
+	DECLARE_SERIAL(OpaqueVal);
+};
 
 // Checks the given value for consistency with the given type.  If an
 // exact match, returns it.  If promotable, returns the promoted version,
