@@ -1,5 +1,3 @@
-// $Id: main.cc 6829 2009-07-09 09:12:59Z vern $
-//
 // See the file "COPYING" in the main distribution directory for copyright.
 
 #include "config.h"
@@ -8,30 +6,36 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <signal.h>
+#include <string.h>
+#include <list>
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
 #endif
 
+#ifdef USE_CURL
+#include <curl/curl.h>
+#endif
+
 #ifdef USE_IDMEF
 extern "C" {
 #include <libidmef/idmefxml.h>
 }
 #endif
 
-#ifdef USE_OPENSSL
+#include <openssl/md5.h>
+#include <magic.h>
+
 extern "C" void OPENSSL_add_all_algorithms_conf(void);
-#endif
 
 #include "bsd-getopt-long.h"
 #include "input.h"
-#include "Active.h"
 #include "ScriptAnaly.h"
 #include "DNS_Mgr.h"
 #include "Frame.h"
 #include "Scope.h"
 #include "Event.h"
 #include "File.h"
-#include "Logger.h"
+#include "Reporter.h"
 #include "Net.h"
 #include "NetVar.h"
 #include "Var.h"
@@ -46,11 +50,29 @@ extern "C" void OPENSSL_add_all_algorithms_conf(void);
 #include "PersistenceSerializer.h"
 #include "EventRegistry.h"
 #include "Stats.h"
-#include "ConnCompressor.h"
-#include "DPM.h"
+#include "BroDoc.h"
+#include "Brofiler.h"
+
+#include "threading/Manager.h"
+#include "input/Manager.h"
+#include "logging/Manager.h"
+#include "logging/writers/Ascii.h"
+#include "input/readers/Raw.h"
+#include "analyzer/Manager.h"
+#include "analyzer/Tag.h"
+#include "plugin/Manager.h"
+
+#include "file_analysis/Manager.h"
 
 #include "binpac_bro.h"
 
+#include "3rdparty/sqlite3.h"
+
+Brofiler brofiler;
+
+magic_t magic_desc_cookie = 0;
+magic_t magic_mime_cookie = 0;
+
 #ifndef HAVE_STRSEP
 extern "C" {
 char* strsep(char**, const char*);
@@ -61,7 +83,7 @@ extern "C" {
 #include "setsignal.h"
 };
 
-#ifdef USE_PERFTOOLS
+#ifdef USE_PERFTOOLS_DEBUG
 HeapLeakChecker* heap_checker = 0;
 int perftools_leaks = 0;
 int perftools_profile = 0;
@@ -72,10 +94,13 @@ char* writefile = 0;
 name_list prefixes;
 DNS_Mgr* dns_mgr;
 TimerMgr* timer_mgr;
-Logger* bro_logger;
-Func* alarm_hook = 0;
+logging::Manager* log_mgr = 0;
+threading::Manager* thread_mgr = 0;
+input::Manager* input_mgr = 0;
+plugin::Manager* plugin_mgr = 0;
+analyzer::Manager* analyzer_mgr = 0;
+file_analysis::Manager* file_mgr = 0;
 Stmt* stmts;
-EventHandlerPtr bro_signal = 0;
 EventHandlerPtr net_done = 0;
 RuleMatcher* rule_matcher = 0;
 PersistenceSerializer* persistence_serializer = 0;
@@ -88,19 +113,26 @@ ProfileLogger* profiling_logger = 0;
 ProfileLogger* segment_logger = 0;
 SampleLogger* sample_logger = 0;
 int signal_val = 0;
-DPM* dpm = 0;
 int optimize = 0;
 int do_notice_analysis = 0;
 int rule_bench = 0;
-int print_loaded_scripts = 0;
+int generate_documentation = 0;
 SecondaryPath* secondary_path = 0;
-ConnCompressor* conn_compressor = 0;
 extern char version[];
 char* command_line_policy = 0;
 vector<string> params;
 char* proc_status_file = 0;
+int snaplen = 0;	// this gets set from the scripting-layer's value
 
-int FLAGS_use_binpac = false;
+OpaqueType* md5_type = 0;
+OpaqueType* sha1_type = 0;
+OpaqueType* sha256_type = 0;
+OpaqueType* entropy_type = 0;
+OpaqueType* cardinality_type = 0;
+OpaqueType* topk_type = 0;
+OpaqueType* bloomfilter_type = 0;
+
+extern std::list<BroDoc*> docs_generated;
 
 // Keep copy of command line
 int bro_argc;
@@ -124,21 +156,26 @@ const char* bro_version()
 #endif
 	}
 
+const char* bro_dns_fake()
+	{
+	if ( ! getenv("BRO_DNS_FAKE") )
+		return "off";
+	else
+		return "on";
+	}
+
 void usage()
 	{
 	fprintf(stderr, "bro version %s\n", bro_version());
 	fprintf(stderr, "usage: %s [options] [file ...]\n", prog);
 	fprintf(stderr, "    <file>                         | policy file, or read stdin\n");
-#ifdef ACTIVE_MAPPING
-	fprintf(stderr, "    -a|--active-mapping <mapfile>  | use active mapping results\n");
-#endif
+	fprintf(stderr, "    -b|--bare-mode                 | don't load scripts from the base/ directory\n");
 	fprintf(stderr, "    -d|--debug-policy              | activate policy file debugging\n");
 	fprintf(stderr, "    -e|--exec <bro code>           | augment loaded policies by given code\n");
 	fprintf(stderr, "    -f|--filter <filter>           | tcpdump filter\n");
 	fprintf(stderr, "    -g|--dump-config               | dump current config into .state dir\n");
 	fprintf(stderr, "    -h|--help|-?                   | command line help\n");
 	fprintf(stderr, "    -i|--iface <interface>         | read from given interface\n");
-	fprintf(stderr, "    -l|--print-scripts             | print all loaded scripts\n");
 	fprintf(stderr, "    -p|--prefix <prefix>           | add given prefix to policy file resolution\n");
 	fprintf(stderr, "    -r|--readfile <readfile>       | read from given tcpdump file\n");
 	fprintf(stderr, "    -y|--flowfile <file>[=<ident>] | read from given flow file\n");
@@ -149,7 +186,6 @@ void usage()
 	fprintf(stderr, "    -v|--version                   | print version and exit\n");
 	fprintf(stderr, "    -x|--print-state <file.bst>    | print contents of state file\n");
 	fprintf(stderr, "    -z|--analyze <analysis>        | run the specified policy file analysis\n");
-	fprintf(stderr, "    -A|--transfile <writefile>     | write transformed trace to given tcpdump file\n");
 #ifdef DEBUG
 	fprintf(stderr, "    -B|--debug <dbgstreams>        | Enable debugging output for selected streams\n");
 #endif
@@ -159,6 +195,7 @@ void usage()
 	fprintf(stderr, "    -I|--print-id <ID name>        | print out given ID\n");
 	fprintf(stderr, "    -K|--md5-hashkey <hashkey>     | set key for MD5-keyed hashing\n");
 	fprintf(stderr, "    -L|--rule-benchmark            | benchmark for rules\n");
+	fprintf(stderr, "    -N|--print-plugins             | print available plugins and exit (-NN for verbose)\n");
 	fprintf(stderr, "    -O|--optimize                  | optimize policy script\n");
 	fprintf(stderr, "    -P|--prime-dns                 | prime DNS\n");
 	fprintf(stderr, "    -R|--replay <events.bst>       | replay events\n");
@@ -166,8 +203,9 @@ void usage()
 	fprintf(stderr, "    -T|--re-level <level>          | set 'RE_level' for rules\n");
 	fprintf(stderr, "    -U|--status-file <file>        | Record process status in file\n");
 	fprintf(stderr, "    -W|--watchdog                  | activate watchdog timer\n");
+	fprintf(stderr, "    -Z|--doc-scripts               | generate documentation for all loaded scripts\n");
 
-#ifdef USE_PERFTOOLS
+#ifdef USE_PERFTOOLS_DEBUG
 	fprintf(stderr, "    -m|--mem-leaks                 | show leaks  [perftools]\n");
 	fprintf(stderr, "    -M|--mem-profile               | record heap [perftools]\n");
 #endif
@@ -182,14 +220,64 @@ void usage()
 	fprintf(stderr, "    -n|--idmef-dtd <idmef-msg.dtd> | specify path to IDMEF DTD file\n");
 #endif
 
-	fprintf(stderr, "    --use-binpac                   | use new-style BinPAC parsers when available\n");
-
 	fprintf(stderr, "    $BROPATH                       | file search path (%s)\n", bro_path());
-	fprintf(stderr, "    $BRO_PREFIXES                  | prefix list (%s)\n", bro_prefixes());
+	fprintf(stderr, "    $BROMAGIC                      | libmagic mime magic database search path (%s)\n", bro_magic_path());
+	fprintf(stderr, "    $BRO_PREFIXES                  | prefix list (%s)\n", bro_prefixes().c_str());
+	fprintf(stderr, "    $BRO_DNS_FAKE                  | disable DNS lookups (%s)\n", bro_dns_fake());
+	fprintf(stderr, "    $BRO_SEED_FILE                 | file to load seeds from (not set)\n");
+	fprintf(stderr, "    $BRO_LOG_SUFFIX                | ASCII log file extension (.%s)\n", logging::writer::Ascii::LogExt().c_str());
+	fprintf(stderr, "    $BRO_PROFILER_FILE             | Output file for script execution statistics (not set)\n");
+
+	fprintf(stderr, "\n");
+	fprintf(stderr, "    Supported log formats: ");
+
+	bool first = true;
+	list<string> fmts = logging::Manager::SupportedFormats();
+
+	for ( list<string>::const_iterator i = fmts.begin(); i != fmts.end(); ++i )
+		{
+		if ( *i == "None" )
+			// Skip, it's uninteresting.
+			continue;
+
+		if ( ! first )
+			fprintf(stderr, ",");
+
+		fprintf(stderr, "%s", (*i).c_str());
+		first = false;
+		}
+
+	fprintf(stderr, "\n");
 
 	exit(1);
 	}
 
+void show_plugins(int level)
+	{
+	plugin::Manager::plugin_list plugins = plugin_mgr->Plugins();
+
+	if ( ! plugins.size() )
+		{
+		printf("No plugins registered, not even any built-ins. This is probably a bug.\n");
+		return;
+		}
+
+	ODesc d;
+
+	if ( level == 1 )
+		d.SetShort();
+
+	for ( plugin::Manager::plugin_list::const_iterator i = plugins.begin(); i != plugins.end(); i++ )
+		{
+		(*i)->Describe(&d);
+
+		if ( ! d.IsShort() )
+			d.Add("\n");
+		}
+
+	printf("%s", d.Description());
+	}
+
 void done_with_network()
 	{
 	set_processing_status("TERMINATING", "done_with_network");
@@ -219,8 +307,10 @@ void done_with_network()
 
 	terminating = true;
 
-	dpm->Done();
+	analyzer_mgr->Done();
 	timer_mgr->Expire();
+	dns_mgr->Flush();
+	mgr.Drain();
 	mgr.Drain();
 
 	if ( remote_serializer )
@@ -228,7 +318,7 @@ void done_with_network()
 
 	net_finish(1);
 
-#ifdef USE_PERFTOOLS
+#ifdef USE_PERFTOOLS_DEBUG
 
 		if ( perftools_profile )
 			{
@@ -250,6 +340,8 @@ void terminate_bro()
 
 	terminating = true;
 
+	brofiler.WriteStats();
+
 	EventHandlerPtr bro_done = internal_handler("bro_done");
 	if ( bro_done )
 		mgr.QueueEvent(bro_done, new val_list);
@@ -270,8 +362,18 @@ void terminate_bro()
 	if ( remote_serializer )
 		remote_serializer->LogStats();
 
+	mgr.Drain();
+
+	file_mgr->Terminate();
+	log_mgr->Terminate();
+	input_mgr->Terminate();
+	thread_mgr->Terminate();
+
+	mgr.Drain();
+
+	plugin_mgr->FinishPlugins();
+
 	delete timer_mgr;
-	delete bro_logger;
 	delete dns_mgr;
 	delete persistence_serializer;
 	delete event_player;
@@ -279,9 +381,15 @@ void terminate_bro()
 	delete state_serializer;
 	delete event_registry;
 	delete secondary_path;
-	delete conn_compressor;
 	delete remote_serializer;
-	delete dpm;
+	delete analyzer_mgr;
+	delete log_mgr;
+	delete plugin_mgr;
+	delete thread_mgr;
+	delete file_mgr;
+	delete reporter;
+
+	reporter = 0;
 	}
 
 void termination_signal()
@@ -289,7 +397,7 @@ void termination_signal()
 	set_processing_status("TERMINATING", "termination_signal");
 
 	Val sval(signal_val, TYPE_COUNT);
-	message("received termination signal");
+	reporter->Info("received termination signal");
 	net_get_final_stats();
 	done_with_network();
 	net_delete();
@@ -309,6 +417,7 @@ RETSIGTYPE sig_handler(int signo)
 	{
 	set_processing_status("TERMINATING", "sig_handler");
 	signal_val = signo;
+
 	return RETSIGVAL;
 	}
 
@@ -324,6 +433,10 @@ static void bro_new_handler()
 
 int main(int argc, char** argv)
 	{
+	std::set_new_handler(bro_new_handler);
+
+	brofiler.ReadStats();
+
 	bro_argc = argc;
 	bro_argv = new char* [argc];
 
@@ -335,12 +448,14 @@ int main(int argc, char** argv)
 	name_list netflows;
 	name_list flow_files;
 	name_list rule_files;
-	char* transformed_writefile = 0;
 	char* bst_file = 0;
 	char* id_name = 0;
 	char* events_file = 0;
-	char* seed_load_file = 0;
+	char* seed_load_file = getenv("BRO_SEED_FILE");
 	char* seed_save_file = 0;
+	char* user_pcap_filter = 0;
+	char* debug_streams = 0;
+	int bare_mode = false;
 	int seed = 0;
 	int dump_cfg = false;
 	int to_xml = 0;
@@ -348,15 +463,17 @@ int main(int argc, char** argv)
 	int override_ignore_checksums = 0;
 	int rule_debug = 0;
 	int RE_level = 4;
+	int print_plugins = 0;
 
 	static struct option long_opts[] = {
+		{"bare-mode",	no_argument,		0,	'b'},
 		{"debug-policy",	no_argument,		0,	'd'},
 		{"dump-config",		no_argument,		0,	'g'},
 		{"exec",		required_argument,	0,	'e'},
 		{"filter",		required_argument,	0,	'f'},
 		{"help",		no_argument,		0,	'h'},
 		{"iface",		required_argument,	0,	'i'},
-		{"print-scripts",	no_argument,		0,	'l'},
+		{"doc-scripts",		no_argument,		0,	'Z'},
 		{"prefix",		required_argument,	0,	'p'},
 		{"readfile",		required_argument,	0,	'r'},
 		{"flowfile",		required_argument,	0,	'y'},
@@ -367,7 +484,6 @@ int main(int argc, char** argv)
 		{"version",		no_argument,		0,	'v'},
 		{"print-state",		required_argument,	0,	'x'},
 		{"analyze",		required_argument,	0,	'z'},
-		{"transfile",		required_argument,	0,	'A'},
 		{"no-checksums",	no_argument,		0,	'C'},
 		{"dfa-cache",		required_argument,	0,	'D'},
 		{"force-dns",		no_argument,		0,	'F'},
@@ -376,6 +492,7 @@ int main(int argc, char** argv)
 		{"set-seed",		required_argument,	0,	'J'},
 		{"md5-hashkey",		required_argument,	0,	'K'},
 		{"rule-benchmark",	no_argument,		0,	'L'},
+		{"print-plugins",	no_argument,		0,	'N'},
 		{"optimize",		no_argument,		0,	'O'},
 		{"prime-dns",		no_argument,		0,	'P'},
 		{"replay",		required_argument,	0,	'R'},
@@ -385,40 +502,31 @@ int main(int argc, char** argv)
 		{"print-id",		required_argument,	0,	'I'},
 		{"status-file",		required_argument,	0,	'U'},
 
-#ifdef	ACTIVE_MAPPING
-		{"active-mapping",	no_argument,		0,	'a'},
-#endif
 #ifdef	DEBUG
 		{"debug",		required_argument,	0,	'B'},
 #endif
 #ifdef	USE_IDMEF
 		{"idmef-dtd",		required_argument,	0,	'n'},
 #endif
-#ifdef	USE_PERFTOOLS
+#ifdef	USE_PERFTOOLS_DEBUG
 		{"mem-leaks",	no_argument,		0,	'm'},
 		{"mem-profile",	no_argument,		0,	'M'},
 #endif
 
 		{"pseudo-realtime",	optional_argument, 0,	'E'},
 
-		{"use-binpac",		no_argument, 		&FLAGS_use_binpac, 1},
-
 		{0,			0,			0,	0},
 	};
 
 	enum DNS_MgrMode dns_type = DNS_DEFAULT;
 
-#ifdef HAVE_NB_DNS
 	dns_type = getenv("BRO_DNS_FAKE") ? DNS_FAKE : DNS_DEFAULT;
-#else
-	dns_type = DNS_FAKE;
-#endif
 
 	RETSIGTYPE (*oldhandler)(int);
 
 	prog = argv[0];
 
-	prefixes.append("");	// "" = "no prefix"
+	prefixes.append(strdup(""));	// "" = "no prefix"
 
 	char* p = getenv("BRO_PREFIXES");
 	if ( p )
@@ -437,24 +545,18 @@ int main(int argc, char** argv)
 	opterr = 0;
 
 	char opts[256];
-	safe_strncpy(opts, "A:a:B:D:e:f:I:i:K:n:p:R:r:s:T:t:U:w:x:X:y:Y:z:CFGHLOPSWdghlv",
+	safe_strncpy(opts, "B:D:e:f:I:i:K:l:n:p:R:r:s:T:t:U:w:x:X:y:Y:z:CFGLNOPSWbdghvZ",
 		     sizeof(opts));
 
-#ifdef USE_PERFTOOLS
+#ifdef USE_PERFTOOLS_DEBUG
 	strncat(opts, "mM", 2);
 #endif
 
 	int op;
 	while ( (op = getopt_long(argc, argv, opts, long_opts, &long_optsind)) != EOF )
 		switch ( op ) {
-		case 'a':
-#ifdef ACTIVE_MAPPING
-			fprintf(stderr, "Using active mapping file %s.\n", optarg);
-			active_file = optarg;
-#else
-			fprintf(stderr, "Bro not compiled for active mapping.\n");
-			exit(1);
-#endif
+		case 'b':
+			bare_mode = true;
 			break;
 
 		case 'd':
@@ -478,10 +580,6 @@ int main(int argc, char** argv)
 			interfaces.append(optarg);
 			break;
 
-		case 'l':
-			print_loaded_scripts = 1;
-			break;
-
 		case 'p':
 			prefixes.append(optarg);
 			break;
@@ -517,10 +615,6 @@ int main(int argc, char** argv)
 				}
 			break;
 
-		case 'A':
-			transformed_writefile = optarg;
-			break;
-
 		case 'C':
 			override_ignore_checksums = 1;
 			break;
@@ -558,8 +652,7 @@ int main(int argc, char** argv)
 			break;
 
 		case 'K':
-			hash_md5(strlen(optarg), (const u_char*) optarg,
-				 shared_hmac_md5_key);
+			MD5((const u_char*) optarg, strlen(optarg), shared_hmac_md5_key);
 			hmac_key_set = 1;
 			break;
 
@@ -567,6 +660,10 @@ int main(int argc, char** argv)
 			++rule_bench;
 			break;
 
+		case 'N':
+			++print_plugins;
+			break;
+
 		case 'O':
 			optimize = 1;
 			break;
@@ -610,7 +707,7 @@ int main(int argc, char** argv)
 			exit(0);
 			break;
 
-#ifdef USE_PERFTOOLS
+#ifdef USE_PERFTOOLS_DEBUG
 		case 'm':
 			perftools_leaks = 1;
 			break;
@@ -630,6 +727,10 @@ int main(int argc, char** argv)
 			break;
 #endif
 
+		case 'Z':
+			generate_documentation = 1;
+			break;
+
 #ifdef USE_IDMEF
 		case 'n':
 			fprintf(stderr, "Using IDMEF XML DTD from %s\n", optarg);
@@ -638,9 +739,7 @@ int main(int argc, char** argv)
 #endif
 
 		case 'B':
-#ifdef DEBUG
-			debug_logger.EnableStreams(optarg);
-#endif
+			debug_streams = optarg;
 			break;
 
 		case 0:
@@ -659,22 +758,40 @@ int main(int argc, char** argv)
 
 	bro_start_time = current_time(true);
 
-	init_random_seed(seed, seed_load_file, seed_save_file);
+	reporter = new Reporter();
+	thread_mgr = new threading::Manager();
+
+#ifdef DEBUG
+	if ( debug_streams )
+		debug_logger.EnableStreams(debug_streams);
+#endif
+
+	init_random_seed(seed, (seed_load_file && *seed_load_file ? seed_load_file : 0) , seed_save_file);
 	// DEBUG_MSG("HMAC key: %s\n", md5_digest_print(shared_hmac_md5_key));
 	init_hash_function();
 
-#ifdef USE_OPENSSL
 	ERR_load_crypto_strings();
 	OPENSSL_add_all_algorithms_conf();
 	SSL_library_init();
 	SSL_load_error_strings();
 
+#ifdef USE_CURL
+	curl_global_init(CURL_GLOBAL_ALL);
+#endif
+
+	bro_init_magic(&magic_desc_cookie, MAGIC_NONE);
+	bro_init_magic(&magic_mime_cookie, MAGIC_MIME);
+
+	int r = sqlite3_initialize();
+
+	if ( r != SQLITE_OK )
+		reporter->Error("Failed to initialize sqlite3: %s", sqlite3_errstr(r));
+
 	// FIXME: On systems that don't provide /dev/urandom, OpenSSL doesn't
 	// seed the PRNG. We should do this here (but at least Linux, FreeBSD
 	// and Solaris provide /dev/urandom).
-#endif
 
-	if ( (interfaces.length() > 0 || netflows.length() > 0) && 
+	if ( (interfaces.length() > 0 || netflows.length() > 0) &&
 	     (read_files.length() > 0 || flow_files.length() > 0 ))
 		usage();
 
@@ -689,11 +806,14 @@ int main(int argc, char** argv)
 	timer_mgr = new PQ_TimerMgr("<GLOBAL>");
 	// timer_mgr = new CQ_TimerMgr();
 
-	add_input_file("bro.init");
+	add_input_file("base/init-bare.bro");
+	if ( ! bare_mode )
+		add_input_file("base/init-default.bro");
 
 	if ( optind == argc &&
 	     read_files.length() == 0 && flow_files.length() == 0 &&
-	     ! (id_name || bst_file) && ! command_line_policy )
+	     interfaces.length() == 0 &&
+	     ! (id_name || bst_file) && ! command_line_policy && ! print_plugins )
 		add_input_file("-");
 
 	// Process remaining arguments.  X=Y arguments indicate script
@@ -707,12 +827,7 @@ int main(int argc, char** argv)
 			add_input_file(argv[optind++]);
 		}
 
-	if ( ! load_mapping_table(active_file.c_str()) )
-		{
-		fprintf(stderr, "Could not load active mapping file %s\n",
-			active_file.c_str());
-		exit(1);
-		}
+	push_scope(0);
 
 	dns_mgr = new DNS_Mgr(dns_type);
 
@@ -723,17 +838,31 @@ int main(int argc, char** argv)
 
 	persistence_serializer = new PersistenceSerializer();
 	remote_serializer = new RemoteSerializer();
-	event_registry = new EventRegistry;
+	event_registry = new EventRegistry();
+	analyzer_mgr = new analyzer::Manager();
+	log_mgr = new logging::Manager();
+	input_mgr = new input::Manager();
+	plugin_mgr = new plugin::Manager();
+	file_mgr = new file_analysis::Manager();
+
+	plugin_mgr->InitPreScript();
+	analyzer_mgr->InitPreScript();
+	file_mgr->InitPreScript();
 
 	if ( events_file )
 		event_player = new EventPlayer(events_file);
 
 	init_event_handlers();
 
-	push_scope(0);
+	input::reader::Raw::ClassInit();
 
-	dpm = new DPM;
-	dpm->PreScriptInit();
+	md5_type = new OpaqueType("md5");
+	sha1_type = new OpaqueType("sha1");
+	sha256_type = new OpaqueType("sha256");
+	entropy_type = new OpaqueType("entropy");
+	cardinality_type = new OpaqueType("cardinality");
+	topk_type = new OpaqueType("topk");
+	bloomfilter_type = new OpaqueType("bloomfilter");
 
 	// The leak-checker tends to produce some false
 	// positives (memory which had already been
@@ -741,25 +870,64 @@ int main(int argc, char** argv)
 	// nevertheless reported; see perftools docs), thus
 	// we suppress some messages here.
 
-#ifdef USE_PERFTOOLS
+#ifdef USE_PERFTOOLS_DEBUG
 	{
 	HeapLeakChecker::Disabler disabler;
 #endif
 
 	yyparse();
 
-#ifdef USE_PERFTOOLS
+	plugin_mgr->InitPostScript();
+	analyzer_mgr->InitPostScript();
+	file_mgr->InitPostScript();
+
+	if ( print_plugins )
+		{
+		show_plugins(print_plugins);
+		exit(1);
+		}
+
+#ifdef USE_PERFTOOLS_DEBUG
 	}
 #endif
 
-	if ( nerr > 0 )
+	if ( generate_documentation )
+		{
+		CreateProtoAnalyzerDoc("proto-analyzers.rst");
+		CreateFileAnalyzerDoc("file-analyzers.rst");
+
+		std::list<BroDoc*>::iterator it;
+
+		for ( it = docs_generated.begin(); it != docs_generated.end(); ++it )
+			(*it)->WriteDocFile();
+
+		for ( it = docs_generated.begin(); it != docs_generated.end(); ++it )
+			delete *it;
+
+		terminate_bro();
+		return 0;
+		}
+
+	if ( reporter->Errors() > 0 )
 		{
 		delete dns_mgr;
 		exit(1);
 		}
 
+	reporter->InitOptions();
+
 	init_general_global_var();
 
+	if ( user_pcap_filter )
+		{
+		ID* id = global_scope()->Lookup("cmd_line_bpf_filter");
+
+		if ( ! id )
+			reporter->InternalError("global cmd_line_bpf_filter not defined");
+
+		id->SetVal(new StringVal(user_pcap_filter));
+		}
+
 	// Parse rule files defined on the script level.
 	char* script_rule_files =
 		copy_string(internal_val("signature_files")->AsString()->CheckString());
@@ -770,6 +938,10 @@ int main(int argc, char** argv)
 		if ( *s )
 			rule_files.append(s);
 
+	// Append signature files defined in @load-sigs
+	for ( size_t i = 0; i < sig_files.size(); ++i )
+		rule_files.append(copy_string(sig_files[i].c_str()));
+
 	if ( rule_files.length() > 0 )
 		{
 		rule_matcher = new RuleMatcher(RE_level);
@@ -785,18 +957,11 @@ int main(int argc, char** argv)
 
 	delete [] script_rule_files;
 
-	conn_compressor = new ConnCompressor();
-
 	if ( g_policy_debug )
 		// ### Add support for debug command file.
 		dbg_init_debugger(0);
 
-	Val* bro_alarm_file = internal_val("bro_alarm_file");
-	bro_logger = new Logger("bro",
-			bro_alarm_file ?
-				bro_alarm_file->AsFile() : new BroFile(stderr));
-
-	if ( (flow_files.length() == 0 || read_files.length() == 0) && 
+	if ( (flow_files.length() == 0 || read_files.length() == 0) &&
 	     (netflows.length() == 0 || interfaces.length() == 0) )
 		{
 		Val* interfaces_val = internal_val("interfaces");
@@ -812,25 +977,18 @@ int main(int argc, char** argv)
 			}
 		}
 
+	snaplen = internal_val("snaplen")->AsCount();
+
 	// Initialize the secondary path, if it's needed.
 	secondary_path = new SecondaryPath();
 
 	if ( dns_type != DNS_PRIME )
 		net_init(interfaces, read_files, netflows, flow_files,
-			writefile, transformed_writefile,
-			user_pcap_filter ? user_pcap_filter : "tcp or udp",
+			writefile, "",
 			secondary_path->Filter(), do_watchdog);
 
-	if ( ! reading_traces )
-		// Only enable actual syslog'ing for live input.
-		bro_logger->SetEnabled(enable_syslog);
-	else
-		bro_logger->SetEnabled(0);
-
 	BroFile::SetDefaultRotation(log_rotate_interval, log_max_size);
 
-	alarm_hook = internal_func("alarm_hook");
-	bro_signal = internal_handler("bro_signal");
 	net_done = internal_handler("net_done");
 
 	if ( ! g_policy_debug )
@@ -850,10 +1008,7 @@ int main(int argc, char** argv)
 		dns_mgr->Resolve();
 
 		if ( ! dns_mgr->Save() )
-			{
-			bro_logger->Log("**Can't update DNS cache");
-			exit(1);
-			}
+			reporter->FatalError("can't update DNS cache");
 
 		mgr.Drain();
 		delete dns_mgr;
@@ -877,15 +1032,13 @@ int main(int argc, char** argv)
 			UnserialInfo info(&s);
 			info.print = stdout;
 			info.install_uniques = true;
-			s.Read(&info, bst_file);
+			if ( ! s.Read(&info, bst_file) )
+				reporter->Error("Failed to read events from %s\n", bst_file);
 			}
 
 		exit(0);
 		}
 
-	if ( using_communication )
-		remote_serializer->Init();
-
 	persistence_serializer->SetDir((const char *)state_dir->AsString()->CheckString());
 
 	// Print the ID.
@@ -895,10 +1048,7 @@ int main(int argc, char** argv)
 
 		ID* id = global_scope()->Lookup(id_name);
 		if ( ! id )
-			{
-			fprintf(stderr, "No such ID: %s\n", id_name);
-			exit(1);
-			}
+			reporter->FatalError("No such ID: %s\n", id_name);
 
 		ODesc desc;
 		desc.SetQuotes(true);
@@ -940,9 +1090,8 @@ int main(int argc, char** argv)
 
 	if ( dead_handlers->length() > 0 && check_for_unused_event_handlers )
 		{
-		warn("event handlers never invoked:");
 		for ( int i = 0; i < dead_handlers->length(); ++i )
-			warn("\t", (*dead_handlers)[i]);
+			reporter->Warning("event handler never invoked: %s", (*dead_handlers)[i]);
 		}
 
 	delete dead_handlers;
@@ -952,9 +1101,9 @@ int main(int argc, char** argv)
 
 	if ( alive_handlers->length() > 0 && dump_used_event_handlers )
 		{
-		message("invoked event handlers:");
+		reporter->Info("invoked event handlers:");
 		for ( int i = 0; i < alive_handlers->length(); ++i )
-			message((*alive_handlers)[i]);
+			reporter->Info("%s", (*alive_handlers)[i]);
 		}
 
 	delete alive_handlers;
@@ -974,18 +1123,37 @@ int main(int argc, char** argv)
 	if ( override_ignore_checksums )
 		ignore_checksums = 1;
 
-	dpm->PostScriptInit();
+	// Queue events reporting loaded scripts.
+	for ( std::list<ScannedFile>::iterator i = files_scanned.begin(); i != files_scanned.end(); i++ )
+		{
+		if ( i->skipped )
+			continue;
 
+		val_list* vl = new val_list;
+		vl->append(new StringVal(i->name.c_str()));
+		vl->append(new Val(i->include_level, TYPE_COUNT));
+		mgr.QueueEvent(bro_script_loaded, vl);
+		}
+
+	reporter->ReportViaEvents(true);
+
+	// Drain the event queue here to support the protocols framework configuring DPM
 	mgr.Drain();
 
+	analyzer_mgr->DumpDebug();
+
 	have_pending_timers = ! reading_traces && timer_mgr->Size() > 0;
 
-	if ( io_sources.Size() > 0 || have_pending_timers )
+	io_sources.Register(thread_mgr, true);
+
+	if ( io_sources.Size() > 0 ||
+	     have_pending_timers ||
+	     BifConst::exit_only_after_terminate )
 		{
 		if ( profiling_logger )
 			profiling_logger->Log();
 
-#ifdef USE_PERFTOOLS
+#ifdef USE_PERFTOOLS_DEBUG
 		if ( perftools_leaks )
 			heap_checker = new HeapLeakChecker("net_run");
 
@@ -996,12 +1164,19 @@ int main(int argc, char** argv)
 			}
 
 #endif
+
 		net_run();
 		done_with_network();
 		net_delete();
 
+#ifdef USE_CURL
+		curl_global_cleanup();
+#endif
+
 		terminate_bro();
 
+		sqlite3_shutdown();
+
 		// Close files after net_delete(), because net_delete()
 		// might write to connection content files.
 		BroFile::CloseCachedFiles();
